@@ -1,15 +1,16 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 # Copyright Daniel Wallin 2006. Use, modification and distribution is
 # subject to the Boost Software License, Version 1.0. (See accompanying
 # file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
+
 import sys
 import atexit
 import libtorrent as lt
 import time
 import os.path
-import sys
+
 
 class WindowsConsole:
     def __init__(self):
@@ -27,6 +28,7 @@ class WindowsConsole:
             return msvcrt.getch()
         return None
 
+
 class UnixConsole:
     def __init__(self):
         self.fd = sys.stdin
@@ -51,11 +53,13 @@ class UnixConsole:
         sys.stdout.flush()
 
     def sleep_and_input(self, seconds):
-        read,_,_ = select.select([self.fd.fileno()], [], [], seconds)
+        read, __, __ = select.select(
+            [self.fd.fileno()], [], [], seconds)
         if len(read) > 0:
             return self.fd.read(1)
         return None
 
+
 if os.name == 'nt':
     import Console
     import msvcrt
@@ -63,9 +67,11 @@ else:
     import termios
     import select
 
+
 def write_line(console, line):
     console.write(line)
 
+
 def add_suffix(val):
     prefix = ['B', 'kB', 'MB', 'GB', 'TB']
     for i in range(len(prefix)):
@@ -78,14 +84,17 @@ def add_suffix(val):
 
     return '%6.3gPB' % val
 
+
 def progress_bar(progress, width):
     assert(progress <= 1)
     progress_chars = int(progress * width + 0.5)
     return progress_chars * '#' + (width - progress_chars) * '-'
 
+
 def print_peer_info(console, peers):
 
-    out = ' down    (total )   up     (total )  q  r flags  block progress  client\n'
+    out = (' down    (total )   up     (total )'
+           '  q  r flags  block progress  client\n')
 
     for p in peers:
 
@@ -106,7 +115,8 @@ def print_peer_info(console, peers):
 
         if p.downloading_piece_index >= 0:
             assert(p.downloading_progress <= p.downloading_total)
-            out += progress_bar(float(p.downloading_progress) / p.downloading_total, 15)
+            out += progress_bar(float(p.downloading_progress) /
+                                p.downloading_total, 15)
         else:
             out += progress_bar(0, 15)
         out += ' '
@@ -114,7 +124,7 @@ def print_peer_info(console, peers):
         if p.flags & lt.peer_info.handshake:
             id = 'waiting for handshake'
         elif p.flags & lt.peer_info.connecting:
-            id =  'connecting to peer'
+            id = 'connecting to peer'
         else:
             id = p.client
 
@@ -128,7 +138,7 @@ def print_download_queue(console, download_queue):
     out = ""
 
     for e in download_queue:
-        out += '%4d: [' % e['piece_index'];
+        out += '%4d: [' % e['piece_index']
         for b in e['blocks']:
             s = b['state']
             if s == 3:
@@ -143,50 +153,65 @@ def print_download_queue(console, download_queue):
 
     write_line(console, out)
 
+
 def add_torrent(ses, filename, options):
-    atp = {}
+    atp = lt.add_torrent_params()
     if filename.startswith('magnet:'):
-        atp = lt.parse_magnet_uri_dict(filename)
+        atp = lt.parse_magnet_uri(filename)
     else:
-        atp['ti'] = lt.torrent_info(filename)
+        atp.ti = lt.torrent_info(filename)
         try:
-            atp["resume_data"] = open(os.path.join(options.save_path, info.name() + '.fastresume'), 'rb').read()
-        except:
+            atp.resume_data = open(os.path.join(options.save_path, atp.info.name() + '.fastresume'), 'rb').read()
+        except Exception:
             pass
 
-    atp["save_path"] = options.save_path
-    atp["storage_mode"] = lt.storage_mode_t.storage_mode_sparse
-    atp["paused"] = False
-    atp["auto_managed"] = True
-    atp["duplicate_is_error"] = True
+    atp.save_path = options.save_path
+    atp.storage_mode = lt.storage_mode_t.storage_mode_sparse
+    atp.flags |= lt.torrent_flags.duplicate_is_error \
+        | lt.torrent_flags.auto_managed \
+        | lt.torrent_flags.duplicate_is_error
     ses.async_add_torrent(atp)
 
+
 def main():
     from optparse import OptionParser
 
     parser = OptionParser()
 
-    parser.add_option('-p', '--port',
-        type='int', help='set listening port')
+    parser.add_option('-p', '--port', type='int', help='set listening port')
 
-    parser.add_option('-d', '--max-download-rate',
-        type='float', help='the maximum download rate given in kB/s. 0 means infinite.')
+    parser.add_option(
+        '-i', '--listen-interface', type='string',
+        help='set interface for incoming connections', )
 
-    parser.add_option('-u', '--max-upload-rate',
-        type='float', help='the maximum upload rate given in kB/s. 0 means infinite.')
+    parser.add_option(
+        '-o', '--outgoing-interface', type='string',
+        help='set interface for outgoing connections')
 
-    parser.add_option('-s', '--save-path',
-        type='string', help='the path where the downloaded file/folder should be placed.')
+    parser.add_option(
+        '-d', '--max-download-rate', type='float',
+        help='the maximum download rate given in kB/s. 0 means infinite.')
 
-    parser.add_option('-r', '--proxy-host',
-        type='string', help='sets HTTP proxy host and port (separated by \':\')')
+    parser.add_option(
+        '-u', '--max-upload-rate', type='float',
+        help='the maximum upload rate given in kB/s. 0 means infinite.')
+
+    parser.add_option(
+        '-s', '--save-path', type='string',
+        help='the path where the downloaded file/folder should be placed.')
+
+    parser.add_option(
+        '-r', '--proxy-host', type='string',
+        help='sets HTTP proxy host and port (separated by \':\')')
 
     parser.set_defaults(
-        port=6881
-      , max_download_rate=0
-      , max_upload_rate=0
-      , save_path='.'
-      , proxy_host=''
+        port=6881,
+        listen_interface='0.0.0.0',
+        outgoing_interface='',
+        max_download_rate=0,
+        max_upload_rate=0,
+        save_path='.',
+        proxy_host=''
     )
 
     (options, args) = parser.parse_args()
@@ -202,11 +227,13 @@ def main():
     if options.max_download_rate <= 0:
         options.max_download_rate = -1
 
-    settings = { 'user_agent': 'python_client/' + lt.__version__,
+    settings = {
+        'user_agent': 'python_client/' + lt.__version__,
+        'listen_interfaces': '%s:%d' % (options.listen_interface, options.port),
         'download_rate_limit': int(options.max_download_rate),
         'upload_rate_limit': int(options.max_upload_rate),
-        'listen_interfaces': '0.0.0.0:%d' % options.port,
-        'alert_mask': lt.alert.category_t.all_categories
+        'alert_mask': lt.alert.category_t.all_categories,
+        'outgoing_interfaces': options.outgoing_interface,
     }
 
     if options.proxy_host != '':
@@ -234,16 +261,16 @@ def main():
 
         out = ''
 
-        for h,t in torrents.items():
+        for h, t in torrents.items():
             out += 'name: %-40s\n' % t.name[:40]
 
             if t.state != lt.torrent_status.seeding:
-                state_str = ['queued', 'checking', 'downloading metadata', \
-                             'downloading', 'finished', 'seeding', \
+                state_str = ['queued', 'checking', 'downloading metadata',
+                             'downloading', 'finished', 'seeding',
                              'allocating', 'checking fastresume']
                 out += state_str[t.state] + ' '
 
-                out += '%5.4f%% ' % (t.progress*100)
+                out += '%5.4f%% ' % (t.progress * 100)
                 out += progress_bar(t.progress, 49)
                 out += '\n'
 
@@ -270,13 +297,13 @@ def main():
             if t.state != lt.torrent_status.seeding:
                 try:
                     out = '\n'
-                    fp = t.handle.file_progress()
+                    fp = h.file_progress()
                     ti = t.torrent_file
-                    for f,p in zip(ti.files(), fp):
+                    for f, p in zip(ti.files(), fp):
                         out += progress_bar(p / float(f.size), 20)
                         out += ' ' + f.path + '\n'
                     write_line(console, out)
-                except:
+                except Exception:
                     pass
 
         write_line(console, 76 * '-' + '\n')
@@ -288,7 +315,7 @@ def main():
             alerts_log.append(a.message())
 
             # add new torrents to our list of torrent_status
-            if type(a) == lt.add_torrent_alert:
+            if isinstance(a, lt.add_torrent_alert):
                 h = a.handle
                 h.set_max_connections(60)
                 h.set_max_uploads(-1)
@@ -296,12 +323,12 @@ def main():
 
             # update our torrent_status array for torrents that have
             # changed some of their state
-            if type(a) == lt.state_update_alert:
+            if isinstance(a, lt.state_update_alert):
                 for s in a.status:
                     torrents[s.handle] = s
 
-        if len(alerts_log) > 8:
-            del alerts_log[:len(alerts_log) - 8]
+        if len(alerts_log) > 20:
+            alerts_log = alerts_log[-20:]
 
         for a in alerts_log:
             write_line(console, a + '\n')
@@ -309,19 +336,23 @@ def main():
         c = console.sleep_and_input(0.5)
 
         ses.post_torrent_updates()
-        if not c: continue
+        if not c:
+            continue
 
         if c == 'r':
-            for h in torrents.keys(): h.force_reannounce()
+            for h in torrents:
+                h.force_reannounce()
         elif c == 'q':
             alive = False
         elif c == 'p':
-            for h in torrents.keys(): h.pause()
+            for h in torrents:
+                h.pause()
         elif c == 'u':
-            for h in torrents.keys(): h.resume()
+            for h in torrents:
+                h.resume()
 
     ses.pause()
-    for h,t in torrents.items():
+    for h, t in torrents.items():
         if not h.is_valid() or not t.has_metadata:
             continue
         h.save_resume_data()
@@ -329,19 +360,20 @@ def main():
     while len(torrents) > 0:
         alerts = ses.pop_alerts()
         for a in alerts:
-            if type(a) == lt.save_resume_data_alert:
-                data = lt.bencode(a.resume_data)
+            if isinstance(a, lt.save_resume_data_alert):
+                print(a)
+                data = lt.write_resume_data_buf(a.params)
                 h = a.handle
                 if h in torrents:
                     open(os.path.join(options.save_path, torrents[h].name + '.fastresume'), 'wb').write(data)
                     del torrents[h]
 
-            if type(a) == lt.save_resume_data_failed_alert:
+            if isinstance(a, lt.save_resume_data_failed_alert):
                 h = a.handle
                 if h in torrents:
                     print('failed to save resume data for ', torrents[h].name)
                     del torrents[h]
         time.sleep(0.5)
 
-main()
 
+main()
