@@ -7,7 +7,7 @@
 #include "bytes.hpp"
 
 using namespace boost::python;
-using namespace libtorrent;
+using namespace lt;
 
 struct entry_to_python
 {
@@ -20,7 +20,7 @@ struct entry_to_python
             result.append(*i);
         }
 
-        return result;
+        return std::move(result);
     }
 
     static object convert(entry::dictionary_type const& d)
@@ -30,7 +30,7 @@ struct entry_to_python
         for (entry::dictionary_type::const_iterator i(d.begin()), e(d.end()); i != e; ++i)
             result[bytes(i->first)] = i->second;
 
-        return result;
+        return std::move(result);
     }
 
     static object convert0(entry const& e)
@@ -59,7 +59,7 @@ struct entry_to_python
         }
     }
 
-    static PyObject* convert(boost::shared_ptr<entry> const& e)
+    static PyObject* convert(std::shared_ptr<entry> const& e)
     {
         if (!e)
             return incref(Py_None);
@@ -172,7 +172,7 @@ struct entry_from_python
 
 void bind_entry()
 {
-    to_python_converter<boost::shared_ptr<libtorrent::entry>, entry_to_python>();
+    to_python_converter<std::shared_ptr<lt::entry>, entry_to_python>();
     to_python_converter<entry, entry_to_python>();
     entry_from_python();
 }
