@@ -2,12 +2,12 @@
 // subject to the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
-#include <libtorrent/ip_filter.hpp>
 #include "boost_python.hpp"
+#include <libtorrent/ip_filter.hpp>
 #include "gil.hpp"
 
 using namespace boost::python;
-using namespace libtorrent;
+using namespace lt;
 
 namespace
 {
