@@ -2,23 +2,25 @@
 // subject to the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
-#include "libtorrent/aux_/disable_warnings_push.hpp"
+// vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4
 
 #include "boost_python.hpp"
-#include <boost/shared_ptr.hpp>
-
-#include "libtorrent/aux_/disable_warnings_pop.hpp"
+#include <memory>
 
 #include "libtorrent/torrent_info.hpp"
-#include "libtorrent/session_settings.hpp"
 #include "libtorrent/time.hpp"
 #include "libtorrent/socket_io.hpp"
 #include "libtorrent/announce_entry.hpp"
 #include "bytes.hpp"
 
+#ifdef _MSC_VER
+#pragma warning(push)
+// warning C4996: X: was declared deprecated
+#pragma warning( disable : 4996 )
+#endif
+
 using namespace boost::python;
-using namespace libtorrent;
-namespace lt = libtorrent;
+using namespace lt;
 
 namespace
 {
@@ -42,12 +44,8 @@ namespace
     {
         list result;
 
-        typedef std::vector<std::pair<std::string, int> > list_type;
-
-        for (list_type::const_iterator i = ti.nodes().begin(); i != ti.nodes().end(); ++i)
-        {
-            result.append(boost::python::make_tuple(i->first, i->second));
-        }
+        for (auto const& i : ti.nodes())
+            result.append(boost::python::make_tuple(i.first, i.second));
 
         return result;
     }
@@ -72,7 +70,7 @@ namespace
     void set_web_seeds(torrent_info& ti, list ws)
     {
         std::vector<web_seed_entry> web_seeds;
-        int const len = boost::python::len(ws);
+        int const len = static_cast<int>(boost::python::len(ws));
         for (int i = 0; i < len; i++)
         {
            dict e = extract<dict>(ws[i]);
@@ -100,13 +98,13 @@ namespace
     void set_merkle_tree(torrent_info& ti, list hashes)
     {
         std::vector<sha1_hash> h;
-        for (int i = 0, e = len(hashes); i < e; ++i)
-            h.push_back(sha1_hash(bytes(extract<bytes>(hashes[i])).arr));
+        for (int i = 0, e = int(len(hashes)); i < e; ++i)
+            h.push_back(sha1_hash(bytes(extract<bytes>(hashes[i])).arr.data()));
 
         ti.set_merkle_tree(h);
     }
 
-    bytes hash_for_piece(torrent_info const& ti, int i)
+    bytes hash_for_piece(torrent_info const& ti, piece_index_t i)
     {
         return bytes(ti.hash_for_piece(i).to_string());
     }
@@ -116,7 +114,7 @@ namespace
         return bytes(ti.metadata().get(), ti.metadata_size());
     }
 
-    list map_block(torrent_info& ti, int piece, boost::int64_t offset, int size)
+    list map_block(torrent_info& ti, piece_index_t piece, std::int64_t offset, int size)
     {
        std::vector<file_slice> p = ti.map_block(piece, offset, size);
        list result;
@@ -127,112 +125,160 @@ namespace
        return result;
     }
 
+#if TORRENT_ABI_VERSION == 1
     // Create getters for announce_entry data members with non-trivial types which need converting.
-    lt::time_point get_next_announce(announce_entry const& ae) { return ae.next_announce; }
-    lt::time_point get_min_announce(announce_entry const& ae) { return ae.min_announce; }
+    lt::time_point get_next_announce(announce_entry const& ae)
+    { return ae.endpoints.empty() ? lt::time_point() : lt::time_point(ae.endpoints.front().next_announce); }
+    lt::time_point get_min_announce(announce_entry const& ae)
+    { return ae.endpoints.empty() ? lt::time_point() : lt::time_point(ae.endpoints.front().min_announce); }
     // announce_entry data member bit-fields.
-    int get_fails(announce_entry const& ae) { return ae.fails; }
-    int get_source(announce_entry const& ae) { return ae.source; }
-    bool get_verified(announce_entry const& ae) { return ae.verified; }
-    bool get_updating(announce_entry const& ae) { return ae.updating; }
-    bool get_start_sent(announce_entry const& ae) { return ae.start_sent; }
-    bool get_complete_sent(announce_entry const& ae) { return ae.complete_sent; }
-    bool get_send_stats(announce_entry const& ae) { return ae.send_stats; }
+    int get_fails(announce_entry const& ae)
+    { return ae.endpoints.empty() ? 0 : ae.endpoints.front().fails; }
+    bool get_updating(announce_entry const& ae)
+    { return ae.endpoints.empty() ? false : ae.endpoints.front().updating; }
+    bool get_start_sent(announce_entry const& ae)
+    { return ae.endpoints.empty() ? false : ae.endpoints.front().start_sent; }
+    bool get_complete_sent(announce_entry const& ae)
+    { return ae.endpoints.empty() ? false : ae.endpoints.front().complete_sent; }
     // announce_entry method requires lt::time_point.
     bool can_announce(announce_entry const& ae, bool is_seed) {
+        // an entry without endpoints implies it has never been announced so it can be now
+        if (ae.endpoints.empty()) return true;
         lt::time_point now = lt::clock_type::now();
-        return ae.can_announce(now, is_seed);
+        return ae.endpoints.front().can_announce(now, is_seed, ae.fail_limit);
     }
+    bool is_working(announce_entry const& ae)
+    { return ae.endpoints.empty() ? false : ae.endpoints.front().is_working(); }
+#endif
+    int get_source(announce_entry const& ae) { return ae.source; }
+    bool get_verified(announce_entry const& ae) { return ae.verified; }
 
-#ifndef TORRENT_NO_DEPRECATE
-    boost::int64_t get_size(file_entry const& fe) { return fe.size; }
-    boost::int64_t get_offset(file_entry const& fe) { return fe.offset; }
-    boost::int64_t get_file_base(file_entry const& fe) { return fe.file_base; }
-    void set_file_base(file_entry& fe, int b) { fe.file_base = b; }
+#if TORRENT_ABI_VERSION == 1
+    std::string get_message(announce_entry const& ae)
+    { return ae.endpoints.empty() ? "" : ae.endpoints.front().message; }
+    error_code get_last_error(announce_entry const& ae)
+    { return ae.endpoints.empty() ? error_code() : ae.endpoints.front().last_error; }
+    int get_scrape_incomplete(announce_entry const& ae)
+    { return ae.endpoints.empty() ? 0 : ae.endpoints.front().scrape_incomplete; }
+    int get_scrape_complete(announce_entry const& ae)
+    { return ae.endpoints.empty() ? 0 : ae.endpoints.front().scrape_complete; }
+    int get_scrape_downloaded(announce_entry const& ae)
+    { return ae.endpoints.empty() ? 0 : ae.endpoints.front().scrape_downloaded; }
+    int next_announce_in(announce_entry const&) { return 0; }
+    int min_announce_in(announce_entry const&) { return 0; }
+    bool get_send_stats(announce_entry const& ae) { return ae.send_stats; }
+    std::int64_t get_size(file_entry const& fe) { return fe.size; }
+    std::int64_t get_offset(file_entry const& fe) { return fe.offset; }
     bool get_pad_file(file_entry const& fe) { return fe.pad_file; }
     bool get_executable_attribute(file_entry const& fe) { return fe.executable_attribute; }
     bool get_hidden_attribute(file_entry const& fe) { return fe.hidden_attribute; }
     bool get_symlink_attribute(file_entry const& fe) { return fe.symlink_attribute; }
 #endif
 
+load_torrent_limits dict_to_limits(dict limits)
+{
+    load_torrent_limits ret;
+
+    list items = limits.items();
+    int const len = int(boost::python::len(limits));
+    for (int i = 0; i < len; i++)
+    {
+        boost::python::api::object_item item = items[i];
+        std::string const key = extract<std::string>(item[0]);
+        object const value = item[1];
+
+        if (key == "max_buffer_size")
+        {
+            ret.max_buffer_size = extract<int>(value);
+            continue;
+        }
+        else if (key == "max_pieces")
+        {
+            ret.max_pieces = extract<int>(value);
+            continue;
+        }
+        else if (key == "max_decode_depth")
+        {
+            ret.max_decode_depth = extract<int>(value);
+            continue;
+        }
+        else if (key == "max_decode_tokens")
+        {
+            ret.max_decode_tokens = extract<int>(value);
+            continue;
+        }
+    }
+    return ret;
+}
+
 } // namespace unnamed
 
-boost::shared_ptr<torrent_info> buffer_constructor0(char const* buf, int len, int flags)
+std::shared_ptr<torrent_info> buffer_constructor0(bytes b)
 {
-   error_code ec;
-   boost::shared_ptr<torrent_info> ret(boost::make_shared<torrent_info>(buf
-		, len, boost::ref(ec), flags));
-#ifndef BOOST_NO_EXCEPTIONS
-   if (ec) throw libtorrent_exception(ec);
-#endif
+   return std::make_shared<torrent_info>(b.arr, from_span);
+}
+
+std::shared_ptr<torrent_info> buffer_constructor1(bytes b, dict limits)
+{
+   std::shared_ptr<torrent_info> ret = std::make_shared<torrent_info>(b.arr
+        , dict_to_limits(limits), from_span);
    return ret;
 }
 
-boost::shared_ptr<torrent_info> buffer_constructor1(char const* buf, int len)
+std::shared_ptr<torrent_info> file_constructor0(std::string const& filename)
 {
-	return buffer_constructor0(buf, len, 0);
+   return std::make_shared<torrent_info>(filename);
 }
 
-boost::shared_ptr<torrent_info> file_constructor0(std::string const& filename, int flags)
+std::shared_ptr<torrent_info> file_constructor1(std::string const& filename, dict limits)
 {
-   error_code ec;
-   boost::shared_ptr<torrent_info> ret(boost::make_shared<torrent_info>(filename
-		, boost::ref(ec), flags));
-#ifndef BOOST_NO_EXCEPTIONS
-   if (ec) throw libtorrent_exception(ec);
-#endif
-   return ret;
+   return std::make_shared<torrent_info>(filename, dict_to_limits(limits));
 }
 
-boost::shared_ptr<torrent_info> file_constructor1(std::string const& filename)
+std::shared_ptr<torrent_info> bencoded_constructor0(entry const& ent)
 {
-	return file_constructor0(filename, 0);
+    std::vector<char> buf;
+    bencode(std::back_inserter(buf), ent);
+    return std::make_shared<torrent_info>(buf, lt::from_span);
 }
 
-boost::shared_ptr<torrent_info> bencoded_constructor0(entry const& ent, int flags)
+std::shared_ptr<torrent_info> bencoded_constructor1(entry const& ent, dict limits)
 {
-	std::vector<char> buf;
-	bencode(std::back_inserter(buf), ent);
-
-	bdecode_node e;
-	error_code ec;
-	if (buf.size() == 0 || bdecode(&buf[0], &buf[0] + buf.size(), e, ec) != 0)
-	{
-#ifndef BOOST_NO_EXCEPTIONS
-		throw invalid_torrent_file(ec);
-#endif
-	}
-
-	boost::shared_ptr<torrent_info> ret(boost::make_shared<torrent_info>(e
-			, boost::ref(ec), flags));
-#ifndef BOOST_NO_EXCEPTIONS
-	if (ec) throw libtorrent_exception(ec);
-#endif
-	return ret;
-}
-
-boost::shared_ptr<torrent_info> bencoded_constructor1(entry const& ent)
-{
-	return bencoded_constructor0(ent, 0);
+    std::vector<char> buf;
+    bencode(std::back_inserter(buf), ent);
+    return std::make_shared<torrent_info>(buf, dict_to_limits(limits)
+        , lt::from_span);
 }
 
+using by_value = return_value_policy<return_by_value>;
 void bind_torrent_info()
 {
     return_value_policy<copy_const_reference> copy;
 
-    void (torrent_info::*rename_file0)(int, std::string const&) = &torrent_info::rename_file;
-#if TORRENT_USE_WSTRING && !defined TORRENT_NO_DEPRECATE
-    void (torrent_info::*rename_file1)(int, std::wstring const&) = &torrent_info::rename_file;
+    void (torrent_info::*rename_file0)(file_index_t, std::string const&) = &torrent_info::rename_file;
+#if TORRENT_ABI_VERSION == 1
+#ifdef TORRENT_WINDOWS
+    void (torrent_info::*rename_file1)(file_index_t, std::wstring const&) = &torrent_info::rename_file;
+#endif
 #endif
 
     class_<file_slice>("file_slice")
-        .def_readwrite("file_index", &file_slice::file_index)
+        .add_property("file_index", make_getter((&file_slice::file_index), by_value()))
         .def_readwrite("offset", &file_slice::offset)
         .def_readwrite("size", &file_slice::size)
         ;
 
-    class_<torrent_info, boost::shared_ptr<torrent_info> >("torrent_info", no_init)
-        .def(init<sha1_hash const&, int>((arg("info_hash"), arg("flags") = 0)))
+    enum_<announce_entry::tracker_source>("tracker_source")
+        .value("source_torrent", announce_entry::source_torrent)
+        .value("source_client", announce_entry::source_client)
+        .value("source_magnet_link", announce_entry::source_magnet_link)
+        .value("source_tex", announce_entry::source_tex)
+    ;
+
+    using add_tracker1 = void (torrent_info::*)(std::string const&, int, announce_entry::tracker_source);
+
+    class_<torrent_info, std::shared_ptr<torrent_info>>("torrent_info", no_init)
+        .def(init<sha1_hash const&>(arg("info_hash")))
         .def("__init__", make_constructor(&bencoded_constructor0))
         .def("__init__", make_constructor(&bencoded_constructor1))
         .def("__init__", make_constructor(&buffer_constructor0))
@@ -241,11 +287,13 @@ void bind_torrent_info()
         .def("__init__", make_constructor(&file_constructor1))
         .def(init<torrent_info const&>((arg("ti"))))
 
-#if TORRENT_USE_WSTRING && !defined TORRENT_NO_DEPRECATE
-        .def(init<std::wstring, int>((arg("file"), arg("flags") = 0)))
+#if TORRENT_ABI_VERSION == 1
+#ifdef TORRENT_WINDOWS
+        .def(init<std::wstring>((arg("file"))))
+#endif
 #endif
 
-        .def("add_tracker", &torrent_info::add_tracker, arg("url"))
+        .def("add_tracker", (add_tracker1)&torrent_info::add_tracker, arg("url"), arg("tier") = 0, arg("source") = announce_entry::source_client)
         .def("add_url_seed", &torrent_info::add_url_seed)
         .def("add_http_seed", &torrent_info::add_http_seed)
         .def("web_seeds", get_web_seeds)
@@ -271,13 +319,13 @@ void bind_torrent_info()
         .def("remap_files", &torrent_info::remap_files)
         .def("files", &torrent_info::files, return_internal_reference<>())
         .def("orig_files", &torrent_info::orig_files, return_internal_reference<>())
-#ifndef TORRENT_NO_DEPRECATE
+#if TORRENT_ABI_VERSION == 1
         .def("file_at", &torrent_info::file_at)
         .def("file_at_offset", &torrent_info::file_at_offset)
-#if TORRENT_USE_WSTRING
+#ifdef TORRENT_WINDOWS
         .def("rename_file", rename_file1)
-#endif // TORRENT_USE_WSTRING
-#endif // TORRENT_NO_DEPRECATE
+#endif
+#endif // TORRENT_ABI_VERSION
 
         .def("is_valid", &torrent_info::is_valid)
         .def("priv", &torrent_info::priv)
@@ -295,7 +343,7 @@ void bind_torrent_info()
         .def("map_file", &torrent_info::map_file)
         ;
 
-#ifndef TORRENT_NO_DEPRECATE
+#if TORRENT_ABI_VERSION == 1
     class_<file_entry>("file_entry")
         .def_readwrite("path", &file_entry::path)
         .def_readwrite("symlink_path", &file_entry::symlink_path)
@@ -307,48 +355,44 @@ void bind_torrent_info()
         .add_property("symlink_attribute", &get_symlink_attribute)
         .add_property("offset", &get_offset)
         .add_property("size", &get_size)
-        .add_property("file_base", &get_file_base, &set_file_base)
         ;
 #endif
 
     class_<announce_entry>("announce_entry", init<std::string const&>())
         .def_readwrite("url", &announce_entry::url)
         .def_readonly("trackerid", &announce_entry::trackerid)
-        .def_readonly("message", &announce_entry::message)
-        .def_readonly("last_error", &announce_entry::last_error)
+#if TORRENT_ABI_VERSION == 1
+        .add_property("message", &get_message)
+        .add_property("last_error", &get_last_error)
         .add_property("next_announce", &get_next_announce)
         .add_property("min_announce", &get_min_announce)
-        .def_readonly("scrape_incomplete", &announce_entry::scrape_incomplete)
-        .def_readonly("scrape_complete", &announce_entry::scrape_complete)
-        .def_readonly("scrape_downloaded", &announce_entry::scrape_downloaded)
+        .add_property("scrape_incomplete", &get_scrape_incomplete)
+        .add_property("scrape_complete", &get_scrape_complete)
+        .add_property("scrape_downloaded", &get_scrape_downloaded)
+#endif
         .def_readwrite("tier", &announce_entry::tier)
         .def_readwrite("fail_limit", &announce_entry::fail_limit)
-        .add_property("fails", &get_fails)
         .add_property("source", &get_source)
         .add_property("verified", &get_verified)
+#if TORRENT_ABI_VERSION == 1
+        .add_property("fails", &get_fails)
         .add_property("updating", &get_updating)
         .add_property("start_sent", &get_start_sent)
         .add_property("complete_sent", &get_complete_sent)
         .add_property("send_stats", &get_send_stats)
-
-        .def("next_announce_in", &announce_entry::next_announce_in)
-        .def("min_announce_in", &announce_entry::min_announce_in)
+        .def("next_announce_in", &next_announce_in)
+        .def("min_announce_in", &min_announce_in)
+        .def("can_announce", &can_announce)
+        .def("is_working", &is_working)
+#endif
         .def("reset", &announce_entry::reset)
-        .def("can_announce", can_announce)
-        .def("is_working", &announce_entry::is_working)
         .def("trim", &announce_entry::trim)
         ;
 
-    enum_<announce_entry::tracker_source>("tracker_source")
-        .value("source_torrent", announce_entry::source_torrent)
-        .value("source_client", announce_entry::source_client)
-        .value("source_magnet_link", announce_entry::source_magnet_link)
-        .value("source_tex", announce_entry::source_tex)
-    ;
-
-#if BOOST_VERSION > 104200
-    implicitly_convertible<boost::shared_ptr<torrent_info>, boost::shared_ptr<const torrent_info> >();
-    boost::python::register_ptr_to_python<boost::shared_ptr<const torrent_info> >();
-#endif
+    implicitly_convertible<std::shared_ptr<torrent_info>, std::shared_ptr<const torrent_info>>();
+    boost::python::register_ptr_to_python<std::shared_ptr<const torrent_info>>();
 }
 
+#ifdef _MSC_VER
+#pragma warning(pop)
+#endif
