@@ -4,6 +4,9 @@
 #include "libtorrent/ed25519.hpp"
 #include "fe.h"
 
+namespace libtorrent
+{
+
 void ed25519_key_exchange(unsigned char *shared_secret
 	, const unsigned char *public_key, const unsigned char *private_key) {
     unsigned char e[32];
@@ -81,3 +84,5 @@ void ed25519_key_exchange(unsigned char *shared_secret
     fe_mul(x2, x2, z2);
     fe_tobytes(shared_secret, x2);
 }
+
+}
