@@ -2,10 +2,13 @@
 #include "libtorrent/aux_/disable_warnings_push.hpp"
 
 #include "libtorrent/ed25519.hpp"
-#include "sha512.h"
+#include "libtorrent/hasher512.hpp"
 #include "ge.h"
 #include "sc.h"
 
+namespace libtorrent
+{
+
 static int consttime_equal(const unsigned char *x, const unsigned char *y) {
     unsigned char r = 0;
 
@@ -47,10 +50,8 @@ static int consttime_equal(const unsigned char *x, const unsigned char *y) {
     return !r;
 }
 
-int ed25519_verify(const unsigned char *signature, const unsigned char *message, size_t message_len, const unsigned char *public_key) {
-    unsigned char h[64];
+int ed25519_verify(const unsigned char *signature, const unsigned char *message, std::ptrdiff_t message_len, const unsigned char *public_key) {
     unsigned char checker[32];
-    sha512_context hash;
     ge_p3 A;
     ge_p2 R;
 
@@ -62,14 +63,15 @@ int ed25519_verify(const unsigned char *signature, const unsigned char *message,
         return 0;
     }
 
-    sha512_init(&hash);
-    sha512_update(&hash, signature, 32);
-    sha512_update(&hash, public_key, 32);
-    sha512_update(&hash, message, message_len);
-    sha512_final(&hash, h);
+    hasher512 hash;
+    hash.update({reinterpret_cast<char const*>(signature), 32});
+    hash.update({reinterpret_cast<char const*>(public_key), 32});
+    hash.update({reinterpret_cast<char const*>(message), message_len});
+    sha512_hash h = hash.final();
     
-    sc_reduce(h);
-    ge_double_scalarmult_vartime(&R, h, &A, signature + 32);
+    sc_reduce(reinterpret_cast<unsigned char*>(h.data()));
+    ge_double_scalarmult_vartime(&R, reinterpret_cast<unsigned char*>(h.data())
+        , &A, signature + 32);
     ge_tobytes(checker, &R);
 
     if (!consttime_equal(checker, signature)) {
@@ -78,3 +80,5 @@ int ed25519_verify(const unsigned char *signature, const unsigned char *message,
 
     return 1;
 }
+
+}
