@@ -41,8 +41,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <libtorrent/alert_types.hpp>
 #include <libtorrent/bencode.hpp>
 #include <libtorrent/torrent_status.hpp>
-#include <libtorrent/magnet_uri.hpp>
+#include <libtorrent/read_resume_data.hpp>
+#include <libtorrent/write_resume_data.hpp>
 #include <libtorrent/error_code.hpp>
+#include <libtorrent/magnet_uri.hpp>
 
 using clk = std::chrono::steady_clock;
 
@@ -61,7 +63,7 @@ char const* state(lt::torrent_status::state_t s)
 	}
 }
 
-int main(int argc, char const* argv[])
+int main(int argc, char const* argv[]) try
 {
 	if (argc != 2) {
 		std::cerr << "usage: " << argv[0] << " <magnet-url>" << std::endl;
@@ -70,27 +72,26 @@ int main(int argc, char const* argv[])
 
 	lt::settings_pack pack;
 	pack.set_int(lt::settings_pack::alert_mask
-		, lt::alert::error_notification
-		| lt::alert::storage_notification
-		| lt::alert::status_notification);
+		, lt::alert_category::error
+		| lt::alert_category::storage
+		| lt::alert_category::status);
 
 	lt::session ses(pack);
-	lt::add_torrent_params atp;
 	clk::time_point last_save_resume = clk::now();
 
 	// load resume data from disk and pass it in as we add the magnet link
 	std::ifstream ifs(".resume_file", std::ios_base::binary);
 	ifs.unsetf(std::ios_base::skipws);
-	atp.resume_data.assign(std::istream_iterator<char>(ifs)
-		, std::istream_iterator<char>());
-	lt::error_code ec;
-	lt::parse_magnet_uri(argv[1], atp, ec);
-	if (ec) {
-		std::cerr << "invalid magnet URI: " << ec.message() << std::endl;
-		return 1;
+	std::vector<char> buf{std::istream_iterator<char>(ifs)
+		, std::istream_iterator<char>()};
+
+	lt::add_torrent_params atp = lt::read_resume_data(buf);
+	lt::add_torrent_params magnet = lt::parse_magnet_uri(argv[1]);
+	if (atp.info_hash != magnet.info_hash) {
+		atp = std::move(magnet);
 	}
 	atp.save_path = "."; // save in current dir
-	ses.async_add_torrent(atp);
+	ses.async_add_torrent(std::move(atp));
 
 	// this is the handle we'll set once we get the notification of it being
 	// added
@@ -117,8 +118,8 @@ int main(int argc, char const* argv[])
 			if (auto rd = lt::alert_cast<lt::save_resume_data_alert>(a)) {
 				std::ofstream of(".resume_file", std::ios_base::binary);
 				of.unsetf(std::ios_base::skipws);
-				lt::bencode(std::ostream_iterator<char>(of)
-					, *rd->resume_data);
+				auto const b = write_resume_data_buf(rd->params);
+				of.write(b.data(), b.size());
 			}
 
 			if (auto st = lt::alert_cast<lt::state_update_alert>(a)) {
@@ -152,4 +153,8 @@ int main(int argc, char const* argv[])
 done:
 	std::cout << "\ndone, shutting down" << std::endl;
 }
+catch (std::exception& e)
+{
+	std::cerr << "Error: " << e.what() << std::endl;
+}
 
