@@ -195,7 +195,6 @@ bool print_ip = true;
 bool print_local_ip = false;
 bool print_timers = false;
 bool print_block = false;
-bool print_peer_rate = false;
 bool print_fails = false;
 bool print_send_bufs = true;
 bool print_disk_stats = false;
@@ -322,7 +321,6 @@ int print_peer_info(std::string& out
 	if (print_timers) out += "inactive wait timeout q-time ";
 	out += "  v disk ^    rtt  ";
 	if (print_block) out += "block-progress ";
-	if (print_peer_rate) out += "est.rec.rate ";
 	out += "client \x1b[K\n";
 	++pos;
 
@@ -453,14 +451,6 @@ int print_peer_info(std::string& out
 			}
 		}
 
-		if (print_peer_rate)
-		{
-			bool const unchoked = !(i->flags & lt::peer_info::choked);
-
-			std::snprintf(str, sizeof(str), " %s"
-				, unchoked ? add_suffix(i->estimated_reciprocation_rate, "/s").c_str() : "      ");
-			out += str;
-		}
 		out += " ";
 
 		if (i->flags & lt::peer_info::handshake)
@@ -523,6 +513,123 @@ void print_settings(int const start, int const num
 	}
 }
 
+void assign_setting(lt::settings_pack& settings, std::string const& key, char const* value)
+{
+	int const sett_name = lt::setting_by_name(key);
+	if (sett_name < 0)
+	{
+		std::fprintf(stderr, "unknown setting: \"%s\"\n", key.c_str());
+		std::exit(1);
+	}
+
+	using lt::settings_pack;
+
+	switch (sett_name & settings_pack::type_mask)
+	{
+		case settings_pack::string_type_base:
+			settings.set_str(sett_name, value);
+			break;
+		case settings_pack::bool_type_base:
+			if (value == "1"_sv || value == "on"_sv || value == "true"_sv)
+			{
+				settings.set_bool(sett_name, true);
+			}
+			else if (value == "0"_sv || value == "off"_sv || value == "false"_sv)
+			{
+				settings.set_bool(sett_name, false);
+			}
+			else
+			{
+				std::fprintf(stderr, "invalid value for \"%s\". expected 0 or 1\n"
+					, key.c_str());
+				std::exit(1);
+			}
+			break;
+		case settings_pack::int_type_base:
+			using namespace libtorrent::literals;
+			static std::map<lt::string_view, int> const enums = {
+				{"no_piece_suggestions"_sv, settings_pack::no_piece_suggestions},
+				{"suggest_read_cache"_sv, settings_pack::suggest_read_cache},
+				{"fixed_slots_choker"_sv, settings_pack::fixed_slots_choker},
+				{"rate_based_choker"_sv, settings_pack::rate_based_choker},
+				{"round_robin"_sv, settings_pack::round_robin},
+				{"fastest_upload"_sv, settings_pack::fastest_upload},
+				{"anti_leech"_sv, settings_pack::anti_leech},
+				{"enable_os_cache"_sv, settings_pack::enable_os_cache},
+				{"disable_os_cache"_sv, settings_pack::disable_os_cache},
+				{"prefer_tcp"_sv, settings_pack::prefer_tcp},
+				{"peer_proportional"_sv, settings_pack::peer_proportional},
+				{"pe_forced"_sv, settings_pack::pe_forced},
+				{"pe_enabled"_sv, settings_pack::pe_enabled},
+				{"pe_disabled"_sv, settings_pack::pe_disabled},
+				{"pe_plaintext"_sv, settings_pack::pe_plaintext},
+				{"pe_rc4"_sv, settings_pack::pe_rc4},
+				{"pe_both"_sv, settings_pack::pe_both},
+				{"none"_sv, settings_pack::none},
+				{"socks4"_sv, settings_pack::socks4},
+				{"socks5"_sv, settings_pack::socks5},
+				{"socks5_pw"_sv, settings_pack::socks5_pw},
+				{"http"_sv, settings_pack::http},
+				{"http_pw"_sv, settings_pack::http_pw},
+				{"i2p_proxy"_sv, settings_pack::i2p_proxy},
+			};
+
+			auto const it = enums.find(lt::string_view(value));
+			if (it != enums.end())
+			{
+				settings.set_int(sett_name, it->second);
+				break;
+			}
+
+			static std::map<lt::string_view, lt::alert_category_t> const alert_categories = {
+				{"error"_sv, lt::alert_category::error},
+				{"peer"_sv, lt::alert_category::peer},
+				{"port_mapping"_sv, lt::alert_category::port_mapping},
+				{"storage"_sv, lt::alert_category::storage},
+				{"tracker"_sv, lt::alert_category::tracker},
+				{"connect"_sv, lt::alert_category::connect},
+				{"status"_sv, lt::alert_category::status},
+				{"ip_block"_sv, lt::alert_category::ip_block},
+				{"performance_warning"_sv, lt::alert_category::performance_warning},
+				{"dht"_sv, lt::alert_category::dht},
+				{"session_log"_sv, lt::alert_category::session_log},
+				{"torrent_log"_sv, lt::alert_category::torrent_log},
+				{"peer_log"_sv, lt::alert_category::peer_log},
+				{"incoming_request"_sv, lt::alert_category::incoming_request},
+				{"dht_log"_sv, lt::alert_category::dht_log},
+				{"dht_operation"_sv, lt::alert_category::dht_operation},
+				{"port_mapping_log"_sv, lt::alert_category::port_mapping_log},
+				{"picker_log"_sv, lt::alert_category::picker_log},
+				{"file_progress"_sv, lt::alert_category::file_progress},
+				{"piece_progress"_sv, lt::alert_category::piece_progress},
+				{"upload"_sv, lt::alert_category::upload},
+				{"block_progress"_sv, lt::alert_category::block_progress},
+				{"all"_sv, lt::alert_category::all},
+			};
+
+			std::stringstream flags(value);
+			std::string f;
+			lt::alert_category_t val;
+			while (std::getline(flags, f, ',')) try
+			{
+				auto const it = alert_categories.find(f);
+				if (it == alert_categories.end())
+					val |= lt::alert_category_t{unsigned(std::stoi(f))};
+				else
+					val |= it->second;
+			}
+			catch (std::invalid_argument const&)
+			{
+				std::fprintf(stderr, "invalid value for \"%s\". expected integer or enum value\n"
+					, key.c_str());
+				std::exit(1);
+			}
+
+			settings.set_int(sett_name, val);
+			break;
+	}
+}
+
 std::string resume_file(lt::sha1_hash const& info_hash)
 {
 	return path_append(save_path, path_append(".resume"
@@ -696,11 +803,11 @@ void print_alert(lt::alert const* a, std::string& str)
 {
 	using namespace lt;
 
-	if (a->category() & alert::error_notification)
+	if (a->category() & alert_category::error)
 	{
 		str += esc("31");
 	}
-	else if (a->category() & (alert::peer_notification | alert::storage_notification))
+	else if (a->category() & (alert_category::peer | alert_category::storage))
 	{
 		str += esc("33");
 	}
@@ -863,10 +970,8 @@ bool handle_alert(torrent_view& view, session_view& ses_view
 	else if (save_resume_data_alert* p = alert_cast<save_resume_data_alert>(a))
 	{
 		--num_outstanding_resume_data;
-		torrent_handle h = p->handle;
 		auto const buf = write_resume_data_buf(p->params);
-		torrent_status st = h.status(torrent_handle::query_save_path);
-		save_file(resume_file(st.info_hash), buf);
+		save_file(resume_file(p->params.info_hash), buf);
 	}
 	else if (save_resume_data_failed_alert* p = alert_cast<save_resume_data_failed_alert>(a))
 	{
@@ -1065,12 +1170,17 @@ DISK OPTIONS
 TORRENT is a path to a .torrent file
 MAGNETURL is a magnet link
 
-example alert_masks:
-   dht | errors                   =  1025
-   peer-log | errors              = 32769
-   torrent-log | errors           = 16385
-   ses-log | errors               =  8193
-   ses-log | torrent-log | errors = 24578
+alert mask flags:
+	error peer port_mapping storage tracker connect status ip_block
+	performance_warning dht session_log torrent_log peer_log incoming_request
+	dht_log dht_operation port_mapping_log picker_log file_progress piece_progress
+	upload block_progress all
+
+examples:
+  --alert_mask=error,port_mapping,tracker,connect,session_log
+  --alert_mask=error,session_log,torrent_log,peer_log
+  --alert_mask=error,dht,dht_log,dht_operation
+  --alert_mask=all
 )") ;
 		return 0;
 	}
@@ -1101,20 +1211,20 @@ example alert_masks:
 
 	settings.set_str(settings_pack::user_agent, "client_test/" LIBTORRENT_VERSION);
 	settings.set_int(settings_pack::alert_mask
-		, alert::error_notification
-		| alert::peer_notification
-		| alert::port_mapping_notification
-		| alert::storage_notification
-		| alert::tracker_notification
-		| alert::connect_notification
-		| alert::status_notification
-		| alert::ip_block_notification
-		| alert::performance_warning
-		| alert::dht_notification
-		| alert::incoming_request_notification
-		| alert::dht_operation_notification
-		| alert::port_mapping_log_notification
-		| alert::file_progress_notification);
+		, lt::alert_category::error
+		| lt::alert_category::peer
+		| lt::alert_category::port_mapping
+		| lt::alert_category::storage
+		| lt::alert_category::tracker
+		| lt::alert_category::connect
+		| lt::alert_category::status
+		| lt::alert_category::ip_block
+		| lt::alert_category::performance_warning
+		| lt::alert_category::dht
+		| lt::alert_category::incoming_request
+		| lt::alert_category::dht_operation
+		| lt::alert_category::port_mapping_log
+		| lt::alert_category::file_progress);
 
 	lt::time_duration refresh_delay = lt::milliseconds(500);
 	bool rate_limit_locals = false;
@@ -1160,34 +1270,7 @@ example alert_masks:
 			std::string const key(start, equal - start);
 			char const* value = equal + 1;
 
-			int const sett_name = lt::setting_by_name(key);
-			if (sett_name < 0)
-			{
-				std::fprintf(stderr, "unknown setting: \"%s\"\n", key.c_str());
-				return 1;
-			}
-
-			switch (sett_name & settings_pack::type_mask)
-			{
-				case settings_pack::string_type_base:
-					settings.set_str(sett_name, value);
-					break;
-				case settings_pack::bool_type_base:
-					if (value == "0"_sv || value == "1"_sv)
-					{
-						settings.set_bool(sett_name, atoi(value) != 0);
-					}
-					else
-					{
-						std::fprintf(stderr, "invalid value for \"%s\". expected 0 or 1\n"
-							, key.c_str());
-						return 1;
-					}
-					break;
-				case settings_pack::int_type_base:
-					settings.set_int(sett_name, atoi(value));
-					break;
-			}
+			assign_setting(settings, key, value);
 			continue;
 		}
 
@@ -1248,7 +1331,7 @@ example alert_masks:
 					rate_limit_locals = true;
 					break;
 				}
-			case '0': disable_storage = true; --i;
+			case '0': disable_storage = true; --i; break;
 			case 'e':
 				{
 					loop_limit = atoi(arg);
@@ -1617,7 +1700,6 @@ example alert_masks:
 				if (c == '2') print_connecting_peers = !print_connecting_peers;
 				if (c == '3') print_timers = !print_timers;
 				if (c == '4') print_block = !print_block;
-				if (c == '5') print_peer_rate = !print_peer_rate;
 				if (c == '6') print_fails = !print_fails;
 				if (c == '7') print_send_bufs = !print_send_bufs;
 				if (c == '8') print_local_ip = !print_local_ip;
@@ -1662,7 +1744,7 @@ up/down arrow keys: select torrent
 COLUMN OPTIONS
 [1] toggle IP column                            [2] toggle show peer connection attempts
 [3] toggle timers column                        [4] toggle block progress column
-[5] toggle peer rate column                     [6] toggle failures column
+                                                [6] toggle failures column
 [7] toggle send buffers column                  [8] toggle local IP column
 )");
 					int tmp;
@@ -1796,19 +1878,32 @@ COLUMN OPTIONS
 				std::vector<lt::announce_entry> tr = h.trackers();
 				for (lt::announce_entry const& ae : h.trackers())
 				{
-					auto best_ae = std::min_element(ae.endpoints.begin(), ae.endpoints.end()
-						, [](lt::announce_endpoint const& l, lt::announce_endpoint const& r) { return l.fails < r.fails; } );
-
-					if (pos + 1 >= terminal_height) break;
-					std::snprintf(str, sizeof(str), "%2d %-55s fails: %-3d (%-3d) %s %s %5d \"%s\" %s\x1b[K\n"
-						, ae.tier, ae.url.c_str()
-						, best_ae != ae.endpoints.end() ? best_ae->fails : 0, ae.fail_limit, ae.verified?"OK ":"-  "
-						, to_string(best_ae != ae.endpoints.end() ? int(total_seconds(best_ae->next_announce - now)) : 0, 8).c_str()
-						, best_ae != ae.endpoints.end() && best_ae->min_announce > now ? int(total_seconds(best_ae->min_announce - now)) : 0
-						, best_ae != ae.endpoints.end() && best_ae->last_error ? best_ae->last_error.message().c_str() : ""
-						, best_ae != ae.endpoints.end() ? best_ae->message.c_str() : "");
+					std::snprintf(str, sizeof(str), "%2d %-55s %s\x1b[K\n"
+						, ae.tier, ae.url.c_str(), ae.verified?"OK ":"-  ");
 					out += str;
 					pos += 1;
+					int idx = 0;
+					for (auto const& ep : ae.endpoints)
+					{
+						++idx;
+						if (!ep.enabled) continue;
+						if (pos + 1 >= terminal_height) break;
+
+						std::snprintf(str, sizeof(str), "  [%2d] fails: %-3d (%-3d) %s %5d \"%s\" %s\x1b[K\n"
+							, idx
+							, ep.fails, ae.fail_limit
+							, to_string(int(total_seconds(ep.next_announce - now)), 8).c_str()
+							, ep.min_announce > now ? int(total_seconds(ep.min_announce - now)) : 0
+							, ep.last_error ? ep.last_error.message().c_str() : ""
+							, ep.message.c_str());
+						out += str;
+						pos += 1;
+						// we only need to show this error once, not for every
+						// endpoint
+						if (ep.last_error == boost::asio::error::host_not_found) break;
+					}
+
+					if (pos + 1 >= terminal_height) break;
 				}
 			}
 
