@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2007-2014, Arvid Norberg
+Copyright (c) 2007-2018, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,60 +30,101 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
-#ifndef TORRENT_ASSERT
+#ifndef TORRENT_ASSERT_HPP_INCLUDED
+#define TORRENT_ASSERT_HPP_INCLUDED
 
 #include "libtorrent/config.hpp"
+#include "libtorrent/aux_/export.hpp"
+
+#if TORRENT_USE_ASSERTS \
+	|| defined TORRENT_ASIO_DEBUGGING \
+	|| defined TORRENT_PROFILE_CALLS \
+	|| defined TORRENT_DEBUG_BUFFERS
 
-#if defined TORRENT_DEBUG || defined TORRENT_ASIO_DEBUGGING \
-	|| TORRENT_RELEASE_ASSERTS || defined TORRENT_DEBUG_BUFFERS
 #include <string>
+namespace libtorrent {
 std::string demangle(char const* name);
-TORRENT_EXPORT void print_backtrace(char* out, int len, int max_depth = 0);
+TORRENT_EXPORT void print_backtrace(char* out, int len, int max_depth = 0, void* ctx = nullptr);
+}
 #endif
 
+// this is to disable the warning of conditional expressions
+// being constant in msvc
+#ifdef _MSC_VER
+#define TORRENT_WHILE_0  \
+	__pragma( warning(push) ) \
+	__pragma( warning(disable:4127) ) \
+	while (false) \
+	__pragma( warning(pop) )
+#else
+#define TORRENT_WHILE_0 while (false)
+#endif
+
+
+namespace libtorrent {
+// declarations of the two functions
+
+// internal
+TORRENT_EXPORT void assert_print(char const* fmt, ...) TORRENT_FORMAT(1,2);
+
+// internal
+TORRENT_EXPORT void assert_fail(const char* expr, int line
+	, char const* file, char const* function, char const* val, int kind = 0);
+
+}
+
 #if TORRENT_USE_ASSERTS
 
-#if TORRENT_PRODUCTION_ASSERTS
-extern char const* libtorrent_assert_log;
+#ifdef TORRENT_PRODUCTION_ASSERTS
+extern TORRENT_EXPORT char const* libtorrent_assert_log;
 #endif
 
-#if (defined __linux__ || defined __MACH__) && defined __GNUC__ && !TORRENT_USE_SYSTEM_ASSERT
-
 #if TORRENT_USE_IOSTREAM
 #include <sstream>
 #endif
 
-TORRENT_EXPORT void assert_fail(const char* expr, int line, char const* file
-	, char const* function, char const* val, int kind = 0);
+#ifndef TORRENT_USE_SYSTEM_ASSERTS
 
 #define TORRENT_ASSERT_PRECOND(x) \
-	do { if (x) {} else assert_fail(#x, __LINE__, __FILE__, __PRETTY_FUNCTION__, "", 1); } while (false)
+	do { if (x) {} else libtorrent::assert_fail(#x, __LINE__, __FILE__, __func__, nullptr, 1); } TORRENT_WHILE_0
 
 #define TORRENT_ASSERT(x) \
-	do { if (x) {} else assert_fail(#x, __LINE__, __FILE__, __PRETTY_FUNCTION__, "", 0); } while (false)
+	do { if (x) {} else libtorrent::assert_fail(#x, __LINE__, __FILE__, __func__, nullptr, 0); } TORRENT_WHILE_0
 
 #if TORRENT_USE_IOSTREAM
 #define TORRENT_ASSERT_VAL(x, y) \
 	do { if (x) {} else { std::stringstream __s__; __s__ << #y ": " << y; \
-	assert_fail(#x, __LINE__, __FILE__, __PRETTY_FUNCTION__, __s__.str().c_str(), 0); } } while (false)
+	libtorrent::assert_fail(#x, __LINE__, __FILE__, __func__, __s__.str().c_str(), 0); } } TORRENT_WHILE_0
+
+#define TORRENT_ASSERT_FAIL_VAL(y) \
+	do { std::stringstream __s__; __s__ << #y ": " << y; \
+	libtorrent::assert_fail("<unconditional>", __LINE__, __FILE__, __func__, __s__.str().c_str(), 0); } TORRENT_WHILE_0
+
 #else
 #define TORRENT_ASSERT_VAL(x, y) TORRENT_ASSERT(x)
+#define TORRENT_ASSERT_FAIL_VAL(x) TORRENT_ASSERT_FAIL()
 #endif
 
+#define TORRENT_ASSERT_FAIL() \
+	libtorrent::assert_fail("<unconditional>", __LINE__, __FILE__, __func__, nullptr, 0)
+
 #else
 #include <cassert>
 #define TORRENT_ASSERT_PRECOND(x) assert(x)
 #define TORRENT_ASSERT(x) assert(x)
 #define TORRENT_ASSERT_VAL(x, y) assert(x)
+#define TORRENT_ASSERT_FAIL_VAL(x) assert(false)
+#define TORRENT_ASSERT_FAIL() assert(false)
 #endif
 
 #else // TORRENT_USE_ASSERTS
 
-#define TORRENT_ASSERT_PRECOND(a) do {} while(false)
-#define TORRENT_ASSERT(a) do {} while(false)
-#define TORRENT_ASSERT_VAL(a, b) do {} while(false)
+#define TORRENT_ASSERT_PRECOND(a) do {} TORRENT_WHILE_0
+#define TORRENT_ASSERT(a) do {} TORRENT_WHILE_0
+#define TORRENT_ASSERT_VAL(a, b) do {} TORRENT_WHILE_0
+#define TORRENT_ASSERT_FAIL_VAL(a) do {} TORRENT_WHILE_0
+#define TORRENT_ASSERT_FAIL() do {} TORRENT_WHILE_0
 
 #endif // TORRENT_USE_ASSERTS
 
-#endif
-
+#endif // TORRENT_ASSERT_HPP_INCLUDED
