@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2007-2016, Arvid Norberg
+Copyright (c) 2007-2018, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -37,29 +37,24 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/config.hpp"
 
-#include "libtorrent/aux_/disable_warnings_push.hpp"
+#include <memory>
 
-#include <boost/shared_ptr.hpp>
+namespace libtorrent {
 
-#include "libtorrent/aux_/disable_warnings_pop.hpp"
-
-namespace libtorrent
-{
 	struct torrent_plugin;
 	struct torrent_handle;
 
 	// constructor function for the ut_metadata extension. The ut_metadata
 	// extension allows peers to request the .torrent file (or more
-	// specifically the 'info'-dictionary of the .torrent file) from each
+	// specifically the info-dictionary of the .torrent file) from each
 	// other. This is the main building block in making magnet links work.
 	// This extension is enabled by default unless explicitly disabled in
 	// the session constructor.
-	// 
+	//
 	// This can either be passed in the add_torrent_params::extensions field, or
 	// via torrent_handle::add_extension().
-	TORRENT_EXPORT boost::shared_ptr<torrent_plugin> create_ut_metadata_plugin(torrent_handle const&, void*);
+	TORRENT_EXPORT std::shared_ptr<torrent_plugin> create_ut_metadata_plugin(torrent_handle const&, void*);
 }
 
 #endif // TORRENT_DISABLE_EXTENSIONS
 #endif // TORRENT_UT_METADATA_HPP_INCLUDED
-
