@@ -38,16 +38,16 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/address.hpp"
 #include "libtorrent/union_endpoint.hpp"
 #include "libtorrent/time.hpp" // for time_point
+#include "libtorrent/aux_/time.hpp" // for time_now
 
-namespace libtorrent { namespace dht
-{
+namespace libtorrent { namespace dht {
 
 struct TORRENT_EXTRA_EXPORT node_entry
 {
-	node_entry(node_id const& id_, udp::endpoint ep, int roundtriptime = 0xffff
+	node_entry(node_id const& id_, udp::endpoint const& ep, int roundtriptime = 0xffff
 		, bool pinged = false);
-	node_entry(udp::endpoint ep);
-	node_entry();
+	explicit node_entry(udp::endpoint const& ep);
+	node_entry() = default;
 	void update_rtt(int new_rtt);
 
 	bool pinged() const { return timeout_count != 0xff; }
@@ -55,32 +55,39 @@ struct TORRENT_EXTRA_EXPORT node_entry
 	void timed_out() { if (pinged() && timeout_count < 0xfe) ++timeout_count; }
 	int fail_count() const { return pinged() ? timeout_count : 0; }
 	void reset_fail_count() { if (pinged()) timeout_count = 0; }
-	udp::endpoint ep() const { return udp::endpoint(address_v4(a), p); }
+	udp::endpoint ep() const { return endpoint; }
 	bool confirmed() const { return timeout_count == 0; }
-	address addr() const { return address_v4(a); }
-	int port() const { return p; }
+	address addr() const { return endpoint.address(); }
+	int port() const { return endpoint.port; }
+
+	// compares which node_entry is "better". Smaller is better
+	bool operator<(node_entry const& rhs) const
+	{
+		return std::make_tuple(!verified, rtt) < std::make_tuple(!rhs.verified, rhs.rtt);
+	}
 
 #ifndef TORRENT_DISABLE_LOGGING
-	time_point first_seen;
+	time_point first_seen = aux::time_now();
 #endif
 
 	// the time we last received a response for a request to this peer
-	time_point last_queried;
+	time_point last_queried = min_time();
 
-	node_id id;
+	node_id id{nullptr};
 
-	address_v4::bytes_type a;
-	boost::uint16_t p;
+	union_endpoint endpoint;
 
 	// the average RTT of this node
-	boost::uint16_t rtt;
+	std::uint16_t rtt = 0xffff;
 
 	// the number of times this node has failed to
 	// respond in a row
-	boost::uint8_t timeout_count;
+	// 0xff is a special value to indicate we have not pinged this node yet
+	std::uint8_t timeout_count = 0xff;
+
+	bool verified = false;
 };
 
 } } // namespace libtorrent::dht
 
 #endif
-
