@@ -33,39 +33,26 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef ROUTING_TABLE_HPP
 #define ROUTING_TABLE_HPP
 
-#include "libtorrent/aux_/disable_warnings_push.hpp"
-
 #include <vector>
 #include <set>
+#include <unordered_set>
+#include <cstdint>
+#include <tuple>
+#include <array>
 
-#include <boost/cstdint.hpp>
-#include <boost/utility.hpp>
-#include <boost/tuple/tuple.hpp>
-#include <boost/array.hpp>
-#include <boost/noncopyable.hpp>
-#include <boost/unordered_set.hpp>
-
-#include "libtorrent/aux_/disable_warnings_pop.hpp"
-
+#include <libtorrent/fwd.hpp>
 #include <libtorrent/kademlia/node_id.hpp>
 #include <libtorrent/kademlia/node_entry.hpp>
-#include <libtorrent/session_settings.hpp>
 #include <libtorrent/assert.hpp>
 #include <libtorrent/time.hpp>
+#include <libtorrent/aux_/vector.hpp>
 
-namespace libtorrent
-{
-#ifndef TORRENT_NO_DEPRECATE
-	struct session_status;
-#endif
-	struct dht_routing_bucket;
-}
+namespace libtorrent { namespace dht {
 
-namespace libtorrent { namespace dht
-{
+struct settings;
 struct dht_logger;
 
-typedef std::vector<node_entry> bucket_t;
+using bucket_t = aux::vector<node_entry>;
 
 struct routing_table_node
 {
@@ -73,6 +60,63 @@ struct routing_table_node
 	bucket_t live_nodes;
 };
 
+struct ipv4_hash
+{
+	using argument_type = address_v4::bytes_type;
+	using result_type = std::size_t;
+	result_type operator()(argument_type const& ip) const
+	{
+		return std::hash<std::uint32_t>()(*reinterpret_cast<std::uint32_t const*>(&ip[0]));
+	}
+};
+
+struct ipv6_hash
+{
+	using argument_type = address_v6::bytes_type;
+	using result_type = std::size_t;
+	result_type operator()(argument_type const& ip) const
+	{
+		return std::hash<std::uint64_t>()(*reinterpret_cast<std::uint64_t const*>(&ip[0]));
+	}
+};
+
+struct TORRENT_EXTRA_EXPORT ip_set
+{
+	void insert(address const& addr);
+	bool exists(address const& addr) const;
+	void erase(address const& addr);
+
+	void clear()
+	{
+		m_ip4s.clear();
+		m_ip6s.clear();
+	}
+
+	bool operator==(ip_set const& rh)
+	{
+		return m_ip4s == rh.m_ip4s && m_ip6s == rh.m_ip6s;
+	}
+
+	std::size_t size() const { return m_ip4s.size() + m_ip6s.size(); }
+
+	// these must be multisets because there can be multiple routing table
+	// entries for a single IP when restrict_routing_ips is set to false
+	std::unordered_multiset<address_v4::bytes_type, ipv4_hash> m_ip4s;
+	std::unordered_multiset<address_v6::bytes_type, ipv6_hash> m_ip6s;
+};
+
+// Each routing table bucket represents node IDs with a certain number of bits
+// of prefix in common with our own node ID. Each bucket fits 8 nodes (and
+// sometimes more, closer to the top). In order to minimize the number of hops
+// necessary to traverse the DHT, we want the nodes in our buckets to be spread
+// out across all possible "sub-branches". This is what the "classify" refers
+// to. The 3 (or more) bits following the shared bit prefix.
+TORRENT_EXTRA_EXPORT std::uint8_t classify_prefix(int bucket_idx, bool last_bucket
+	, int bucket_size, node_id nid);
+
+TORRENT_EXTRA_EXPORT bool all_in_same_bucket(span<node_entry const> b
+	, node_id const& id, int bucket_index);
+
 // differences in the implementation from the description in
 // the paper:
 //
@@ -82,30 +126,32 @@ struct routing_table_node
 // 	the most times is replaced. If none of the nodes in the
 // 	bucket has failed, then it is put in the replacement
 // 	cache (just like in the paper).
+// * The routing table bucket sizes are larger towards the "top" of the routing
+// 	table. This is to get closer to the target in fewer round-trips.
+// * Nodes with lower RTT are preferred and may replace nodes with higher RTT
+// * Nodes that are "verified" (i.e. use a node-ID derived from their IP) are
+// 	preferred and may replace nodes that are not verified.
 
-namespace impl
-{
-	template <typename F>
-	inline void forwarder(void* userdata, node_entry const& node)
-	{
-		F* f = reinterpret_cast<F*>(userdata);
-		(*f)(node);
-	}
-}
+TORRENT_EXTRA_EXPORT bool mostly_verified_nodes(bucket_t const&);
+TORRENT_EXTRA_EXPORT bool compare_ip_cidr(address const& lhs, address const& rhs);
 
-class TORRENT_EXTRA_EXPORT routing_table : boost::noncopyable
+class TORRENT_EXTRA_EXPORT routing_table
 {
 public:
 	// TODO: 3 to improve memory locality and scanning performance, turn the
 	// routing table into a single vector with boundaries for the nodes instead.
 	// Perhaps replacement nodes should be in a separate vector.
-	typedef std::vector<routing_table_node> table_t;
+	using table_t = aux::vector<routing_table_node>;
 
-	routing_table(node_id const& id, int bucket_size
-		, dht_settings const& settings
+	routing_table(node_id const& id, udp proto
+		, int bucket_size
+		, dht::settings const& settings
 		, dht_logger* log);
 
-#ifndef TORRENT_NO_DEPRECATE
+	routing_table(routing_table const&) = delete;
+	routing_table& operator=(routing_table const&) = delete;
+
+#if TORRENT_ABI_VERSION == 1
 	void status(session_status& s) const;
 #endif
 
@@ -115,12 +161,12 @@ public:
 
 	// adds an endpoint that will never be added to
 	// the routing table
-	void add_router_node(udp::endpoint router);
+	void add_router_node(udp::endpoint const& router);
 
 	// iterates over the router nodes added
-	typedef std::set<udp::endpoint>::const_iterator router_iterator;
-	router_iterator router_begin() const { return m_router_nodes.begin(); }
-	router_iterator router_end() const { return m_router_nodes.end(); }
+	using router_iterator = std::set<udp::endpoint>::const_iterator;
+	router_iterator begin() const { return m_router_nodes.begin(); }
+	router_iterator end() const { return m_router_nodes.end(); }
 
 	enum add_node_status_t {
 		failed_to_add = 0,
@@ -129,13 +175,13 @@ public:
 	};
 	add_node_status_t add_node_impl(node_entry e);
 
-	bool add_node(node_entry e);
+	bool add_node(node_entry const& e);
 
 	// this function is called every time the node sees
 	// a sign of a node being alive. This node will either
 	// be inserted in the k-buckets or be moved to the top
 	// of its bucket.
-	bool node_seen(node_id const& id, udp::endpoint ep, int rtt);
+	bool node_seen(node_id const& id, udp::endpoint const& ep, int rtt);
 
 	// this may add a node to the routing table and mark it as
 	// not pinged. If the bucket the node falls into is full,
@@ -144,7 +190,7 @@ public:
 
 	// change our node ID. This can be expensive since nodes must be moved around
 	// and potentially dropped
-	void update_node_id(node_id id);
+	void update_node_id(node_id const& id);
 
 	node_entry const* next_refresh();
 
@@ -158,12 +204,11 @@ public:
 	// are nearest to the given id.
 	void find_node(node_id const& id, std::vector<node_entry>& l
 		, int options, int count = 0);
-	void remove_node(node_entry* n
-		, table_t::iterator bucket) ;
+	void remove_node(node_entry* n, bucket_t* b);
 
 	int bucket_size(int bucket) const
 	{
-		int num_buckets = m_buckets.size();
+		int num_buckets = int(m_buckets.size());
 		if (num_buckets == 0) return 0;
 		if (bucket >= num_buckets) bucket = num_buckets - 1;
 		table_t::const_iterator i = m_buckets.begin();
@@ -171,36 +216,27 @@ public:
 		return int(i->live_nodes.size());
 	}
 
-	template <typename F>
-	void for_each_node(F f)
-	{
-		for_each_node(&impl::forwarder<F>, &impl::forwarder<F>, reinterpret_cast<void*>(&f));
-	}
+	void for_each_node(std::function<void(node_entry const&)> live_cb
+		, std::function<void(node_entry const&)> replacements_cb) const;
 
-	void for_each_node(void (*)(void*, node_entry const&)
-		, void (*)(void*, node_entry const&), void* userdata) const;
+	void for_each_node(std::function<void(node_entry const&)> f) const
+	{ for_each_node(f, f); }
 
 	int bucket_size() const { return m_bucket_size; }
 
 	// returns the number of nodes in the main buckets, number of nodes in the
 	// replacement buckets and the number of nodes in the main buckets that have
 	// been pinged and confirmed up
-	boost::tuple<int, int, int> size() const;
+	std::tuple<int, int, int> size() const;
 
-	boost::int64_t num_global_nodes() const;
+	std::int64_t num_global_nodes() const;
 
 	// the number of bits down we have full buckets
 	// i.e. essentially the number of full buckets
 	// we have
 	int depth() const;
 
-	int num_active_buckets() const { return m_buckets.size(); }
-
-#if defined TORRENT_DEBUG
-	// used for debug and monitoring purposes. This will print out
-	// the state of the routing table to the given stream
-	void print_state(std::ostream& os) const;
-#endif
+	int num_active_buckets() const { return int(m_buckets.size()); }
 
 	int bucket_limit(int bucket) const;
 
@@ -210,23 +246,46 @@ public:
 
 	bool is_full(int bucket) const;
 
+	bool native_address(address const& addr) const
+	{
+		return (addr.is_v4() && m_protocol == udp::v4())
+			|| (addr.is_v6() && m_protocol == udp::v6());
+	}
+
+	bool native_endpoint(udp::endpoint const& ep) const
+	{ return ep.protocol() == m_protocol; }
+
+	node_id const& id() const
+	{ return m_id; }
+
+	table_t const& buckets() const
+	{ return m_buckets; }
+
 private:
 
 #ifndef TORRENT_DISABLE_LOGGING
 	dht_logger* m_log;
+	void log_node_failed(node_id const& nid, node_entry const& ne) const;
 #endif
 
 	table_t::iterator find_bucket(node_id const& id);
+	void remove_node_internal(node_entry* n, bucket_t& b);
 
 	void split_bucket();
 
 	// return a pointer the node_entry with the given endpoint
 	// or 0 if we don't have such a node. Both the address and the
 	// port has to match
-	node_entry* find_node(udp::endpoint const& ep
-		, routing_table::table_t::iterator* bucket);
+	std::tuple<node_entry*, routing_table::table_t::iterator, bucket_t*>
+	find_node(udp::endpoint const& ep);
 
-	dht_settings const& m_settings;
+	// if the bucket is not full, try to fill it with nodes from the
+	// replacement list
+	void fill_from_replacements(table_t::iterator bucket);
+
+	void prune_empty_bucket();
+
+	dht::settings const& m_settings;
 
 	// (k-bucket, replacement cache) pairs
 	// the first entry is the bucket the furthest
@@ -237,6 +296,7 @@ private:
 	table_t m_buckets;
 
 	node_id m_id; // our own node id
+	udp m_protocol; // protocol this table is for
 
 	// the last seen depth (i.e. levels in the routing table)
 	// it's mutable because it's updated by depth(), which is const
@@ -254,15 +314,21 @@ private:
 
 	// these are all the IPs that are in the routing
 	// table. It's used to only allow a single entry
-	// per IP in the whole table. Currently only for
-	// IPv4
-	boost::unordered_multiset<address_v4::bytes_type> m_ips;
+	// per IP in the whole table.
+	ip_set m_ips;
 
 	// constant called k in paper
-	int m_bucket_size;
+	int const m_bucket_size;
 };
 
+TORRENT_EXTRA_EXPORT routing_table::add_node_status_t
+replace_node_impl(node_entry const& e, bucket_t& b, ip_set& ips
+	, int bucket_index, int bucket_size_limit, bool last_bucket
+#ifndef TORRENT_DISABLE_LOGGING
+	, dht_logger* log
+#endif
+	);
+
 } } // namespace libtorrent::dht
 
 #endif // ROUTING_TABLE_HPP
-
