@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2007-2014, Arvid Norberg
+Copyright (c) 2007-2018, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -34,52 +34,43 @@ POSSIBILITY OF SUCH DAMAGE.
 #define TORRENT_LSD_HPP
 
 #include "libtorrent/socket.hpp"
-#include "libtorrent/peer_id.hpp"
+#include "libtorrent/sha1_hash.hpp"
 #include "libtorrent/broadcast_socket.hpp"
-#include "libtorrent/intrusive_ptr_base.hpp"
 #include "libtorrent/deadline_timer.hpp"
+#include "libtorrent/aux_/lsd.hpp"
 
-#include <boost/function/function2.hpp>
-#include <boost/noncopyable.hpp>
-#include <boost/shared_ptr.hpp>
+namespace libtorrent {
 
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-#include <stdio.h>
-#endif
-
-namespace libtorrent
+struct lsd : std::enable_shared_from_this<lsd>
 {
-
-typedef boost::function<void(tcp::endpoint, sha1_hash)> peer_callback_t;
-
-class lsd : public intrusive_ptr_base<lsd>
-{
-public:
-	lsd(io_service& ios, address const& listen_interface
-		, peer_callback_t const& cb);
+	lsd(io_service& ios, aux::lsd_callback& cb
+		, address const& listen_address, address const& netmask);
 	~lsd();
 
-//	void rebind(address const& listen_interface);
+	void start(error_code& ec);
 
-	void announce(sha1_hash const& ih, int listen_port, bool broadcast = false);
+	void announce(sha1_hash const& ih, int listen_port);
 	void close();
 
 private:
 
-	void announce_impl(sha1_hash const& ih, int listen_port
-		, bool broadcast, int retry_count);
+	std::shared_ptr<lsd> self() { return shared_from_this(); }
+
+	void announce_impl(sha1_hash const& ih, int listen_port, int retry_count);
 	void resend_announce(error_code const& e, sha1_hash const& ih
 		, int listen_port, int retry_count);
-	void on_announce(udp::endpoint const& from, char* buffer
-		, std::size_t bytes_transferred);
+	void on_announce(error_code const& ec);
 
-	peer_callback_t m_callback;
+	aux::lsd_callback& m_callback;
 
-	// the udp socket used to send and receive
-	// multicast messages on
-	broadcast_socket m_socket;
-#if TORRENT_USE_IPV6
-	broadcast_socket m_socket6;
+	address m_listen_address;
+	address m_netmask;
+
+	udp::socket m_socket;
+
+#ifndef TORRENT_DISABLE_LOGGING
+	bool should_log() const;
+	void debug_log(char const* fmt, ...) const TORRENT_FORMAT(2, 3);
 #endif
 
 	// used to resend udp packets in case
@@ -93,17 +84,9 @@ private:
 	// as a peer
 	int m_cookie;
 
-	bool m_disabled;
-#if TORRENT_USE_IPV6
-	bool m_disabled6;
-#endif
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-	FILE* m_log;
-#endif
+	bool m_disabled = false;
 };
 
 }
 
-
 #endif
-
