@@ -31,25 +31,28 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "libtorrent/bdecode.hpp"
-#include "libtorrent/alloca.hpp"
-#include <boost/system/error_code.hpp>
+#include "libtorrent/aux_/alloca.hpp"
+#include "libtorrent/aux_/numeric_cast.hpp"
+#include "libtorrent/error_code.hpp"
 #include <limits>
 #include <cstring> // for memset
+#include <cstdio> // for snprintf
+#include <cinttypes> // for PRId64 et.al.
 
 #ifndef BOOST_SYSTEM_NOEXCEPT
 #define BOOST_SYSTEM_NOEXCEPT throw()
 #endif
 
-namespace libtorrent
-{
+namespace libtorrent {
+
 	using detail::bdecode_token;
 
-	namespace
-	{
+namespace {
+
 	bool numeric(char c) { return c >= '0' && c <= '9'; }
 
 	// finds the end of an integer and verifies that it looks valid this does
-	// not detect all overflows, just the ones that are an order of magnitued
+	// not detect all overflows, just the ones that are an order of magnitude
 	// beyond. Exact overflow checking is done when the integer value is queried
 	// from a bdecode_node.
 	char const* check_integer(char const* start, char const* end
@@ -100,30 +103,43 @@ namespace libtorrent
 
 	struct stack_frame
 	{
-		stack_frame(int t): token(t), state(0) {}
+		stack_frame() : token(0), state(0) {}
+		explicit stack_frame(int const t): token(std::uint32_t(t)), state(0) {}
 		// this is an index into m_tokens
-		boost::uint32_t token:31;
+		std::uint32_t token:31;
 		// this is used for dictionaries to indicate whether we're
 		// reading a key or a vale. 0 means key 1 is value
-		boost::uint32_t state:1;
+		std::uint32_t state:1;
 	};
 
-	// str1 is null-terminated
-	// str2 is not, str2 is len2 chars
-	bool string_equal(char const* str1, char const* str2, int len2)
+	// diff between current and next item offset
+	// should only be called for non last item in array
+	int token_source_span(bdecode_token const& t)
 	{
-		while (len2 > 0)
-		{
-			if (*str1 != *str2) return false;
-			if (*str1 == 0) return false;
-			++str1;
-			++str2;
-			--len2;
-		}
-		return *str1 == 0;
+		return (&t)[1].offset - t.offset;
 	}
 
-	} // anonymous namespace
+} // anonymous namespace
+
+namespace detail {
+	void escape_string(std::string& ret, char const* str, int len)
+	{
+		for (int i = 0; i < len; ++i)
+		{
+			if (str[i] >= 32 && str[i] < 127)
+			{
+				ret += str[i];
+			}
+			else
+			{
+				char tmp[5];
+				std::snprintf(tmp, sizeof(tmp), "\\x%02x", std::uint8_t(str[i]));
+				ret += tmp;
+			}
+		}
+	}
+}
+
 
 
 	// reads the string between start and end, or up to the first occurrance of
@@ -133,7 +149,7 @@ namespace libtorrent
 	// cannot be represented by the variable 'val' and overflow error is reported
 	// by 'ec'.
 	char const* parse_int(char const* start, char const* end, char delimiter
-		, boost::int64_t& val, bdecode_errors::error_code_enum& ec)
+		, std::int64_t& val, bdecode_errors::error_code_enum& ec)
 	{
 		while (start < end && *start != delimiter)
 		{
@@ -142,14 +158,14 @@ namespace libtorrent
 				ec = bdecode_errors::expected_digit;
 				return start;
 			}
-			if (val > (std::numeric_limits<boost::int64_t>::max)() / 10)
+			if (val > std::numeric_limits<std::int64_t>::max() / 10)
 			{
 				ec = bdecode_errors::overflow;
 				return start;
 			}
 			val *= 10;
 			int digit = *start - '0';
-			if (val > (std::numeric_limits<boost::int64_t>::max)() - digit)
+			if (val > std::numeric_limits<std::int64_t>::max() - digit)
 			{
 				ec = bdecode_errors::overflow;
 				return start;
@@ -161,13 +177,13 @@ namespace libtorrent
 	}
 
 
-	struct bdecode_error_category : boost::system::error_category
+	struct bdecode_error_category final : boost::system::error_category
 	{
-		virtual const char* name() const BOOST_SYSTEM_NOEXCEPT;
-		virtual std::string message(int ev) const BOOST_SYSTEM_NOEXCEPT;
-		virtual boost::system::error_condition default_error_condition(
-			int ev) const BOOST_SYSTEM_NOEXCEPT
-		{ return boost::system::error_condition(ev, *this); }
+		const char* name() const BOOST_SYSTEM_NOEXCEPT override;
+		std::string message(int ev) const override;
+		boost::system::error_condition default_error_condition(
+			int ev) const BOOST_SYSTEM_NOEXCEPT override
+		{ return {ev, *this}; }
 	};
 
 	const char* bdecode_error_category::name() const BOOST_SYSTEM_NOEXCEPT
@@ -175,7 +191,7 @@ namespace libtorrent
 		return "bdecode";
 	}
 
-	std::string bdecode_error_category::message(int ev) const BOOST_SYSTEM_NOEXCEPT
+	std::string bdecode_error_category::message(int ev) const
 	{
 		static char const* msgs[] =
 		{
@@ -199,30 +215,14 @@ namespace libtorrent
 		return bdecode_category;
 	}
 
-#ifndef TORRENT_NO_DEPRECATED
-	boost::system::error_category& get_bdecode_category()
-	{ return bdecode_category(); }
-#endif
-
 	namespace bdecode_errors
 	{
 		boost::system::error_code make_error_code(error_code_enum e)
 		{
-			return boost::system::error_code(e, bdecode_category());
+			return {e, bdecode_category()};
 		}
 	}
 
-
-	bdecode_node::bdecode_node()
-		: m_root_tokens(0)
-		, m_buffer(NULL)
-		, m_buffer_size(0)
-		, m_token_idx(-1)
-		, m_last_index(-1)
-		, m_last_token(-1)
-		, m_size(-1)
-	{}
-
 	bdecode_node::bdecode_node(bdecode_node const& n)
 		: m_tokens(n.m_tokens)
 		, m_root_tokens(n.m_root_tokens)
@@ -238,6 +238,7 @@ namespace libtorrent
 
 	bdecode_node& bdecode_node::operator=(bdecode_node const& n)
 	{
+		if (&n == this) return *this;
 		m_tokens = n.m_tokens;
 		m_root_tokens = n.m_root_tokens;
 		m_buffer = n.m_buffer;
@@ -255,6 +256,8 @@ namespace libtorrent
 		return *this;
 	}
 
+	bdecode_node::bdecode_node(bdecode_node&&) noexcept = default;
+
 	bdecode_node::bdecode_node(bdecode_token const* tokens, char const* buf
 		, int len, int idx)
 		: m_root_tokens(tokens)
@@ -265,7 +268,7 @@ namespace libtorrent
 		, m_last_token(-1)
 		, m_size(-1)
 	{
-		TORRENT_ASSERT(tokens != NULL);
+		TORRENT_ASSERT(tokens != nullptr);
 		TORRENT_ASSERT(idx >= 0);
 	}
 
@@ -282,14 +285,14 @@ namespace libtorrent
 	void bdecode_node::clear()
 	{
 		m_tokens.clear();
-		m_root_tokens = NULL;
+		m_root_tokens = nullptr;
 		m_token_idx = -1;
 		m_size = -1;
 		m_last_index = -1;
 		m_last_token = -1;
 	}
 
-	void bdecode_node::switch_underlying_buffer(char const* buf)
+	void bdecode_node::switch_underlying_buffer(char const* buf) noexcept
 	{
 		TORRENT_ASSERT(!m_tokens.empty());
 		if (m_tokens.empty()) return;
@@ -297,23 +300,112 @@ namespace libtorrent
 		m_buffer = buf;
 	}
 
-	bdecode_node::type_t bdecode_node::type() const
+	bool bdecode_node::has_soft_error(span<char> error) const
+	{
+		if (type() == none_t) return false;
+
+		bdecode_token const* tokens = m_root_tokens;
+		int token = m_token_idx;
+
+		// we don't know what the original depth_limit was
+		// so this has to go on the heap
+		std::vector<int> stack;
+		// make the initial allocation the default depth_limit
+		stack.reserve(100);
+
+		do
+		{
+			switch (tokens[token].type)
+			{
+			case bdecode_token::integer:
+				if (m_buffer[tokens[token].offset + 1] == '0'
+					&& m_buffer[tokens[token].offset + 2] != 'e')
+				{
+					std::snprintf(error.data(), std::size_t(error.size()), "leading zero in integer");
+					return true;
+				}
+				break;
+			case bdecode_token::string:
+				if (m_buffer[tokens[token].offset] == '0'
+					&& m_buffer[tokens[token].offset + 1] != ':')
+				{
+					std::snprintf(error.data(), std::size_t(error.size()), "leading zero in string length");
+					return true;
+				}
+				break;
+			case bdecode_token::dict:
+			case bdecode_token::list:
+				stack.push_back(token);
+				break;
+			case bdecode_token::end:
+				auto const parent = stack.back();
+				stack.pop_back();
+				if (tokens[parent].type == bdecode_token::dict
+					&& token != parent + 1)
+				{
+					// this is the end of a non-empty dict
+					// check the sort order of the keys
+					int k1 = parent + 1;
+					for (;;)
+					{
+						// skip to the first key's value
+						int const v1 = k1 + tokens[k1].next_item;
+						// then to the next key
+						int const k2 = v1 + tokens[v1].next_item;
+
+						// check if k1 was the last key in the dict
+						if (k2 == token)
+							break;
+
+						int const v2 = k2 + tokens[k2].next_item;
+
+						int const k1_start = tokens[k1].offset + tokens[k1].start_offset();
+						int const k1_len = tokens[v1].offset - k1_start;
+						int const k2_start = tokens[k2].offset + tokens[k2].start_offset();
+						int const k2_len = tokens[v2].offset - k2_start;
+
+						int const min_len = std::min(k1_len, k2_len);
+
+						int cmp = std::memcmp(m_buffer + k1_start, m_buffer + k2_start, std::size_t(min_len));
+						if (cmp > 0 || (cmp == 0 && k1_len > k2_len))
+						{
+							std::snprintf(error.data(), std::size_t(error.size()), "unsorted dictionary key");
+							return true;
+						}
+						else if (cmp == 0 && k1_len == k2_len)
+						{
+							std::snprintf(error.data(), std::size_t(error.size()), "duplicate dictionary key");
+							return true;
+						}
+
+						k1 = k2;
+					}
+				}
+				break;
+			}
+
+			++token;
+		} while (!stack.empty());
+		return false;
+	}
+
+	bdecode_node::type_t bdecode_node::type() const noexcept
 	{
 		if (m_token_idx == -1) return none_t;
 		return static_cast<bdecode_node::type_t>(m_root_tokens[m_token_idx].type);
 	}
 
-	bdecode_node::operator bool() const
+	bdecode_node::operator bool() const noexcept
 	{ return m_token_idx != -1; }
 
-	std::pair<char const*, int> bdecode_node::data_section() const
+	span<char const> bdecode_node::data_section() const noexcept
 	{
-		if (m_token_idx == -1) return std::make_pair(m_buffer, 0);
+		if (m_token_idx == -1) return {};
 
 		TORRENT_ASSERT(m_token_idx != -1);
 		bdecode_token const& t = m_root_tokens[m_token_idx];
 		bdecode_token const& next = m_root_tokens[m_token_idx + t.next_item];
-		return std::make_pair(m_buffer + t.offset, next.offset - t.offset);
+		return {m_buffer + t.offset, static_cast<std::ptrdiff_t>(next.offset - t.offset)};
 	}
 
 	bdecode_node bdecode_node::list_at(int i) const
@@ -350,18 +442,18 @@ namespace libtorrent
 		return bdecode_node(tokens, m_buffer, m_buffer_size, token);
 	}
 
-	std::string bdecode_node::list_string_value_at(int i
-		, char const* default_val) const
+	string_view bdecode_node::list_string_value_at(int i
+		, string_view default_val) const
 	{
-		bdecode_node n = list_at(i);
+		bdecode_node const n = list_at(i);
 		if (n.type() != bdecode_node::string_t) return default_val;
 		return n.string_value();
 	}
 
-	boost::int64_t bdecode_node::list_int_value_at(int i
-		, boost::int64_t default_val) const
+	std::int64_t bdecode_node::list_int_value_at(int i
+		, std::int64_t default_val) const
 	{
-		bdecode_node n = list_at(i);
+		bdecode_node const n = list_at(i);
 		if (n.type() != bdecode_node::int_t) return default_val;
 		return n.int_value();
 	}
@@ -379,7 +471,7 @@ namespace libtorrent
 		// this is the first item
 		int token = m_token_idx + 1;
 		int ret = 0;
-		
+
 		// do we have a lookup cached?
 		if (m_last_index != -1)
 		{
@@ -397,11 +489,11 @@ namespace libtorrent
 		return ret;
 	}
 
-	std::pair<std::string, bdecode_node> bdecode_node::dict_at(int i) const
+	std::pair<string_view, bdecode_node> bdecode_node::dict_at(int i) const
 	{
 		TORRENT_ASSERT(type() == dict_t);
 		TORRENT_ASSERT(m_token_idx != -1);
-	
+
 		bdecode_token const* tokens = m_root_tokens;
 		TORRENT_ASSERT(tokens[m_token_idx].type == bdecode_token::dict);
 
@@ -485,12 +577,12 @@ namespace libtorrent
 		return ret;
 	}
 
-	bdecode_node bdecode_node::dict_find(std::string key) const
+	bdecode_node bdecode_node::dict_find(string_view key) const
 	{
 		TORRENT_ASSERT(type() == dict_t);
 
-		bdecode_token const* tokens = m_root_tokens;
-	
+		bdecode_token const* const tokens = m_root_tokens;
+
 		// this is the first item
 		int token = m_token_idx + 1;
 
@@ -498,15 +590,15 @@ namespace libtorrent
 		{
 			bdecode_token const& t = tokens[token];
 			TORRENT_ASSERT(t.type == bdecode_token::string);
-			int size = m_root_tokens[token + 1].offset - t.offset - t.start_offset();
+			int const size = token_source_span(t) - t.start_offset();
 			if (int(key.size()) == size
-				&& std::equal(key.c_str(), key.c_str() + size, m_buffer
+				&& std::equal(key.data(), key.data() + size, m_buffer
 					+ t.offset + t.start_offset()))
 			{
 				// skip key
 				token += t.next_item;
 				TORRENT_ASSERT(tokens[token].type != bdecode_token::end);
-			
+
 				return bdecode_node(tokens, m_buffer, m_buffer_size, token);
 			}
 
@@ -521,7 +613,7 @@ namespace libtorrent
 		return bdecode_node();
 	}
 
-	bdecode_node bdecode_node::dict_find_list(char const* key) const
+	bdecode_node bdecode_node::dict_find_list(string_view key) const
 	{
 		bdecode_node ret = dict_find(key);
 		if (ret.type() == bdecode_node::list_t)
@@ -529,7 +621,7 @@ namespace libtorrent
 		return bdecode_node();
 	}
 
-	bdecode_node bdecode_node::dict_find_dict(std::string key) const
+	bdecode_node bdecode_node::dict_find_dict(string_view key) const
 	{
 		bdecode_node ret = dict_find(key);
 		if (ret.type() == bdecode_node::dict_t)
@@ -537,15 +629,7 @@ namespace libtorrent
 		return bdecode_node();
 	}
 
-	bdecode_node bdecode_node::dict_find_dict(char const* key) const
-	{
-		bdecode_node ret = dict_find(key);
-		if (ret.type() == bdecode_node::dict_t)
-			return ret;
-		return bdecode_node();
-	}
-
-	bdecode_node bdecode_node::dict_find_string(char const* key) const
+	bdecode_node bdecode_node::dict_find_string(string_view key) const
 	{
 		bdecode_node ret = dict_find(key);
 		if (ret.type() == bdecode_node::string_t)
@@ -553,7 +637,7 @@ namespace libtorrent
 		return bdecode_node();
 	}
 
-	bdecode_node bdecode_node::dict_find_int(char const* key) const
+	bdecode_node bdecode_node::dict_find_int(string_view key) const
 	{
 		bdecode_node ret = dict_find(key);
 		if (ret.type() == bdecode_node::int_t)
@@ -561,71 +645,35 @@ namespace libtorrent
 		return bdecode_node();
 	}
 
-
-	bdecode_node bdecode_node::dict_find(char const* key) const
-	{
-		TORRENT_ASSERT(type() == dict_t);
-
-		bdecode_token const* tokens = m_root_tokens;
-	
-		// this is the first item
-		int token = m_token_idx + 1;
-
-		while (tokens[token].type != bdecode_token::end)
-		{
-			bdecode_token const& t = tokens[token];
-			TORRENT_ASSERT(t.type == bdecode_token::string);
-			int size = m_root_tokens[token + 1].offset - t.offset - t.start_offset();
-			if (string_equal(key, m_buffer + t.offset + t.start_offset(), size))
-			{
-				// skip key
-				token += t.next_item;
-				TORRENT_ASSERT(tokens[token].type != bdecode_token::end);
-			
-				return bdecode_node(tokens, m_buffer, m_buffer_size, token);
-			}
-
-			// skip key
-			token += t.next_item;
-			TORRENT_ASSERT(tokens[token].type != bdecode_token::end);
-
-			// skip value
-			token += tokens[token].next_item;
-		}
-
-		return bdecode_node();
-	}
-
-	std::string bdecode_node::dict_find_string_value(char const* key
-		, char const* default_value) const
+	string_view bdecode_node::dict_find_string_value(string_view key
+		, string_view default_value) const
 	{
 		bdecode_node n = dict_find(key);
 		if (n.type() != bdecode_node::string_t) return default_value;
 		return n.string_value();
 	}
 
-	boost::int64_t bdecode_node::dict_find_int_value(char const* key
-		, boost::int64_t default_val) const
+	std::int64_t bdecode_node::dict_find_int_value(string_view key
+		, std::int64_t default_val) const
 	{
 		bdecode_node n = dict_find(key);
 		if (n.type() != bdecode_node::int_t) return default_val;
 		return n.int_value();
 	}
 
-	boost::int64_t bdecode_node::int_value() const
+	std::int64_t bdecode_node::int_value() const
 	{
 		TORRENT_ASSERT(type() == int_t);
 		bdecode_token const& t = m_root_tokens[m_token_idx];
-		int size = m_root_tokens[m_token_idx + 1].offset - t.offset;
+		int const size = token_source_span(t);
 		TORRENT_ASSERT(t.type == bdecode_token::integer);
 
 		// +1 is to skip the 'i'
 		char const* ptr = m_buffer + t.offset + 1;
-		boost::int64_t val = 0;
-		bool negative = false;
-		if (*ptr == '-') negative = true;
+		std::int64_t val = 0;
+		bool const negative = (*ptr == '-');
 		bdecode_errors::error_code_enum ec = bdecode_errors::no_error;
-		char const* end = parse_int(ptr + negative
+		char const* end = parse_int(ptr + int(negative)
 			, ptr + size, 'e', val, ec);
 		if (ec) return 0;
 		TORRENT_UNUSED(end);
@@ -634,14 +682,14 @@ namespace libtorrent
 		return val;
 	}
 
-	std::string bdecode_node::string_value() const
+	string_view bdecode_node::string_value() const
 	{
 		TORRENT_ASSERT(type() == string_t);
 		bdecode_token const& t = m_root_tokens[m_token_idx];
-		int size = m_root_tokens[m_token_idx + 1].offset - t.offset - t.start_offset();
+		std::size_t const size = aux::numeric_cast<std::size_t>(token_source_span(t) - t.start_offset());
 		TORRENT_ASSERT(t.type == bdecode_token::string);
 
-		return std::string(m_buffer + t.offset + t.start_offset(), size);
+		return string_view(m_buffer + t.offset + t.start_offset(), size);
 	}
 
 	char const* bdecode_node::string_ptr() const
@@ -657,11 +705,11 @@ namespace libtorrent
 		TORRENT_ASSERT(type() == string_t);
 		bdecode_token const& t = m_root_tokens[m_token_idx];
 		TORRENT_ASSERT(t.type == bdecode_token::string);
-		return m_root_tokens[m_token_idx + 1].offset - t.offset - t.start_offset();
+		return token_source_span(t) - t.start_offset();
 	}
 
 	void bdecode_node::reserve(int tokens)
-	{ m_tokens.reserve(tokens); }
+	{ m_tokens.reserve(aux::numeric_cast<std::size_t>(tokens)); }
 
 	void bdecode_node::swap(bdecode_node& n)
 	{
@@ -697,28 +745,46 @@ namespace libtorrent
 
 #define TORRENT_FAIL_BDECODE(code) do { \
 	ec = code; \
-	if (error_pos) *error_pos = start - orig_start; \
+	if (error_pos) *error_pos = int(start - orig_start); \
 	goto done; \
 	} TORRENT_WHILE_0
 
 	int bdecode(char const* start, char const* end, bdecode_node& ret
+		, error_code& ec, int* error_pos, int const depth_limit, int token_limit)
+	{
+		ret = bdecode({start, end - start}, ec, error_pos, depth_limit, token_limit);
+		return ec ? -1 : 0;
+	}
+
+	bdecode_node bdecode(span<char const> buffer, int depth_limit, int token_limit)
+	{
+		error_code ec;
+		bdecode_node ret = bdecode(buffer, ec, nullptr, depth_limit, token_limit);
+		if (ec) throw system_error(ec);
+		return ret;
+	}
+
+	bdecode_node bdecode(span<char const> buffer
 		, error_code& ec, int* error_pos, int depth_limit, int token_limit)
 	{
+		bdecode_node ret;
 		ec.clear();
-		ret.clear();
 
-		if (end - start > bdecode_token::max_offset)
+		if (buffer.size() > bdecode_token::max_offset)
 		{
 			if (error_pos) *error_pos = 0;
 			ec = bdecode_errors::limit_exceeded;
-			return -1;
+			return ret;
 		}
 
 		// this is the stack of bdecode_token indices, into m_tokens.
 		// sp is the stack pointer, as index into the array, stack
 		int sp = 0;
-		stack_frame* stack = TORRENT_ALLOCA(stack_frame, depth_limit);
+		TORRENT_ALLOCA(stack, stack_frame, depth_limit);
 
+		// TODO: 2 attempt to simplify this implementation by embracing the span
+		char const* start = buffer.data();
+		char const* end = start + buffer.size();
 		char const* const orig_start = start;
 
 		if (start == end)
@@ -736,16 +802,16 @@ namespace libtorrent
 				TORRENT_FAIL_BDECODE(bdecode_errors::limit_exceeded);
 
 			// look for a new token
-			const char t = *start;
+			char const t = *start;
 
-			const int current_frame = sp;
+			int const current_frame = sp;
 
 			// if we're currently parsing a dictionary, assert that
 			// every other node is a string.
 			if (current_frame > 0
-				&& ret.m_tokens[stack[current_frame-1].token].type == bdecode_token::dict)
+				&& ret.m_tokens[stack[current_frame - 1].token].type == bdecode_token::dict)
 			{
-				if (stack[current_frame-1].state == 0)
+				if (stack[current_frame - 1].state == 0)
 				{
 					// the current parent is a dict and we are parsing a key.
 					// only allow a digit (for a string) or 'e' to terminate
@@ -757,26 +823,24 @@ namespace libtorrent
 			switch (t)
 			{
 				case 'd':
-					stack[sp++] = ret.m_tokens.size();
+					stack[sp++] = stack_frame(int(ret.m_tokens.size()));
 					// we push it into the stack so that we know where to fill
 					// in the next_node field once we pop this node off the stack.
 					// i.e. get to the node following the dictionary in the buffer
-					ret.m_tokens.push_back(bdecode_token(start - orig_start
-						, bdecode_token::dict));
+					ret.m_tokens.push_back({start - orig_start, bdecode_token::dict});
 					++start;
 					break;
 				case 'l':
-					stack[sp++] = ret.m_tokens.size();
+					stack[sp++] = stack_frame(int(ret.m_tokens.size()));
 					// we push it into the stack so that we know where to fill
 					// in the next_node field once we pop this node off the stack.
 					// i.e. get to the node following the list in the buffer
-					ret.m_tokens.push_back(bdecode_token(start - orig_start
-						, bdecode_token::list));
+					ret.m_tokens.push_back({start - orig_start, bdecode_token::list});
 					++start;
 					break;
 				case 'i':
 				{
-					char const* int_start = start;
+					char const* const int_start = start;
 					bdecode_errors::error_code_enum e = bdecode_errors::no_error;
 					// +1 here to point to the first digit, rather than 'i'
 					start = check_integer(start + 1, end, e);
@@ -784,13 +848,13 @@ namespace libtorrent
 					{
 						// in order to gracefully terminate the tree,
 						// make sure the end of the previous token is set correctly
-						if (error_pos) *error_pos = start - orig_start;
-						error_pos = NULL;
+						if (error_pos) *error_pos = int(start - orig_start);
+						error_pos = nullptr;
 						start = int_start;
 						TORRENT_FAIL_BDECODE(e);
 					}
-					ret.m_tokens.push_back(bdecode_token(int_start - orig_start
-						, 1, bdecode_token::integer, 1));
+					ret.m_tokens.push_back({int_start - orig_start
+						, 1, bdecode_token::integer, 1});
 					TORRENT_ASSERT(*start == 'e');
 
 					// skip 'e'
@@ -804,8 +868,8 @@ namespace libtorrent
 						TORRENT_FAIL_BDECODE(bdecode_errors::unexpected_eof);
 
 					if (sp > 0
-						&& ret.m_tokens[stack[sp-1].token].type == bdecode_token::dict
-						&& stack[sp-1].state == 1)
+						&& ret.m_tokens[stack[sp - 1].token].type == bdecode_token::dict
+						&& stack[sp - 1].state == 1)
 					{
 						// this means we're parsing a dictionary and about to parse a
 						// value associated with a key. Instead, we got a termination
@@ -813,20 +877,20 @@ namespace libtorrent
 					}
 
 					// insert the end-of-sequence token
-					ret.m_tokens.push_back(bdecode_token(start - orig_start, 1
-						, bdecode_token::end));
+					ret.m_tokens.push_back({start - orig_start, 1, bdecode_token::end});
 
 					// and back-patch the start of this sequence with the offset
 					// to the next token we'll insert
-					int top = stack[sp-1].token;
+					int const top = stack[sp - 1].token;
 					// subtract the token's own index, since this is a relative
 					// offset
-					if (ret.m_tokens.size() - top > bdecode_token::max_next_item)
+					if (int(ret.m_tokens.size()) - top > bdecode_token::max_next_item)
 						TORRENT_FAIL_BDECODE(bdecode_errors::limit_exceeded);
 
-					ret.m_tokens[top].next_item = ret.m_tokens.size() - top;
+					ret.m_tokens[std::size_t(top)].next_item = std::uint32_t(int(ret.m_tokens.size()) - top);
 
 					// and pop it from the stack.
+					TORRENT_ASSERT(sp > 0);
 					--sp;
 					++start;
 					break;
@@ -838,8 +902,8 @@ namespace libtorrent
 					if (!numeric(t))
 						TORRENT_FAIL_BDECODE(bdecode_errors::expected_value);
 
-					boost::int64_t len = t - '0';
-					char const* str_start = start;
+					std::int64_t len = t - '0';
+					char const* const str_start = start;
 					++start;
 					if (start >= end) TORRENT_FAIL_BDECODE(bdecode_errors::unexpected_eof);
 					bdecode_errors::error_code_enum e = bdecode_errors::no_error;
@@ -850,7 +914,7 @@ namespace libtorrent
 						TORRENT_FAIL_BDECODE(bdecode_errors::expected_colon);
 
 					// remaining buffer size excluding ':'
-					const ptrdiff_t buff_size = end - start - 1;
+					ptrdiff_t const buff_size = end - start - 1;
 					if (len > buff_size)
 						TORRENT_FAIL_BDECODE(bdecode_errors::unexpected_eof);
 					if (len < 0)
@@ -858,25 +922,28 @@ namespace libtorrent
 
 					// skip ':'
 					++start;
-					if (start >= end) TORRENT_FAIL_BDECODE(bdecode_errors::unexpected_eof);
+					// no need to range check start here
+					// the check above ensures that the buffer is long enough to hold
+					// the string's length which guarantees that start <= end
 
 					// the bdecode_token only has 8 bits to keep the header size
 					// in. If it overflows, fail!
 					if (start - str_start - 2 > detail::bdecode_token::max_header)
 						TORRENT_FAIL_BDECODE(bdecode_errors::limit_exceeded);
 
-					ret.m_tokens.push_back(bdecode_token(str_start - orig_start
-						, 1, bdecode_token::string, start - str_start));
+					ret.m_tokens.push_back({str_start - orig_start
+						, 1, bdecode_token::string, std::uint8_t(start - str_start)});
 					start += len;
 					break;
 				}
 			}
 
 			if (current_frame > 0
-				&& ret.m_tokens[stack[current_frame-1].token].type == bdecode_token::dict)
+				&& ret.m_tokens[stack[current_frame - 1].token].type == bdecode_token::dict)
 			{
 				// the next item we parse is the opposite
-				stack[current_frame-1].state = ~stack[current_frame-1].state;
+				// state is an unsigned 1-bit member. adding 1 will flip the bit
+				stack[current_frame - 1].state = (stack[current_frame - 1].state + 1) & 1;
 			}
 
 			// this terminates the top level node, we're done!
@@ -898,27 +965,24 @@ done:
 				&& stack[sp].state == 1)
 			{
 				// insert an empty dictionary as the value
-				ret.m_tokens.push_back(bdecode_token(start - orig_start
-					, 2, bdecode_token::dict));
-				ret.m_tokens.push_back(bdecode_token(start - orig_start
-					, bdecode_token::end));
+				ret.m_tokens.push_back({start - orig_start, 2, bdecode_token::dict});
+				ret.m_tokens.push_back({start - orig_start, bdecode_token::end});
 			}
 
-			int top = stack[sp].token;
-			TORRENT_ASSERT(ret.m_tokens.size() - top <= bdecode_token::max_next_item);
-			ret.m_tokens[top].next_item = ret.m_tokens.size() - top;
-			ret.m_tokens.push_back(bdecode_token(start - orig_start, 1, bdecode_token::end));
+			int const top = stack[sp].token;
+			TORRENT_ASSERT(int(ret.m_tokens.size()) - top <= bdecode_token::max_next_item);
+			ret.m_tokens[std::size_t(top)].next_item = std::uint32_t(int(ret.m_tokens.size()) - top);
+			ret.m_tokens.push_back({start - orig_start, 1, bdecode_token::end});
 		}
 
-		ret.m_tokens.push_back(bdecode_token(start - orig_start, 0
-			, bdecode_token::end));
+		ret.m_tokens.push_back({start - orig_start, 0, bdecode_token::end});
 
 		ret.m_token_idx = 0;
 		ret.m_buffer = orig_start;
-		ret.m_buffer_size = start - orig_start;
-		ret.m_root_tokens = &ret.m_tokens[0];
+		ret.m_buffer_size = int(start - orig_start);
+		ret.m_root_tokens = ret.m_tokens.data();
 
-		return ec ? -1 : 0;
+		return ret;
 	}
 
 	namespace {
@@ -933,7 +997,7 @@ done:
 			if (line_len > limit) return -1;
 			for (int i = 0; i < e.list_size(); ++i)
 			{
-				int ret = line_longer_than(e.list_at(i), limit - line_len);
+				int const ret = line_longer_than(e.list_at(i), limit - line_len);
 				if (ret == -1) return -1;
 				line_len += ret + 2;
 			}
@@ -943,9 +1007,9 @@ done:
 			if (line_len > limit) return -1;
 			for (int i = 0; i < e.dict_size(); ++i)
 			{
-				line_len += 4 + e.dict_at(i).first.size();
+				line_len += 4 + int(e.dict_at(i).first.size());
 				if (line_len > limit) return -1;
-				int ret = line_longer_than(e.dict_at(i).second, limit - line_len);
+				int const ret = line_longer_than(e.dict_at(i).second, limit - line_len);
 				if (ret == -1) return -1;
 				line_len += ret + 1;
 			}
@@ -955,7 +1019,7 @@ done:
 			break;
 		case bdecode_node::int_t:
 		{
-			boost::int64_t val = e.int_value();
+			std::int64_t val = e.int_value();
 			while (val > 0)
 			{
 				++line_len;
@@ -973,29 +1037,13 @@ done:
 		return line_len;
 	}
 
-	void escape_string(std::string& ret, char const* str, int len)
-	{
-		for (int i = 0; i < len; ++i)
-		{
-			if (str[i] >= 32 && str[i] < 127)
-			{
-				ret += str[i];
-			}
-			else
-			{
-				char tmp[5];
-				snprintf(tmp, sizeof(tmp), "\\x%02x", boost::uint8_t(str[i]));
-				ret += tmp;
-			}
-		}
-	}
-
-	void print_string(std::string& ret, char const* str, int len, bool single_line)
+	void print_string(std::string& ret, string_view str, bool single_line)
 	{
+		int const len = int(str.size());
 		bool printable = true;
 		for (int i = 0; i < len; ++i)
 		{
-			char c = str[i];
+			char const c = str[std::size_t(i)];
 			if (c >= 32 && c < 127) continue;
 			printable = false;
 			break;
@@ -1005,24 +1053,24 @@ done:
 		{
 			if (single_line && len > 30)
 			{
-				ret.append(str, 14);
+				ret.append(str.data(), 14);
 				ret += "...";
-				ret.append(str + len-14, 14);
+				ret.append(str.data() + len - 14, 14);
 			}
 			else
-				ret.append(str, len);
+				ret.append(str.data(), std::size_t(len));
 			ret += "'";
 			return;
 		}
 		if (single_line && len > 20)
 		{
-			escape_string(ret, str, 9);
+			detail::escape_string(ret, str.data(), 9);
 			ret += "...";
-			escape_string(ret, str + len - 9, 9);
+			detail::escape_string(ret, str.data() + len - 9, 9);
 		}
 		else
 		{
-			escape_string(ret, str, len);
+			detail::escape_string(ret, str.data(), len);
 		}
 		ret += "'";
 	}
@@ -1038,7 +1086,7 @@ done:
 		indent_str[0] = ',';
 		indent_str[1] = '\n';
 		indent_str[199] = 0;
-		if (indent < 197 && indent >= 0) indent_str[indent+2] = 0;
+		if (indent < 197 && indent >= 0) indent_str[indent + 2] = 0;
 		std::string ret;
 		switch (e.type())
 		{
@@ -1046,12 +1094,12 @@ done:
 			case bdecode_node::int_t:
 			{
 				char str[100];
-				snprintf(str, sizeof(str), "%" PRId64, e.int_value());
+				std::snprintf(str, sizeof(str), "%" PRId64, e.int_value());
 				return str;
 			}
 			case bdecode_node::string_t:
 			{
-				print_string(ret, e.string_ptr(), e.string_length(), single_line);
+				print_string(ret, e.string_value(), single_line);
 				return ret;
 			}
 			case bdecode_node::list_t:
@@ -1062,35 +1110,34 @@ done:
 				if (!one_liner) ret += indent_str + 1;
 				for (int i = 0; i < e.list_size(); ++i)
 				{
-					if (i == 0 && one_liner) ret += " ";
+					if (i == 0 && one_liner) ret += ' ';
 					ret += print_entry(e.list_at(i), single_line, indent + 2);
-					if (i < e.list_size() - 1) ret += (one_liner?", ":indent_str);
-					else ret += (one_liner?" ":indent_str+1);
+					if (i < e.list_size() - 1) ret += (one_liner ? ", " : indent_str);
+					else ret += (one_liner ? " " : indent_str + 1);
 				}
-				ret += "]";
+				ret += ']';
 				return ret;
 			}
 			case bdecode_node::dict_t:
 			{
-				ret += "{";
+				ret += '{';
 				bool one_liner = line_longer_than(e, 200) != -1 || single_line;
 
-				if (!one_liner) ret += indent_str+1;
+				if (!one_liner) ret += indent_str + 1;
 				for (int i = 0; i < e.dict_size(); ++i)
 				{
-					if (i == 0 && one_liner) ret += " ";
-					std::pair<std::string, bdecode_node> ent = e.dict_at(i);
-					print_string(ret, ent.first.c_str(), ent.first.size(), true);
+					if (i == 0 && one_liner) ret += ' ';
+					std::pair<string_view, bdecode_node> ent = e.dict_at(i);
+					print_string(ret, ent.first, true);
 					ret += ": ";
 					ret += print_entry(ent.second, single_line, indent + 2);
-					if (i < e.dict_size() - 1) ret += (one_liner?", ":indent_str);
-					else ret += (one_liner?" ":indent_str+1);
+					if (i < e.dict_size() - 1) ret += (one_liner ? ", " : indent_str);
+					else ret += (one_liner ? " " : indent_str + 1);
 				}
-				ret += "}";
+				ret += '}';
 				return ret;
 			}
 		}
 		return ret;
 	}
 }
-
