@@ -31,33 +31,34 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "libtorrent/bloom_filter.hpp"
+#include "libtorrent/aux_/numeric_cast.hpp"
 
-namespace libtorrent
-{
-	bool has_bits(boost::uint8_t const* k, boost::uint8_t const* bits, int len)
+namespace libtorrent {
+
+	bool has_bits(std::uint8_t const* k, std::uint8_t const* bits, int const len)
 	{
-		boost::uint32_t idx1 = boost::uint32_t(k[0]) | (boost::uint32_t(k[1]) << 8);
-		boost::uint32_t idx2 = boost::uint32_t(k[2]) | (boost::uint32_t(k[3]) << 8);
-		idx1 %= len * 8;
-		idx2 %= len * 8;
-		return (bits[idx1/8] & (1 << (idx1 & 7))) != 0
-			&& (bits[idx2/8] & (1 << (idx2 & 7))) != 0;
+		std::uint32_t idx1 = std::uint32_t(k[0]) | (std::uint32_t(k[1]) << 8);
+		std::uint32_t idx2 = std::uint32_t(k[2]) | (std::uint32_t(k[3]) << 8);
+		idx1 %= aux::numeric_cast<std::uint32_t>(len * 8);
+		idx2 %= aux::numeric_cast<std::uint32_t>(len * 8);
+		return (bits[idx1 / 8] & (1 << (idx1 & 7))) != 0
+			&& (bits[idx2 / 8] & (1 << (idx2 & 7))) != 0;
 	}
 
-	void set_bits(boost::uint8_t const* k, boost::uint8_t* bits, int len)
+	void set_bits(std::uint8_t const* k, std::uint8_t* bits, int const len)
 	{
-		boost::uint32_t idx1 = boost::uint32_t(k[0]) | (boost::uint32_t(k[1]) << 8);
-		boost::uint32_t idx2 = boost::uint32_t(k[2]) | (boost::uint32_t(k[3]) << 8);
-		idx1 %= len * 8;
-		idx2 %= len * 8;
-		bits[idx1/8] |= (1 << (idx1 & 7));
-		bits[idx2/8] |= (1 << (idx2 & 7));
+		std::uint32_t idx1 = std::uint32_t(k[0]) | (std::uint32_t(k[1]) << 8);
+		std::uint32_t idx2 = std::uint32_t(k[2]) | (std::uint32_t(k[3]) << 8);
+		idx1 %= aux::numeric_cast<std::uint32_t>(len * 8);
+		idx2 %= aux::numeric_cast<std::uint32_t>(len * 8);
+		bits[idx1 / 8] |= (1 << (idx1 & 7));
+		bits[idx2 / 8] |= (1 << (idx2 & 7));
 	}
 
-	int count_zero_bits(boost::uint8_t const* bits, int len)
+	int count_zero_bits(std::uint8_t const* bits, int const len)
 	{
 		// number of bits _not_ set in a nibble
-		boost::uint8_t bitcount[16] =
+		std::uint8_t bitcount[16] =
 		{
 			// 0000, 0001, 0010, 0011, 0100, 0101, 0110, 0111,
 			// 1000, 1001, 1010, 1011, 1100, 1101, 1110, 1111
@@ -73,4 +74,3 @@ namespace libtorrent
 		return ret;
 	}
 }
-
