@@ -33,8 +33,8 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/disk_job_pool.hpp"
 #include "libtorrent/disk_io_job.hpp"
 
-namespace libtorrent
-{
+namespace libtorrent {
+
 	disk_job_pool::disk_job_pool()
 		: m_jobs_in_use(0)
 		, m_read_jobs(0)
@@ -48,21 +48,21 @@ namespace libtorrent
 //		TORRENT_ASSERT(m_jobs_in_use == 0);
 	}
 
-	disk_io_job* disk_job_pool::allocate_job(int type)
+	disk_io_job* disk_job_pool::allocate_job(job_action_t const type)
 	{
-		mutex::scoped_lock l(m_job_mutex);
+		std::unique_lock<std::mutex> l(m_job_mutex);
 		disk_io_job* ptr = static_cast<disk_io_job*>(m_job_pool.malloc());
 		m_job_pool.set_next_size(100);
-		if (ptr == 0) return 0;
+		if (ptr == nullptr) return nullptr;
 		++m_jobs_in_use;
-		if (type == disk_io_job::read) ++m_read_jobs;
-		else if (type == disk_io_job::write) ++m_write_jobs;
+		if (type == job_action_t::read) ++m_read_jobs;
+		else if (type == job_action_t::write) ++m_write_jobs;
 		l.unlock();
 		TORRENT_ASSERT(ptr);
 
 		new (ptr) disk_io_job;
-		ptr->action = static_cast<disk_io_job::action_t>(type);
-#if defined TORRENT_DEBUG || defined TORRENT_RELEASE_ASSERTS
+		ptr->action = type;
+#if TORRENT_USE_ASSERTS
 		ptr->in_use = true;
 #endif
 		return ptr;
@@ -71,21 +71,21 @@ namespace libtorrent
 	void disk_job_pool::free_job(disk_io_job* j)
 	{
 		TORRENT_ASSERT(j);
-		if (j == 0) return;
-#if defined TORRENT_DEBUG || defined TORRENT_RELEASE_ASSERTS
+		if (j == nullptr) return;
+#if TORRENT_USE_ASSERTS
 		TORRENT_ASSERT(j->in_use);
 		j->in_use = false;
 #endif
-		int type = j->action;
+		job_action_t const type = j->action;
 		j->~disk_io_job();
-		mutex::scoped_lock l(m_job_mutex);
-		if (type == disk_io_job::read) --m_read_jobs;
-		else if (type == disk_io_job::write) --m_write_jobs;
+		std::lock_guard<std::mutex> l(m_job_mutex);
+		if (type == job_action_t::read) --m_read_jobs;
+		else if (type == job_action_t::write) --m_write_jobs;
 		--m_jobs_in_use;
-		m_job_pool.free(j);	
+		m_job_pool.free(j);
 	}
 
-	void disk_job_pool::free_jobs(disk_io_job** j, int num)
+	void disk_job_pool::free_jobs(disk_io_job** j, int const num)
 	{
 		if (num == 0) return;
 
@@ -93,18 +93,17 @@ namespace libtorrent
 		int write_jobs = 0;
 		for (int i = 0; i < num; ++i)
 		{
-			int type = j[i]->action;
+			job_action_t const type = j[i]->action;
 			j[i]->~disk_io_job();
-			if (type == disk_io_job::read) ++read_jobs;
-			else if (type == disk_io_job::write) ++write_jobs;
+			if (type == job_action_t::read) ++read_jobs;
+			else if (type == job_action_t::write) ++write_jobs;
 		}
-	
-		mutex::scoped_lock l(m_job_mutex);
+
+		std::lock_guard<std::mutex> l(m_job_mutex);
 		m_read_jobs -= read_jobs;
 		m_write_jobs -= write_jobs;
 		m_jobs_in_use -= num;
 		for (int i = 0; i < num; ++i)
-			m_job_pool.free(j[i]);	
+			m_job_pool.free(j[i]);
 	}
 }
-
