@@ -30,20 +30,19 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
-#include <boost/version.hpp>
-
 #include "libtorrent/config.hpp"
 #include "libtorrent/error_code.hpp"
 #include "libtorrent/string_util.hpp" // for to_string()
-#include "libtorrent/aux_/escape_string.hpp" // for convert_to_native
 
-namespace libtorrent
-{
-	struct libtorrent_error_category : boost::system::error_category
+#include <sstream>
+
+namespace libtorrent {
+
+	struct libtorrent_error_category final : boost::system::error_category
 	{
-		virtual const char* name() const BOOST_SYSTEM_NOEXCEPT;
-		virtual std::string message(int ev) const BOOST_SYSTEM_NOEXCEPT;
-		virtual boost::system::error_condition default_error_condition(int ev) const BOOST_SYSTEM_NOEXCEPT
+		const char* name() const BOOST_SYSTEM_NOEXCEPT override;
+		std::string message(int ev) const override;
+		boost::system::error_condition default_error_condition(int ev) const BOOST_SYSTEM_NOEXCEPT override
 		{ return boost::system::error_condition(ev, *this); }
 	};
 
@@ -52,7 +51,7 @@ namespace libtorrent
 		return "libtorrent";
 	}
 
-	std::string libtorrent_error_category::message(int ev) const BOOST_SYSTEM_NOEXCEPT
+	std::string libtorrent_error_category::message(int ev) const
 	{
 		static char const* msgs[] =
 		{
@@ -171,8 +170,8 @@ namespace libtorrent
 			"invalid SSL certificate",
 			"not an SSL torrent",
 			"banned by port filter",
-			"",
-			"",
+			"invalid session handle used",
+			"listen socket has been closed",
 			"",
 			"",
 			"",
@@ -246,7 +245,7 @@ namespace libtorrent
 			"udp tracker response packet has invalid size",
 			"invalid transaction id in udp tracker response",
 			"invalid action field in udp tracker response",
-#ifndef TORRENT_NO_DEPRECATE
+#if TORRENT_ABI_VERSION == 1
 			"",
 			"",
 			"",
@@ -266,7 +265,13 @@ namespace libtorrent
 			"bencoded nesting depth exceeded",
 			"bencoded item count limit exceeded",
 			"integer overflow",
+			"",
+			"",
+			"",
+#else
+			"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
 #endif
+			"random number generator failed",
 		};
 		if (ev < 0 || ev >= int(sizeof(msgs)/sizeof(msgs[0])))
 			return "Unknown error";
@@ -279,23 +284,15 @@ namespace libtorrent
 		return libtorrent_category;
 	}
 
-#ifndef TORRENT_NO_DEPRECATE
-	boost::system::error_category& get_libtorrent_category()
-	{ return libtorrent_category(); }
-
-	boost::system::error_category& get_http_category()
-	{ return http_category(); }
-#endif
-
-	struct TORRENT_EXPORT http_error_category : boost::system::error_category
+	struct http_error_category final : boost::system::error_category
 	{
-		virtual const char* name() const BOOST_SYSTEM_NOEXCEPT
+		const char* name() const BOOST_SYSTEM_NOEXCEPT override
 		{ return "http"; }
-		virtual std::string message(int ev) const BOOST_SYSTEM_NOEXCEPT
+		std::string message(int ev) const override
 		{
 			std::string ret;
-			ret += to_string(ev).elems;
-			ret += " ";
+			ret += to_string(ev).data();
+			ret += ' ';
 			switch (ev)
 			{
 				case errors::cont: ret += "Continue"; break;
@@ -319,8 +316,8 @@ namespace libtorrent
 			}
 			return ret;
 		}
-		virtual boost::system::error_condition default_error_condition(
-			int ev) const BOOST_SYSTEM_NOEXCEPT
+		boost::system::error_condition default_error_condition(
+			int ev) const BOOST_SYSTEM_NOEXCEPT override
 		{ return boost::system::error_condition(ev, *this); }
 	};
 
@@ -330,24 +327,6 @@ namespace libtorrent
 		return http_category;
 	}
 
-#ifndef BOOST_NO_EXCEPTIONS
-	const char* libtorrent_exception::what() const TORRENT_EXCEPTION_THROW_SPECIFIER
-	{
-		if (!m_msg)
-		{
-			std::string msg = convert_from_native(m_error.message());
-			m_msg = allocate_string_copy(msg.c_str());
-		}
-
-		return m_msg;
-	}
-
-	libtorrent_exception::~libtorrent_exception() TORRENT_EXCEPTION_THROW_SPECIFIER
-	{
-		free(m_msg);
-	}
-#endif
-
 	namespace errors
 	{
 		// hidden
@@ -357,5 +336,13 @@ namespace libtorrent
 		}
 	}
 
-}
+	std::string print_error(error_code const& ec)
+	{
+		if (!ec) return {};
+		std::stringstream ret;
+		ret << "ERROR: (" << ec.category().name() << ":" << ec.value() << ") "
+			<< ec.message();
+		return ret.str();
+	}
 
+}
