@@ -33,58 +33,31 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/kademlia/node_entry.hpp"
 #include "libtorrent/aux_/time.hpp" // for aux::time_now()
 
-namespace libtorrent { namespace dht
-{
+namespace libtorrent { namespace dht {
 
-	node_entry::node_entry(node_id const& id_, udp::endpoint ep
+	node_entry::node_entry(node_id const& id_, udp::endpoint const& ep
 		, int roundtriptime
 		, bool pinged)
 		: last_queried(pinged ? aux::time_now() : min_time())
 		, id(id_)
-		, a(ep.address().to_v4().to_bytes())
-		, p(ep.port())
+		, endpoint(ep)
 		, rtt(roundtriptime & 0xffff)
 		, timeout_count(pinged ? 0 : 0xff)
+		, verified(verify_id(id_, ep.address()))
 	{
-#ifndef TORRENT_DISABLE_LOGGING
-		first_seen = aux::time_now();
-#endif
 	}
 
-	node_entry::node_entry(udp::endpoint ep)
-		: last_queried(min_time())
-		, id(0)
-		, a(ep.address().to_v4().to_bytes())
-		, p(ep.port())
-		, rtt(0xffff)
-		, timeout_count(0xff)
-	{
-#ifndef TORRENT_DISABLE_LOGGING
-		first_seen = aux::time_now();
-#endif
-	}
+	node_entry::node_entry(udp::endpoint const& ep)
+		: endpoint(ep)
+	{}
 
-	node_entry::node_entry()
-		: last_queried(min_time())
-		, id(0)
-		, p(0)
-		, rtt(0xffff)
-		, timeout_count(0xff)
-	{
-#ifndef TORRENT_DISABLE_LOGGING
-		first_seen = aux::time_now();
-#endif
-	}
-
-	void node_entry::update_rtt(int new_rtt)
+	void node_entry::update_rtt(int const new_rtt)
 	{
 		TORRENT_ASSERT(new_rtt <= 0xffff);
 		TORRENT_ASSERT(new_rtt >= 0);
 		if (new_rtt == 0xffff) return;
-		if (rtt == 0xffff) rtt = new_rtt;
-		else rtt = int(rtt) * 2 / 3 + int(new_rtt) / 3;
+		if (rtt == 0xffff) rtt = std::uint16_t(new_rtt);
+		else rtt = std::uint16_t(int(rtt) * 2 / 3 + new_rtt / 3);
 	}
 
 }}
-
-
