@@ -32,8 +32,8 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/aux_/merkle.hpp"
 
-namespace libtorrent
-{
+namespace libtorrent {
+
 	int merkle_get_parent(int tree_node)
 	{
 		// node 0 doesn't have a parent
