@@ -32,8 +32,7 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/proxy_base.hpp"
 
-namespace libtorrent
-{
+namespace libtorrent {
 
 	proxy_base::proxy_base(io_service& io_service)
 		: m_sock(io_service)
@@ -41,15 +40,14 @@ namespace libtorrent
 		, m_resolver(io_service)
 	{}
 
-	proxy_base::~proxy_base() {}
+	proxy_base::~proxy_base() = default;
 
-	bool proxy_base::handle_error(error_code const& e, boost::shared_ptr<handler_type> const& h)
+	bool proxy_base::handle_error(error_code const& e, handler_type const& h)
 	{
 		if (!e) return false;
-		(*h)(e);
+		h(e);
 		error_code ec;
 		close(ec);
 		return true;
 	}
 }
-
