@@ -36,14 +36,13 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/peer_connection.hpp"
 #include "libtorrent/crc32c.hpp"
 #include "libtorrent/ip_voter.hpp"
+#include "libtorrent/io.hpp" // for write_uint16
 
-#include <boost/predef/other/endian.h>
+namespace libtorrent {
 
-namespace libtorrent
-{
 	namespace {
 
-		void apply_mask(boost::uint8_t* b, boost::uint8_t const* mask, int size)
+		void apply_mask(std::uint8_t* b, std::uint8_t const* mask, int size)
 		{
 			for (int i = 0; i < size; ++i)
 			{
@@ -67,38 +66,31 @@ namespace libtorrent
 	//   the lower 64 bits are always unmasked
 	//
 	// * for IPv6 addresses, compare /32 and /48 instead of /16 and /24
-	// 
+	//
 	// * the two IP addresses that are used to calculate the rank must
 	//   always be of the same address family
 	//
 	// * all IP addresses are in network byte order when hashed
-	boost::uint32_t peer_priority(tcp::endpoint e1, tcp::endpoint e2)
+	std::uint32_t peer_priority(tcp::endpoint e1, tcp::endpoint e2)
 	{
-		TORRENT_ASSERT(e1.address().is_v4() == e2.address().is_v4());
+		TORRENT_ASSERT(is_v4(e1) == is_v4(e2));
 
 		using std::swap;
 
-		boost::uint32_t ret;
+		std::uint32_t ret;
 		if (e1.address() == e2.address())
 		{
 			if (e1.port() > e2.port())
 				swap(e1, e2);
-			boost::uint32_t p;
-#if BOOST_ENDIAN_BIG_BYTE
-			p = e1.port() << 16;
-			p |= e2.port();
-#elif BOOST_ENDIAN_LITTLE_BYTE
-			p = aux::host_to_network(e2.port()) << 16;
-			p |= aux::host_to_network(e1.port());
-#else
-#error unsupported endianness
-#endif
+			std::uint32_t p;
+			auto ptr = reinterpret_cast<char*>(&p);
+			detail::write_uint16(e1.port(), ptr);
+			detail::write_uint16(e2.port(), ptr);
 			ret = crc32c_32(p);
 		}
-#if TORRENT_USE_IPV6
-		else if (e1.address().is_v6())
+		else if (is_v6(e1))
 		{
-			static const boost::uint8_t v6mask[][8] = {
+			static const std::uint8_t v6mask[][8] = {
 				{ 0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55 },
 				{ 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x55, 0x55 },
 				{ 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff }
@@ -107,19 +99,18 @@ namespace libtorrent
 			if (e1 > e2) swap(e1, e2);
 			address_v6::bytes_type b1 = e1.address().to_v6().to_bytes();
 			address_v6::bytes_type b2 = e2.address().to_v6().to_bytes();
-			int mask = memcmp(&b1[0], &b2[0], 4) ? 0
-				: memcmp(&b1[0], &b2[0], 6) ? 1 : 2;
-			apply_mask(&b1[0], v6mask[mask], 8);
-			apply_mask(&b2[0], v6mask[mask], 8);
-			boost::uint64_t addrbuf[4];
-			memcpy(&addrbuf[0], &b1[0], 16);
-			memcpy(&addrbuf[2], &b2[0], 16);
+			int const mask = std::memcmp(b1.data(), b2.data(), 4) ? 0
+				: std::memcmp(b1.data(), b2.data(), 6) ? 1 : 2;
+			apply_mask(b1.data(), v6mask[mask], 8);
+			apply_mask(b2.data(), v6mask[mask], 8);
+			std::uint64_t addrbuf[4];
+			memcpy(&addrbuf[0], b1.data(), 16);
+			memcpy(&addrbuf[2], b2.data(), 16);
 			ret = crc32c(addrbuf, 4);
 		}
-#endif
 		else
 		{
-			static const boost::uint8_t v4mask[][4] = {
+			static const std::uint8_t v4mask[][4] = {
 				{ 0xff, 0xff, 0x55, 0x55 },
 				{ 0xff, 0xff, 0xff, 0x55 },
 				{ 0xff, 0xff, 0xff, 0xff }
@@ -132,7 +123,7 @@ namespace libtorrent
 				: memcmp(&b1[0], &b2[0], 3) ? 1 : 2;
 			apply_mask(&b1[0], v4mask[mask], 4);
 			apply_mask(&b2[0], v4mask[mask], 4);
-			boost::uint64_t addrbuf;
+			std::uint64_t addrbuf;
 			memcpy(&addrbuf, &b1[0], 4);
 			memcpy(reinterpret_cast<char*>(&addrbuf) + 4, &b2[0], 4);
 			ret = crc32c(&addrbuf, 1);
@@ -141,10 +132,11 @@ namespace libtorrent
 		return ret;
 	}
 
-	torrent_peer::torrent_peer(boost::uint16_t port_, bool conn, int src)
+	torrent_peer::torrent_peer(std::uint16_t port_, bool conn
+		, peer_source_flags_t const src)
 		: prev_amount_upload(0)
 		, prev_amount_download(0)
-		, connection(0)
+		, connection(nullptr)
 		, peer_rank(0)
 		, last_optimistically_unchoked(0)
 		, last_connected(0)
@@ -156,17 +148,15 @@ namespace libtorrent
 		, seed(false)
 		, fast_reconnects(0)
 		, trust_points(0)
-		, source(src)
-#if !defined(TORRENT_DISABLE_ENCRYPTION) && !defined(TORRENT_DISABLE_EXTENSIONS)
+		, source(static_cast<std::uint8_t>(src))
+#if !defined TORRENT_DISABLE_ENCRYPTION
 		// assume no support in order to
 		// prefer opening non-encrypted
 		// connections. If it fails, we'll
 		// retry with encryption
 		, pe_support(false)
 #endif
-#if TORRENT_USE_IPV6
 		, is_v6_addr(false)
-#endif
 #if TORRENT_USE_I2P
 		, is_i2p_addr(false)
 #endif
@@ -176,19 +166,15 @@ namespace libtorrent
 		, confirmed_supports_utp(false)
 		, supports_holepunch(false)
 		, web_seed(false)
-#if TORRENT_USE_ASSERTS
-		, in_use(false)
-#endif
-	{
-		TORRENT_ASSERT((src & 0xff) == src);
-	}
+	{}
 
-	boost::uint32_t torrent_peer::rank(external_ip const& external, int external_port) const
+	std::uint32_t torrent_peer::rank(external_ip const& external, int external_port) const
 	{
+		TORRENT_ASSERT(in_use);
 //TODO: how do we deal with our external address changing?
 		if (peer_rank == 0)
 			peer_rank = peer_priority(
-				tcp::endpoint(external.external_address(this->address()), external_port)
+				tcp::endpoint(external.external_address(this->address()), std::uint16_t(external_port))
 				, tcp::endpoint(this->address(), this->port));
 		return peer_rank;
 	}
@@ -196,75 +182,70 @@ namespace libtorrent
 #ifndef TORRENT_DISABLE_LOGGING
 	std::string torrent_peer::to_string() const
 	{
+		TORRENT_ASSERT(in_use);
 #if TORRENT_USE_I2P
-		if (is_i2p_addr) return dest();
+		if (is_i2p_addr) return dest().to_string();
 #endif // TORRENT_USE_I2P
 		error_code ec;
 		return address().to_string(ec);
 	}
 #endif
 
-	boost::uint64_t torrent_peer::total_download() const
+	std::int64_t torrent_peer::total_download() const
 	{
-		if (connection != 0)
+		TORRENT_ASSERT(in_use);
+		if (connection != nullptr)
 		{
 			TORRENT_ASSERT(prev_amount_download == 0);
 			return connection->statistics().total_payload_download();
 		}
 		else
 		{
-			return boost::uint64_t(prev_amount_download) << 10;
+			return std::int64_t(prev_amount_download) << 10;
 		}
 	}
 
-	boost::uint64_t torrent_peer::total_upload() const
+	std::int64_t torrent_peer::total_upload() const
 	{
-		if (connection != 0)
+		TORRENT_ASSERT(in_use);
+		if (connection != nullptr)
 		{
 			TORRENT_ASSERT(prev_amount_upload == 0);
 			return connection->statistics().total_payload_upload();
 		}
 		else
 		{
-			return boost::uint64_t(prev_amount_upload) << 10;
+			return std::int64_t(prev_amount_upload) << 10;
 		}
 	}
 
-	ipv4_peer::ipv4_peer(
-		tcp::endpoint const& ep, bool c, int src
-	)
+	ipv4_peer::ipv4_peer(tcp::endpoint const& ep, bool c
+		, peer_source_flags_t const src)
 		: torrent_peer(ep.port(), c, src)
 		, addr(ep.address().to_v4())
 	{
-#if TORRENT_USE_IPV6
 		is_v6_addr = false;
-#endif
 #if TORRENT_USE_I2P
 		is_i2p_addr = false;
 #endif
 	}
 
-	ipv4_peer::ipv4_peer(ipv4_peer const& p)
-		: torrent_peer(p), addr(p.addr) {}
+	ipv4_peer::ipv4_peer(ipv4_peer const&) = default;
+	ipv4_peer& ipv4_peer::operator=(ipv4_peer const& p) = default;
 
 #if TORRENT_USE_I2P
-	i2p_peer::i2p_peer(char const* dest, bool connectable, int src)
-		: torrent_peer(0, connectable, src), destination(allocate_string_copy(dest))
+	i2p_peer::i2p_peer(string_view dest, bool connectable_
+		, peer_source_flags_t const src)
+		: torrent_peer(0, connectable_, src)
+		, destination(dest)
 	{
-#if TORRENT_USE_IPV6
 		is_v6_addr = false;
-#endif
 		is_i2p_addr = true;
 	}
-
-	i2p_peer::~i2p_peer()
-	{ free(destination); }
 #endif // TORRENT_USE_I2P
 
-#if TORRENT_USE_IPV6
-	ipv6_peer::ipv6_peer(
-		tcp::endpoint const& ep, bool c, int src
-	)
+	ipv6_peer::ipv6_peer(tcp::endpoint const& ep, bool c
+		, peer_source_flags_t const src)
 		: torrent_peer(ep.port(), c, src)
 		, addr(ep.address().to_v6().to_bytes())
 	{
@@ -274,25 +255,23 @@ namespace libtorrent
 #endif
 	}
 
-#endif // TORRENT_USE_IPV6
+	ipv6_peer::ipv6_peer(ipv6_peer const&) = default;
 
 #if TORRENT_USE_I2P
-	char const* torrent_peer::dest() const
+	string_view torrent_peer::dest() const
 	{
 		if (is_i2p_addr)
-			return static_cast<i2p_peer const*>(this)->destination;
+			return *static_cast<i2p_peer const*>(this)->destination;
 		return "";
 	}
 #endif
 
 	libtorrent::address torrent_peer::address() const
 	{
-#if TORRENT_USE_IPV6
 		if (is_v6_addr)
 			return libtorrent::address_v6(
 				static_cast<ipv6_peer const*>(this)->addr);
 		else
-#endif
 #if TORRENT_USE_I2P
 		if (is_i2p_addr) return libtorrent::address();
 		else
@@ -301,4 +280,3 @@ namespace libtorrent
 	}
 
 }
-
