@@ -31,37 +31,24 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "libtorrent/config.hpp"
-#include "libtorrent/peer_connection.hpp"
 #include "libtorrent/bt_peer_connection.hpp"
 #include "libtorrent/peer_connection_handle.hpp"
 #include "libtorrent/bencode.hpp"
 #include "libtorrent/torrent.hpp"
-#include "libtorrent/torrent_handle.hpp"
 #include "libtorrent/extensions.hpp"
 #include "libtorrent/broadcast_socket.hpp"
 #include "libtorrent/socket_io.hpp"
 #include "libtorrent/peer_info.hpp"
-#include "libtorrent/random.hpp"
-#include "libtorrent/socket_type.hpp" // for is_utp
+#include "libtorrent/aux_/socket_type.hpp" // for is_utp
 #include "libtorrent/performance_counters.hpp" // for counters
-
 #include "libtorrent/extensions/ut_pex.hpp"
-
-#ifndef TORRENT_DISABLE_LOGGING
-#include "libtorrent/lazy_entry.hpp"
-#endif
+#include "libtorrent/aux_/time.hpp"
 
 #ifndef TORRENT_DISABLE_EXTENSIONS
 
-#include "libtorrent/aux_/disable_warnings_push.hpp"
+namespace libtorrent { namespace {
 
-#include <boost/shared_ptr.hpp>
-
-#include "libtorrent/aux_/disable_warnings_pop.hpp"
-
-namespace libtorrent { namespace
-{
-	static const char extension_name[] = "ut_pex";
+	const char extension_name[] = "ut_pex";
 
 	enum
 	{
@@ -73,7 +60,7 @@ namespace libtorrent { namespace
 	{
 		// don't send out those peers that we haven't connected to
 		// (that have connected to us) and that aren't sharing their
-		// listening port 
+		// listening port
 		if (!p.is_outgoing() && !p.received_listen_port()) return false;
 		// don't send out peers that we haven't successfully connected to
 		if (p.is_connecting()) return false;
@@ -81,20 +68,20 @@ namespace libtorrent { namespace
 		return true;
 	}
 
-	struct ut_pex_plugin TORRENT_FINAL
+	struct ut_pex_plugin final
 		: torrent_plugin
 	{
 		// randomize when we rebuild the pex message
 		// to evenly spread it out across all torrents
 		// the more torrents we have, the longer we can
 		// delay the rebuilding
-		ut_pex_plugin(torrent& t)
+		explicit ut_pex_plugin(torrent& t)
 			: m_torrent(t)
 			, m_last_msg(min_time())
 			, m_peers_in_message(0) {}
 
-		virtual boost::shared_ptr<peer_plugin> new_connection(
-			peer_connection_handle const& pc) TORRENT_OVERRIDE;
+		std::shared_ptr<peer_plugin> new_connection(
+			peer_connection_handle const& pc) override;
 
 		std::vector<char>& get_ut_pex_msg()
 		{
@@ -111,14 +98,15 @@ namespace libtorrent { namespace
 		// are calculated here and the pex message is created
 		// each peer connection will use this message
 		// max_peer_entries limits the packet size
-		virtual void tick() TORRENT_OVERRIDE
+		void tick() override
 		{
-			time_point now = aux::time_now();
+			if (m_torrent.flags() & torrent_flags::disable_pex) return;
+
+			time_point const now = aux::time_now();
 			if (now - seconds(60) < m_last_msg) return;
 			m_last_msg = now;
 
-			int num_peers = m_torrent.num_peers();
-			if (num_peers == 0) return;
+			if (m_torrent.num_peers() == 0) return;
 
 			entry pex;
 			std::string& pla = pex["added"].string();
@@ -127,37 +115,33 @@ namespace libtorrent { namespace
 			std::back_insert_iterator<std::string> pla_out(pla);
 			std::back_insert_iterator<std::string> pld_out(pld);
 			std::back_insert_iterator<std::string> plf_out(plf);
-#if TORRENT_USE_IPV6
 			std::string& pla6 = pex["added6"].string();
 			std::string& pld6 = pex["dropped6"].string();
 			std::string& plf6 = pex["added6.f"].string();
 			std::back_insert_iterator<std::string> pla6_out(pla6);
 			std::back_insert_iterator<std::string> pld6_out(pld6);
 			std::back_insert_iterator<std::string> plf6_out(plf6);
-#endif
 
 			std::set<tcp::endpoint> dropped;
 			m_old_peers.swap(dropped);
 
 			m_peers_in_message = 0;
 			int num_added = 0;
-			for (torrent::peer_iterator i = m_torrent.begin()
-				, end(m_torrent.end()); i != end; ++i)
+			for (auto const peer : m_torrent)
 			{
-				peer_connection* peer = *i;
 				if (!send_peer(*peer)) continue;
 
 				tcp::endpoint remote = peer->remote();
 				m_old_peers.insert(remote);
 
-				std::set<tcp::endpoint>::iterator di = dropped.find(remote);
+				auto const di = dropped.find(remote);
 				if (di == dropped.end())
 				{
 					// don't write too big of a package
 					if (num_added >= max_peer_entries) break;
 
 					// only send proper bittorrent peers
-					if (peer->type() != peer_connection::bittorrent_connection)
+					if (peer->type() != connection_type::bittorrent)
 						continue;
 
 					bt_peer_connection* p = static_cast<bt_peer_connection*>(peer);
@@ -165,9 +149,12 @@ namespace libtorrent { namespace
 					// if the peer has told us which port its listening on,
 					// use that port. But only if we didn't connect to the peer.
 					// if we connected to it, use the port we know works
-					torrent_peer *pi = 0;
-					if (!p->is_outgoing() && (pi = peer->peer_info_struct()) && pi->port > 0)
-						remote.port(pi->port);
+					if (!p->is_outgoing())
+					{
+						torrent_peer const* const pi = peer->peer_info_struct();
+						if (pi != nullptr && pi->port > 0)
+							remote.port(pi->port);
+					}
 
 					// no supported flags to set yet
 					// 0x01 - peer supports encryption
@@ -179,26 +166,24 @@ namespace libtorrent { namespace
 					//        flag is received from a peer, it can be
 					//        used as a rendezvous point in case direct
 					//        connections to the peer fail
-					int flags = p->is_seed() ? 2 : 0;
-#if !defined(TORRENT_DISABLE_ENCRYPTION) && !defined(TORRENT_DISABLE_EXTENSIONS)
-					flags |= p->supports_encryption() ? 1 : 0;
+					pex_flags_t flags = p->is_seed() ? pex_seed : pex_flags_t{};
+#if !defined TORRENT_DISABLE_ENCRYPTION
+					flags |= p->supports_encryption() ? pex_encryption : pex_flags_t{};
 #endif
-					flags |= is_utp(*p->get_socket()) ? 4 :  0;
-					flags |= p->supports_holepunch() ? 8 : 0;
+					flags |= is_utp(*p->get_socket()) ? pex_utp : pex_flags_t{};
+					flags |= p->supports_holepunch() ? pex_holepunch : pex_flags_t{};
 
 					// i->first was added since the last time
-					if (remote.address().is_v4())
+					if (is_v4(remote))
 					{
 						detail::write_endpoint(remote, pla_out);
-						detail::write_uint8(flags, plf_out);
+						detail::write_uint8(static_cast<std::uint8_t>(flags), plf_out);
 					}
-#if TORRENT_USE_IPV6
 					else
 					{
 						detail::write_endpoint(remote, pla6_out);
-						detail::write_uint8(flags, plf6_out);
+						detail::write_uint8(static_cast<std::uint8_t>(flags), plf6_out);
 					}
-#endif
 					++num_added;
 					++m_peers_in_message;
 				}
@@ -210,15 +195,12 @@ namespace libtorrent { namespace
 				}
 			}
 
-			for (std::set<tcp::endpoint>::const_iterator i = dropped.begin()
-				, end(dropped.end()); i != end; ++i)
-			{	
-				if (i->address().is_v4())
-					detail::write_endpoint(*i, pld_out);
-#if TORRENT_USE_IPV6
+			for (auto const& i : dropped)
+			{
+				if (is_v4(i))
+					detail::write_endpoint(i, pld_out);
 				else
-					detail::write_endpoint(*i, pld6_out);
-#endif
+					detail::write_endpoint(i, pld6_out);
 				++m_peers_in_message;
 			}
 
@@ -235,11 +217,11 @@ namespace libtorrent { namespace
 		int m_peers_in_message;
 
 		// explicitly disallow assignment, to silence msvc warning
-		ut_pex_plugin& operator=(ut_pex_plugin const&);
+		ut_pex_plugin& operator=(ut_pex_plugin const&) = delete;
 	};
 
-	struct ut_pex_peer_plugin TORRENT_FINAL
-		: peer_plugin
+	struct ut_pex_peer_plugin final
+		: ut_pex_peer_store, peer_plugin
 	{
 		ut_pex_peer_plugin(torrent& t, peer_connection& pc, ut_pex_plugin& tp)
 			: m_torrent(t)
@@ -249,67 +231,67 @@ namespace libtorrent { namespace
 			, m_message_index(0)
 			, m_first_time(true)
 		{
-			const int num_pex_timers = sizeof(m_last_pex)/sizeof(m_last_pex[0]);
+			const int num_pex_timers = sizeof(m_last_pex) / sizeof(m_last_pex[0]);
 			for (int i = 0; i < num_pex_timers; ++i)
 			{
-				m_last_pex[i]= min_time();
+				m_last_pex[i] = min_time();
 			}
 		}
 
-		virtual char const* type() const TORRENT_OVERRIDE { return "ut_pex"; }
-
-		virtual void add_handshake(entry& h) TORRENT_OVERRIDE
+		void add_handshake(entry& h) override
 		{
 			entry& messages = h["m"];
 			messages[extension_name] = extension_index;
 		}
 
-		virtual bool on_extension_handshake(bdecode_node const& h) TORRENT_OVERRIDE
+		bool on_extension_handshake(bdecode_node const& h) override
 		{
 			m_message_index = 0;
 			if (h.type() != bdecode_node::dict_t) return false;
-			bdecode_node messages = h.dict_find_dict("m");
+			bdecode_node const messages = h.dict_find_dict("m");
 			if (!messages) return false;
 
-			int index = int(messages.dict_find_int_value(extension_name, -1));
+			int const index = int(messages.dict_find_int_value(extension_name, -1));
 			if (index == -1) return false;
 			m_message_index = index;
 			return true;
 		}
 
-		virtual bool on_extended(int length, int msg, buffer::const_interval body) TORRENT_OVERRIDE
+		bool on_extended(int const length, int const msg, span<char const> body) override
 		{
 			if (msg != extension_index) return false;
 			if (m_message_index == 0) return false;
 
+			if (m_torrent.flags() & torrent_flags::disable_pex) return true;
+
 			if (length > 500 * 1024)
 			{
-				m_pc.disconnect(errors::pex_message_too_large, op_bittorrent, 2);
+				m_pc.disconnect(errors::pex_message_too_large, operation_t::bittorrent, peer_connection_interface::peer_error);
 				return true;
 			}
 
-			if (body.left() < length) return true;
+			if (int(body.size()) < length) return true;
 
-			time_point now = aux::time_now();
+			time_point const now = aux::time_now();
 			if (now - seconds(60) <  m_last_pex[0])
 			{
 				// this client appears to be trying to flood us
 				// with pex messages. Don't allow that.
-				m_pc.disconnect(errors::too_frequent_pex, op_bittorrent);
+				m_pc.disconnect(errors::too_frequent_pex, operation_t::bittorrent);
 				return true;
 			}
 
-			const int num_pex_timers = sizeof(m_last_pex)/sizeof(m_last_pex[0]);
-			for (int i = 0; i < num_pex_timers-1; ++i)
-				m_last_pex[i] = m_last_pex[i+1];
-			m_last_pex[num_pex_timers-1] = now;
+			int const num_pex_timers = sizeof(m_last_pex) / sizeof(m_last_pex[0]);
+			for (int i = 0; i < num_pex_timers - 1; ++i)
+				m_last_pex[i] = m_last_pex[i + 1];
+			m_last_pex[num_pex_timers - 1] = now;
 
 			bdecode_node pex_msg;
 			error_code ec;
-			int ret = bdecode(body.begin, body.end, pex_msg, ec);
+			int const ret = bdecode(body.begin(), body.end(), pex_msg, ec);
 			if (ret != 0 || pex_msg.type() != bdecode_node::dict_t)
 			{
-				m_pc.disconnect(errors::invalid_pex_message, op_bittorrent, 2);
+				m_pc.disconnect(errors::invalid_pex_message, operation_t::bittorrent, peer_connection_interface::peer_error);
 				return true;
 			}
 
@@ -318,24 +300,24 @@ namespace libtorrent { namespace
 #ifndef TORRENT_DISABLE_LOGGING
 			int num_dropped = 0;
 			int num_added = 0;
-			if (p) num_dropped += p.string_length()/6;
+			if (p) num_dropped += p.string_length() / 6;
 #endif
 			if (p)
 			{
-				int num_peers = p.string_length() / 6;
+				int const num_peers = p.string_length() / 6;
 				char const* in = p.string_ptr();
 
 				for (int i = 0; i < num_peers; ++i)
 				{
-					tcp::endpoint adr = detail::read_v4_endpoint<tcp::endpoint>(in);
-					peers4_t::value_type v(adr.address().to_v4().to_bytes(), adr.port());
-					peers4_t::iterator j = std::lower_bound(m_peers.begin(), m_peers.end(), v);
+					tcp::endpoint const adr = detail::read_v4_endpoint<tcp::endpoint>(in);
+					peers4_t::value_type const v(adr.address().to_v4().to_bytes(), adr.port());
+					auto const j = std::lower_bound(m_peers.begin(), m_peers.end(), v);
 					if (j != m_peers.end() && *j == v) m_peers.erase(j);
 				}
 			}
 
 			p = pex_msg.dict_find_string("added");
-			bdecode_node pf = pex_msg.dict_find_string("added.f");
+			bdecode_node const pf = pex_msg.dict_find_string("added.f");
 
 			bool peers_added = false;
 #ifndef TORRENT_DISABLE_LOGGING
@@ -343,14 +325,14 @@ namespace libtorrent { namespace
 #endif
 			if (p && pf && pf.string_length() == p.string_length() / 6)
 			{
-				int num_peers = pf.string_length();
+				int const num_peers = pf.string_length();
 				char const* in = p.string_ptr();
 				char const* fin = pf.string_ptr();
 
 				for (int i = 0; i < num_peers; ++i)
 				{
-					tcp::endpoint adr = detail::read_v4_endpoint<tcp::endpoint>(in);
-					char flags = *fin++;
+					tcp::endpoint const adr = detail::read_v4_endpoint<tcp::endpoint>(in);
+					pex_flags_t const flags(static_cast<std::uint8_t>(*fin++));
 
 					if (int(m_peers.size()) >= m_torrent.settings().get_int(settings_pack::max_pex_peers))
 						break;
@@ -358,8 +340,8 @@ namespace libtorrent { namespace
 					// ignore local addresses unless the peer is local to us
 					if (is_local(adr.address()) && !is_local(m_pc.remote().address())) continue;
 
-					peers4_t::value_type v(adr.address().to_v4().to_bytes(), adr.port());
-					peers4_t::iterator j = std::lower_bound(m_peers.begin(), m_peers.end(), v);
+					peers4_t::value_type const v(adr.address().to_v4().to_bytes(), adr.port());
+					auto const j = std::lower_bound(m_peers.begin(), m_peers.end(), v);
 					// do we already know about this peer?
 					if (j != m_peers.end() && *j == v) continue;
 					m_peers.insert(j, v);
@@ -368,52 +350,46 @@ namespace libtorrent { namespace
 				}
 			}
 
-#if TORRENT_USE_IPV6
-
-			bdecode_node p6 = pex_msg.dict_find("dropped6");
-#ifndef TORRENT_DISABLE_LOGGING
-			if (p6) num_dropped += p6.string_length() / 18;
-#endif
-			if (p6 != 0 && p6.type() == bdecode_node::string_t)
+			bdecode_node p6 = pex_msg.dict_find_string("dropped6");
+			if (p6)
 			{
-				int num_peers = p6.string_length() / 18;
+#ifndef TORRENT_DISABLE_LOGGING
+				num_dropped += p6.string_length() / 18;
+#endif
+				int const num_peers = p6.string_length() / 18;
 				char const* in = p6.string_ptr();
 
 				for (int i = 0; i < num_peers; ++i)
 				{
-					tcp::endpoint adr = detail::read_v6_endpoint<tcp::endpoint>(in);
-					peers6_t::value_type v(adr.address().to_v6().to_bytes(), adr.port());
-					peers6_t::iterator j = std::lower_bound(m_peers6.begin(), m_peers6.end(), v);
+					tcp::endpoint const adr = detail::read_v6_endpoint<tcp::endpoint>(in);
+					peers6_t::value_type const v(adr.address().to_v6().to_bytes(), adr.port());
+					auto const j = std::lower_bound(m_peers6.begin(), m_peers6.end(), v);
 					if (j != m_peers6.end() && *j == v) m_peers6.erase(j);
 				}
 			}
 
-			p6 = pex_msg.dict_find("added6");
+			p6 = pex_msg.dict_find_string("added6");
 #ifndef TORRENT_DISABLE_LOGGING
 			if (p6) num_added += p6.string_length() / 18;
 #endif
-			bdecode_node p6f = pex_msg.dict_find("added6.f");
-			if (p6 != 0
-				&& p6f != 0
-				&& p6.type() == bdecode_node::string_t
-				&& p6f.type() == bdecode_node::string_t
-				&& p6f.string_length() == p6.string_length() / 18)
+			bdecode_node const p6f = pex_msg.dict_find_string("added6.f");
+			if (p6 && p6f && p6f.string_length() == p6.string_length() / 18)
 			{
-				int num_peers = p6f.string_length();
+				int const num_peers = p6f.string_length();
 				char const* in = p6.string_ptr();
 				char const* fin = p6f.string_ptr();
 
 				for (int i = 0; i < num_peers; ++i)
 				{
-					tcp::endpoint adr = detail::read_v6_endpoint<tcp::endpoint>(in);
-					char flags = *fin++;
+					tcp::endpoint const adr = detail::read_v6_endpoint<tcp::endpoint>(in);
+					pex_flags_t const flags(static_cast<std::uint8_t>(*fin++));
 					// ignore local addresses unless the peer is local to us
 					if (is_local(adr.address()) && !is_local(m_pc.remote().address())) continue;
 					if (int(m_peers6.size()) >= m_torrent.settings().get_int(settings_pack::max_pex_peers))
 						break;
 
-					peers6_t::value_type v(adr.address().to_v6().to_bytes(), adr.port());
-					peers6_t::iterator j = std::lower_bound(m_peers6.begin(), m_peers6.end(), v);
+					peers6_t::value_type const v(adr.address().to_v6().to_bytes(), adr.port());
+					auto const j = std::lower_bound(m_peers6.begin(), m_peers6.end(), v);
 					// do we already know about this peer?
 					if (j != m_peers6.end() && *j == v) continue;
 					m_peers6.insert(j, v);
@@ -421,7 +397,6 @@ namespace libtorrent { namespace
 					peers_added = true;
 				}
 			}
-#endif
 #ifndef TORRENT_DISABLE_LOGGING
 			m_pc.peer_log(peer_log_alert::incoming_message, "PEX", "dropped: %d added: %d"
 				, num_dropped, num_added);
@@ -435,12 +410,12 @@ namespace libtorrent { namespace
 
 		// the peers second tick
 		// every minute we send a pex message
-		virtual void tick() TORRENT_OVERRIDE
+		void tick() override
 		{
 			// no handshake yet
 			if (!m_message_index) return;
 
-			time_point now = aux::time_now();
+			time_point const now = aux::time_now();
 			if (now - seconds(60) < m_last_msg)
 			{
 #ifndef TORRENT_DISABLE_LOGGING
@@ -451,13 +426,13 @@ namespace libtorrent { namespace
 			}
 			static time_point global_last = min_time();
 
-			int num_peers = m_torrent.num_peers();
+			int const num_peers = m_torrent.num_peers();
 			if (num_peers <= 1) return;
 
 			// don't send pex messages more often than 1 every 100 ms, and
 			// allow pex messages to be sent 5 seconds apart if there isn't
 			// contention
-			int delay = (std::min)((std::max)(60000 / num_peers, 100), 3000);
+			int const delay = std::min(std::max(60000 / num_peers, 100), 3000);
 
 			if (now - milliseconds(delay) < global_last)
 			{
@@ -490,6 +465,8 @@ namespace libtorrent { namespace
 
 		void send_ut_peer_diff()
 		{
+			if (m_torrent.flags() & torrent_flags::disable_pex) return;
+
 			// if there's no change in out peer set, don't send anything
 			if (m_tp.peers_in_msg() == 0) return;
 
@@ -498,39 +475,44 @@ namespace libtorrent { namespace
 			char msg[6];
 			char* ptr = msg;
 
-			detail::write_uint32(1 + 1 + pex_msg.size(), ptr);
+			detail::write_uint32(1 + 1 + int(pex_msg.size()), ptr);
 			detail::write_uint8(bt_peer_connection::msg_extended, ptr);
 			detail::write_uint8(m_message_index, ptr);
-			m_pc.send_buffer(msg, sizeof(msg));
-			m_pc.send_buffer(&pex_msg[0], pex_msg.size());
+			m_pc.send_buffer(msg);
+			m_pc.send_buffer(pex_msg);
 
 			m_pc.stats_counters().inc_stats_counter(counters::num_outgoing_extended);
 			m_pc.stats_counters().inc_stats_counter(counters::num_outgoing_pex);
 
 #ifndef TORRENT_DISABLE_LOGGING
-			bdecode_node m;
-			error_code ec;
-			int ret = bdecode(&pex_msg[0], &pex_msg[0] + pex_msg.size(), m, ec);
-			TORRENT_ASSERT(ret == 0);
-			TORRENT_ASSERT(!ec);
-			TORRENT_UNUSED(ret);
-			int num_dropped = 0;
-			int num_added = 0;
-			bdecode_node e = m.dict_find_string("added");
-			if (e) num_added += e.string_length() / 6;
-			e = m.dict_find_string("dropped");
-			if (e) num_dropped += e.string_length() / 6;
-			e = m.dict_find_string("added6");
-			if (e) num_added += e.string_length() / 18;
-			e = m.dict_find_string("dropped6");
-			if (e) num_dropped += e.string_length() / 18;
-			m_pc.peer_log(peer_log_alert::outgoing_message, "PEX_DIFF", "dropped: %d added: %d msg_size: %d"
-				, num_dropped, num_added, int(pex_msg.size()));
+			if (m_pc.should_log(peer_log_alert::outgoing_message))
+			{
+				bdecode_node m;
+				error_code ec;
+				int const ret = bdecode(&pex_msg[0], &pex_msg[0] + pex_msg.size(), m, ec);
+				TORRENT_ASSERT(ret == 0);
+				TORRENT_ASSERT(!ec);
+				TORRENT_UNUSED(ret);
+				int num_dropped = 0;
+				int num_added = 0;
+				bdecode_node e = m.dict_find_string("added");
+				if (e) num_added += e.string_length() / 6;
+				e = m.dict_find_string("dropped");
+				if (e) num_dropped += e.string_length() / 6;
+				e = m.dict_find_string("added6");
+				if (e) num_added += e.string_length() / 18;
+				e = m.dict_find_string("dropped6");
+				if (e) num_dropped += e.string_length() / 18;
+				m_pc.peer_log(peer_log_alert::outgoing_message, "PEX_DIFF", "dropped: %d added: %d msg_size: %d"
+					, num_dropped, num_added, int(pex_msg.size()));
+			}
 #endif
 		}
 
 		void send_ut_peer_list()
 		{
+			if (m_torrent.flags() & torrent_flags::disable_pex) return;
+
 			entry pex;
 			// leave the dropped string empty
 			pex["dropped"].string();
@@ -539,26 +521,22 @@ namespace libtorrent { namespace
 			std::back_insert_iterator<std::string> pla_out(pla);
 			std::back_insert_iterator<std::string> plf_out(plf);
 
-#if TORRENT_USE_IPV6
 			pex["dropped6"].string();
 			std::string& pla6 = pex["added6"].string();
 			std::string& plf6 = pex["added6.f"].string();
 			std::back_insert_iterator<std::string> pla6_out(pla6);
 			std::back_insert_iterator<std::string> plf6_out(plf6);
-#endif
 
 			int num_added = 0;
-			for (torrent::peer_iterator i = m_torrent.begin()
-				, end(m_torrent.end()); i != end; ++i)
+			for (auto const peer : m_torrent)
 			{
-				peer_connection* peer = *i;
 				if (!send_peer(*peer)) continue;
 
 				// don't write too big of a package
 				if (num_added >= max_peer_entries) break;
 
 				// only send proper bittorrent peers
-				if (peer->type() != peer_connection::bittorrent_connection)
+				if (peer->type() != connection_type::bittorrent)
 					continue;
 
 				bt_peer_connection* p = static_cast<bt_peer_connection*>(peer);
@@ -574,7 +552,7 @@ namespace libtorrent { namespace
 				//        used as a rendezvous point in case direct
 				//        connections to the peer fail
 				int flags = p->is_seed() ? 2 : 0;
-#if !defined(TORRENT_DISABLE_ENCRYPTION) && !defined(TORRENT_DISABLE_EXTENSIONS)
+#if !defined TORRENT_DISABLE_ENCRYPTION
 				flags |= p->supports_encryption() ? 1 : 0;
 #endif
 				flags |= is_utp(*p->get_socket()) ? 4 :  0;
@@ -582,23 +560,24 @@ namespace libtorrent { namespace
 
 				tcp::endpoint remote = peer->remote();
 
-				torrent_peer *pi = 0;
-				if (!p->is_outgoing() && (pi = peer->peer_info_struct()) && pi->port > 0)
-					remote.port(pi->port);
+				if (!p->is_outgoing())
+				{
+					torrent_peer const* const pi = peer->peer_info_struct();
+					if (pi != nullptr && pi->port > 0)
+						remote.port(pi->port);
+				}
 
 				// i->first was added since the last time
-				if (remote.address().is_v4())
+				if (is_v4(remote))
 				{
 					detail::write_endpoint(remote, pla_out);
 					detail::write_uint8(flags, plf_out);
 				}
-#if TORRENT_USE_IPV6
 				else
 				{
 					detail::write_endpoint(remote, pla6_out);
 					detail::write_uint8(flags, plf6_out);
 				}
-#endif
 				++num_added;
 			}
 			std::vector<char> pex_msg;
@@ -607,11 +586,11 @@ namespace libtorrent { namespace
 			char msg[6];
 			char* ptr = msg;
 
-			detail::write_uint32(1 + 1 + pex_msg.size(), ptr);
+			detail::write_uint32(1 + 1 + int(pex_msg.size()), ptr);
 			detail::write_uint8(bt_peer_connection::msg_extended, ptr);
 			detail::write_uint8(m_message_index, ptr);
-			m_pc.send_buffer(msg, sizeof(msg));
-			m_pc.send_buffer(&pex_msg[0], pex_msg.size());
+			m_pc.send_buffer(msg);
+			m_pc.send_buffer(pex_msg);
 
 			m_pc.stats_counters().inc_stats_counter(counters::num_outgoing_extended);
 			m_pc.stats_counters().inc_stats_counter(counters::num_outgoing_pex);
@@ -625,20 +604,7 @@ namespace libtorrent { namespace
 		torrent& m_torrent;
 		peer_connection& m_pc;
 		ut_pex_plugin& m_tp;
-		// stores all peers this this peer is connected to. These lists
-		// are updated with each pex message and are limited in size
-		// to protect against malicious clients. These lists are also
-		// used for looking up which peer a peer that supports holepunch
-		// came from.
-		// these are vectors to save memory and keep the items close
-		// together for performance. Inserting and removing is relatively
-		// cheap since the lists' size is limited
-		typedef std::vector<std::pair<address_v4::bytes_type, boost::uint16_t> > peers4_t;
-		peers4_t m_peers;
-#if TORRENT_USE_IPV6
-		typedef std::vector<std::pair<address_v6::bytes_type, boost::uint16_t> > peers6_t;
-		peers6_t m_peers6;
-#endif
+
 		// the last pex messages we received
 		// [0] is the oldest one. There is a problem with
 		// rate limited connections, because we may sit
@@ -658,55 +624,32 @@ namespace libtorrent { namespace
 		bool m_first_time;
 
 		// explicitly disallow assignment, to silence msvc warning
-		ut_pex_peer_plugin& operator=(ut_pex_peer_plugin const&);
+		ut_pex_peer_plugin& operator=(ut_pex_peer_plugin const&) = delete;
 	};
 
-	boost::shared_ptr<peer_plugin> ut_pex_plugin::new_connection(peer_connection_handle const& pc)
+	std::shared_ptr<peer_plugin> ut_pex_plugin::new_connection(peer_connection_handle const& pc)
 	{
-		if (pc.type() != peer_connection::bittorrent_connection)
-			return boost::shared_ptr<peer_plugin>();
+		if (pc.type() != connection_type::bittorrent) return {};
 
-		return boost::shared_ptr<peer_plugin>(new ut_pex_peer_plugin(m_torrent
-			, *pc.native_handle(), *this));
+		bt_peer_connection* c = static_cast<bt_peer_connection*>(pc.native_handle().get());
+		auto p = std::make_shared<ut_pex_peer_plugin>(m_torrent, *c, *this);
+		c->set_ut_pex(p);
+		return p;
 	}
 } }
 
-namespace libtorrent
-{
-	boost::shared_ptr<torrent_plugin> create_ut_pex_plugin(torrent_handle const& th, void*)
+namespace libtorrent {
+
+	std::shared_ptr<torrent_plugin> create_ut_pex_plugin(torrent_handle const& th, void*)
 	{
 		torrent* t = th.native_handle().get();
 		if (t->torrent_file().priv() || (t->torrent_file().is_i2p()
 			&& !t->settings().get_bool(settings_pack::allow_i2p_mixed)))
 		{
-			return boost::shared_ptr<torrent_plugin>();
+			return {};
 		}
-		return boost::shared_ptr<torrent_plugin>(new ut_pex_plugin(*t));
-	}
-
-	bool was_introduced_by(peer_plugin const* pp, tcp::endpoint const& ep)
-	{
-		ut_pex_peer_plugin const* p = static_cast<ut_pex_peer_plugin const*>(pp);
-#if TORRENT_USE_IPV6
-		if (ep.address().is_v4())
-		{
-#endif
-			ut_pex_peer_plugin::peers4_t::value_type v(ep.address().to_v4().to_bytes(), ep.port());
-			ut_pex_peer_plugin::peers4_t::const_iterator i
-				= std::lower_bound(p->m_peers.begin(), p->m_peers.end(), v);
-			return i != p->m_peers.end() && *i == v;
-#if TORRENT_USE_IPV6
-		}
-		else
-		{
-			ut_pex_peer_plugin::peers6_t::value_type v(ep.address().to_v6().to_bytes(), ep.port());
-			ut_pex_peer_plugin::peers6_t::const_iterator i
-				= std::lower_bound(p->m_peers6.begin(), p->m_peers6.end(), v);
-			return i != p->m_peers6.end() && *i == v;
-		}
-#endif
+		return std::make_shared<ut_pex_plugin>(*t);
 	}
 }
 
 #endif
-
