@@ -69,10 +69,22 @@ namespace libtorrent {
 		ret["num_incomplete"] = atp.num_incomplete;
 		ret["num_downloaded"] = atp.num_downloaded;
 
-		ret["sequential_download"] = bool(atp.flags & torrent_flags::sequential_download);
-
 		ret["seed_mode"] = bool(atp.flags & torrent_flags::seed_mode);
+		ret["upload_mode"] = bool(atp.flags & torrent_flags::upload_mode);
+#ifndef TORRENT_DISABLE_SHARE_MODE
+		ret["share_mode"] = bool(atp.flags & torrent_flags::share_mode);
+#endif
+		ret["apply_ip_filter"] = bool(atp.flags & torrent_flags::apply_ip_filter);
+		ret["paused"] = bool(atp.flags & torrent_flags::paused);
+		ret["auto_managed"] = bool(atp.flags & torrent_flags::auto_managed);
+#ifndef TORRENT_DISABLE_SUPERSEEDING
 		ret["super_seeding"] = bool(atp.flags & torrent_flags::super_seeding);
+#endif
+		ret["sequential_download"] = bool(atp.flags & torrent_flags::sequential_download);
+		ret["stop_when_ready"] = bool(atp.flags & torrent_flags::stop_when_ready);
+		ret["disable_dht"] = bool(atp.flags & torrent_flags::disable_dht);
+		ret["disable_lsd"] = bool(atp.flags & torrent_flags::disable_lsd);
+		ret["disable_pex"] = bool(atp.flags & torrent_flags::disable_pex);
 
 		ret["added_time"] = atp.added_time;
 		ret["completed_time"] = atp.completed_time;
@@ -216,9 +228,6 @@ namespace libtorrent {
 		ret["download_rate_limit"] = atp.download_limit;
 		ret["max_connections"] = atp.max_connections;
 		ret["max_uploads"] = atp.upload_limit;
-		ret["paused"] = bool(atp.flags & torrent_flags::paused);
-		ret["auto_managed"] = bool(atp.flags & torrent_flags::auto_managed);
-		ret["stop_when_ready"] = bool(atp.flags & torrent_flags::stop_when_ready);
 
 		if (!atp.file_priorities.empty())
 		{
