@@ -40,20 +40,26 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <exception>
 #include <sstream>
 #include <vector>
+#include <cstdio> // for std::snprintf
+#include <cinttypes> // for PRId64 et.al.
+
 #include <boost/preprocessor/cat.hpp>
 
 #include "libtorrent/config.hpp"
 
 // tests are expected to even test deprecated functionality. There is no point
 // in warning about deprecated use in any of the tests.
-
+// the unreachable code warnings are disabled since the test macros may
+// sometimes have conditions that are known at compile time
 #if defined __clang__
 #pragma clang diagnostic ignored "-Wdeprecated"
 #pragma clang diagnostic ignored "-Wdeprecated-declarations"
+#pragma clang diagnostic ignored "-Wunreachable-code"
 
 #elif defined __GNUC__
 #pragma GCC diagnostic ignored "-Wdeprecated"
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#pragma GCC diagnostic ignored "-Wunreachable-code"
 
 #elif defined _MSC_VER
 #pragma warning(disable : 4996)
@@ -70,8 +76,9 @@ POSSIBILITY OF SUCH DAMAGE.
 void EXPORT report_failure(char const* err, char const* file, int line);
 int EXPORT print_failures();
 int EXPORT test_counter();
+void EXPORT reset_output();
 
-typedef void (*unit_test_fun_t)();
+using unit_test_fun_t = void (*)();
 
 struct unit_test_t
 {
@@ -85,6 +92,7 @@ struct unit_test_t
 extern unit_test_t EXPORT _g_unit_tests[1024];
 extern int EXPORT _g_num_unit_tests;
 extern int EXPORT _g_test_failures;
+extern int _g_test_idx;
 
 #define TORRENT_TEST(test_name) \
 	static void BOOST_PP_CAT(unit_test_, test_name)(); \
@@ -95,7 +103,7 @@ extern int EXPORT _g_test_failures;
 			t.name = __FILE__ "." #test_name; \
 			t.num_failures = 0; \
 			t.run = false; \
-			t.output = NULL; \
+			t.output = nullptr; \
 			_g_num_unit_tests++; \
 		} \
 	} BOOST_PP_CAT(_static_registrar_, test_name); \
@@ -106,68 +114,91 @@ extern int EXPORT _g_test_failures;
 
 #ifdef BOOST_NO_EXCEPTIONS
 #define TEST_CHECK(x) \
-	if (!(x)) \
-		TEST_REPORT_AUX("TEST_CHECK failed: \"" #x "\"", __FILE__, __LINE__);
+	do if (!(x)) { \
+		TEST_REPORT_AUX("TEST_ERROR: check failed: \"" #x "\"", __FILE__, __LINE__);
+	} while (false)
 #define TEST_EQUAL(x, y) \
-	if ((x) != (y)) { \
+	do if ((x) != (y)) { \
 		std::stringstream s__; \
-		s__ << "TEST_EQUAL_ERROR:\n" #x ": " << (x) << "\nexpected: " << (y); \
+		s__ << "TEST_ERROR: equal check failed:\n" #x ": " << (x) << "\nexpected: " << (y); \
 		TEST_REPORT_AUX(s__.str().c_str(), __FILE__, __LINE__); \
-	}
+	} while (false)
+#define TEST_NE(x, y) \
+	do if ((x) == (y)) { \
+		std::stringstream s__; \
+		s__ << "TEST_ERROR: not equal check failed:\n" #x ": " << (x) << "\nexpected not equal to: " << (y); \
+		TEST_REPORT_AUX(s__.str().c_str(), __FILE__, __LINE__); \
+	} while (false)
 #else
 #define TEST_CHECK(x) \
-	try \
+	do try \
 	{ \
 		if (!(x)) \
-			TEST_REPORT_AUX("TEST_CHECK failed: \"" #x "\"", __FILE__, __LINE__); \
+			TEST_REPORT_AUX("TEST_ERROR: check failed: \"" #x "\"", __FILE__, __LINE__); \
 	} \
-	catch (std::exception& e) \
+	catch (std::exception const& e) \
 	{ \
-		TEST_ERROR("Exception thrown: " #x " :" + std::string(e.what())); \
+		TEST_ERROR("TEST_ERROR: Exception thrown: " #x " :" + std::string(e.what())); \
 	} \
 	catch (...) \
 	{ \
-		TEST_ERROR("Exception thrown: " #x); \
-	}
+		TEST_ERROR("TEST_ERROR: Exception thrown: " #x); \
+	} while (false)
 
 #define TEST_EQUAL(x, y) \
-	try { \
+	do try { \
 		if ((x) != (y)) { \
 			std::stringstream s__; \
-			s__ << "TEST_EQUAL_ERROR: " #x ": " << (x) << " expected: " << (y); \
+			s__ << "TEST_ERROR: " #x ": " << (x) << " expected: " << (y); \
 			TEST_REPORT_AUX(s__.str().c_str(), __FILE__, __LINE__); \
 		} \
 	} \
-	catch (std::exception& e) \
+	catch (std::exception const& e) \
 	{ \
-		TEST_ERROR("Exception thrown: " #x " :" + std::string(e.what())); \
+		TEST_ERROR("TEST_ERROR: Exception thrown: " #x " :" + std::string(e.what())); \
 	} \
 	catch (...) \
 	{ \
-		TEST_ERROR("Exception thrown: " #x); \
-	}
+		TEST_ERROR("TEST_ERROR: Exception thrown: " #x); \
+	} while (false)
+#define TEST_NE(x, y) \
+	do try { \
+		if ((x) == (y)) { \
+			std::stringstream s__; \
+			s__ << "TEST_ERROR: " #x ": " << (x) << " expected not equal to: " << (y); \
+			TEST_REPORT_AUX(s__.str().c_str(), __FILE__, __LINE__); \
+		} \
+	} \
+	catch (std::exception const& e) \
+	{ \
+		TEST_ERROR("TEST_ERROR: Exception thrown: " #x " :" + std::string(e.what())); \
+	} \
+	catch (...) \
+	{ \
+		TEST_ERROR("TEST_ERROR: Exception thrown: " #x); \
+	} while (false)
 #endif
 
 #define TEST_ERROR(x) \
-	TEST_REPORT_AUX((std::string("ERROR: \"") + (x) + "\"").c_str(), __FILE__, __LINE__)
+	TEST_REPORT_AUX((std::string("TEST_ERROR: \"") + (x) + "\"").c_str(), __FILE__, __LINE__)
 
 #define TEST_NOTHROW(x) \
-	try \
+	do try \
 	{ \
 		x; \
 	} \
 	catch (...) \
 	{ \
-		TEST_ERROR("Exception thrown: " #x); \
-	}
+		TEST_ERROR("TEST_ERROR: Exception thrown: " #x); \
+	} while (false)
 
 #define TEST_THROW(x) \
-	try \
+	do try \
 	{ \
 		x; \
 		TEST_ERROR("No exception thrown: " #x); \
 	} \
-	catch (...) {}
+	catch (...) {} while (false)
 
 #endif // TEST_HPP
 
