@@ -41,17 +41,18 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/time.hpp"
 #include "libtorrent/aux_/session_settings.hpp"
 
-#include <boost/function.hpp>
-#include <boost/bind.hpp>
-#include <boost/shared_ptr.hpp>
-#include <boost/enable_shared_from_this.hpp>
+#include <cmath>
+#include <functional>
 #include <iostream>
-#include <math.h>
+#include <utility>
 
 struct torrent;
 struct peer_connection;
 
-using namespace libtorrent;
+using namespace lt;
+using namespace std::placeholders;
+
+namespace {
 
 const float sample_time = 20.f; // seconds
 
@@ -59,7 +60,7 @@ const float sample_time = 20.f; // seconds
 
 bandwidth_channel global_bwc;
 
-struct peer_connection: bandwidth_socket, boost::enable_shared_from_this<peer_connection>
+struct peer_connection: bandwidth_socket, std::enable_shared_from_this<peer_connection>
 {
 	peer_connection(bandwidth_manager& bwm
 		, bandwidth_channel& torrent_bwc, int prio, bool ignore_limits, std::string name)
@@ -67,13 +68,12 @@ struct peer_connection: bandwidth_socket, boost::enable_shared_from_this<peer_co
 		, m_torrent_bandwidth_channel(torrent_bwc)
 		, m_priority(prio)
 		, m_ignore_limits(ignore_limits)
-		, m_name(name)
+		, m_name(std::move(name))
 		, m_quota(0)
 	{}
 
-	bool is_disconnecting() const { return false; }
-	bool ignore_bandwidth_limits() { return m_ignore_limits; }
-	void assign_bandwidth(int channel, int amount);
+	bool is_disconnecting() const override { return false; }
+	void assign_bandwidth(int channel, int amount) override;
 
 	void throttle(int limit) { m_bandwidth_channel.throttle(limit); }
 
@@ -87,14 +87,14 @@ struct peer_connection: bandwidth_socket, boost::enable_shared_from_this<peer_co
 	int m_priority;
 	bool m_ignore_limits;
 	std::string m_name;
-	int m_quota;
+	std::int64_t m_quota;
 };
 
-void peer_connection::assign_bandwidth(int channel, int amount)
+void peer_connection::assign_bandwidth(int /*channel*/, int amount)
 {
 	m_quota += amount;
 #ifdef VERBOSE_LOGGING
-	std::cerr << " [" << m_name
+	std::cout << " [" << m_name
 		<< "] assign bandwidth, " << amount << std::endl;
 #endif
 	TEST_CHECK(amount > 0);
@@ -113,7 +113,7 @@ void peer_connection::start()
 }
 
 
-typedef std::vector<boost::shared_ptr<peer_connection> > connections_t;
+using connections_t = std::vector<std::shared_ptr<peer_connection>>;
 
 void do_change_rate(bandwidth_channel& t1, bandwidth_channel& t2, int limit)
 {
@@ -137,7 +137,7 @@ void do_change_peer_rate(connections_t& v, int limit)
 	if (count == 0)
 	{
 		std::for_each(v.begin(), v.end()
-			, boost::bind(&peer_connection::throttle, _1, limit));
+			, std::bind(&peer_connection::throttle, _1, limit));
 		return;
 	}
 
@@ -150,15 +150,14 @@ static void nop() {}
 
 void run_test(connections_t& v
 	, bandwidth_manager& manager
-	, boost::function<void()> f = &nop)
+	, std::function<void()> f = &nop)
 {
-	std::cerr << "-------------" << std::endl;
-	
-	std::for_each(v.begin(), v.end()
-		, boost::bind(&peer_connection::start, _1));
+	std::cout << "-------------" << std::endl;
 
-	libtorrent::aux::session_settings s;
-	initialize_default_settings(s);
+	std::for_each(v.begin(), v.end()
+		, std::bind(&peer_connection::start, _1));
+
+	lt::aux::session_settings s;
 	int tick_interval = s.get_int(settings_pack::tick_interval);
 
 	for (int i = 0; i < int(sample_time * 1000 / tick_interval); ++i)
@@ -168,9 +167,9 @@ void run_test(connections_t& v
 	}
 }
 
-bool close_to(float val, float comp, float err)
+bool close_to(float const val, float const comp, float const err)
 {
-	return fabs(val - comp) <= err;
+	return std::abs(val - comp) <= err;
 }
 
 void spawn_connections(connections_t& v, bandwidth_manager& bwm
@@ -179,14 +178,14 @@ void spawn_connections(connections_t& v, bandwidth_manager& bwm
 	for (int i = 0; i < num; ++i)
 	{
 		char name[200];
-		snprintf(name, sizeof(name), "%s%d", prefix, i);
-		v.push_back(boost::shared_ptr<peer_connection>(new peer_connection(bwm, bwc, 200, false, name)));
+		std::snprintf(name, sizeof(name), "%s%d", prefix, i);
+		v.push_back(std::make_shared<peer_connection>(bwm, bwc, 200, false, name));
 	}
 }
 
 void test_equal_connections(int num, int limit)
 {
-	std::cerr << "\ntest equal connections " << num << " " << limit << std::endl;
+	std::cout << "\ntest equal connections " << num << " " << limit << std::endl;
 	bandwidth_manager manager(0);
 	global_bwc.throttle(limit);
 
@@ -197,25 +196,25 @@ void test_equal_connections(int num, int limit)
 	run_test(v, manager);
 
 	float sum = 0.f;
-	float err = (std::max)(limit / num * 0.3f, 1000.f);
+	float const err = std::max(limit / num * 0.3f, 1000.f);
 	for (connections_t::iterator i = v.begin()
 		, end(v.end()); i != end; ++i)
 	{
 		sum += (*i)->m_quota;
 
-		std::cerr << (*i)->m_quota / sample_time
+		std::cout << (*i)->m_quota / sample_time
 			<< " target: " << (limit / num) << " eps: " << err << std::endl;
-		TEST_CHECK(close_to((*i)->m_quota / sample_time, limit / num, err));
+		TEST_CHECK(close_to((*i)->m_quota / sample_time, float(limit) / num, err));
 	}
 	sum /= sample_time;
-	std::cerr << "sum: " << sum << " target: " << limit << std::endl;
+	std::cout << "sum: " << sum << " target: " << limit << std::endl;
 	TEST_CHECK(sum > 0);
-	TEST_CHECK(close_to(sum, limit, 50));
+	TEST_CHECK(close_to(sum, float(limit), 50));
 }
 
 void test_connections_variable_rate(int num, int limit, int torrent_limit)
 {
-	std::cerr << "\ntest connections variable rate" << num
+	std::cout << "\ntest connections variable rate" << num
 		<< " l: " << limit
 		<< " t: " << torrent_limit
 		<< std::endl;
@@ -229,14 +228,14 @@ void test_connections_variable_rate(int num, int limit, int torrent_limit)
 	connections_t v;
 	spawn_connections(v, manager, t1, num, "p");
 	std::for_each(v.begin(), v.end()
-		, boost::bind(&peer_connection::throttle, _1, limit));
+		, std::bind(&peer_connection::throttle, _1, limit));
 
-	run_test(v, manager, boost::bind(&do_change_peer_rate
-		, boost::ref(v), limit));
+	run_test(v, manager, std::bind(&do_change_peer_rate
+		, std::ref(v), limit));
 
 	if (torrent_limit > 0 && limit * num > torrent_limit)
 		limit = torrent_limit / num;
-	
+
 	float sum = 0.f;
 	float err = limit * 0.3f;
 	for (connections_t::iterator i = v.begin()
@@ -244,19 +243,19 @@ void test_connections_variable_rate(int num, int limit, int torrent_limit)
 	{
 		sum += (*i)->m_quota;
 
-		std::cerr << (*i)->m_quota / sample_time
+		std::cout << (*i)->m_quota / sample_time
 			<< " target: " << limit << " eps: " << err << std::endl;
-		TEST_CHECK(close_to((*i)->m_quota / sample_time, limit, err));
+		TEST_CHECK(close_to((*i)->m_quota / sample_time, float(limit), err));
 	}
 	sum /= sample_time;
-	std::cerr << "sum: " << sum << " target: " << (limit * num) << std::endl;
+	std::cout << "sum: " << sum << " target: " << (limit * num) << std::endl;
 	TEST_CHECK(sum > 0);
-	TEST_CHECK(close_to(sum, limit * num, limit * 0.3f * num));
+	TEST_CHECK(close_to(sum, float(limit) * num, limit * 0.3f * num));
 }
 
 void test_single_peer(int limit, bool torrent_limit)
 {
-	std::cerr << "\ntest single peer " << limit << " " << torrent_limit << std::endl;
+	std::cout << "\ntest single peer " << limit << " " << torrent_limit << std::endl;
 	bandwidth_manager manager(0);
 	bandwidth_channel t1;
 	global_bwc.throttle(0);
@@ -277,14 +276,14 @@ void test_single_peer(int limit, bool torrent_limit)
 		sum += (*i)->m_quota;
 	}
 	sum /= sample_time;
-	std::cerr << sum << " target: " << limit << std::endl;
+	std::cout << sum << " target: " << limit << std::endl;
 	TEST_CHECK(sum > 0);
-	TEST_CHECK(close_to(sum, limit, 1000));
+	TEST_CHECK(close_to(sum, float(limit), 1000));
 }
 
 void test_torrents(int num, int limit1, int limit2, int global_limit)
 {
-	std::cerr << "\ntest equal torrents " << num
+	std::cout << "\ntest equal torrents " << num
 		<< " l1: " << limit1
 		<< " l2: " << limit2
 		<< " g: " << global_limit << std::endl;
@@ -308,7 +307,7 @@ void test_torrents(int num, int limit1, int limit2, int global_limit)
 
 	if (global_limit > 0 && global_limit < limit1 + limit2)
 	{
-		limit1 = (std::min)(limit1, global_limit / 2);
+		limit1 = std::min(limit1, global_limit / 2);
 		limit2 = global_limit - limit1;
 	}
 	float sum = 0.f;
@@ -318,9 +317,9 @@ void test_torrents(int num, int limit1, int limit2, int global_limit)
 		sum += (*i)->m_quota;
 	}
 	sum /= sample_time;
-	std::cerr << sum << " target: " << limit1 << std::endl;
+	std::cout << sum << " target: " << limit1 << std::endl;
 	TEST_CHECK(sum > 0);
-	TEST_CHECK(close_to(sum, limit1, 1000));
+	TEST_CHECK(close_to(sum, float(limit1), 1000));
 
 	sum = 0.f;
 	for (connections_t::iterator i = v2.begin()
@@ -329,14 +328,14 @@ void test_torrents(int num, int limit1, int limit2, int global_limit)
 		sum += (*i)->m_quota;
 	}
 	sum /= sample_time;
-	std::cerr << sum << " target: " << limit2 << std::endl;
+	std::cout << sum << " target: " << limit2 << std::endl;
 	TEST_CHECK(sum > 0);
-	TEST_CHECK(close_to(sum, limit2, 1000));
+	TEST_CHECK(close_to(sum, float(limit2), 1000));
 }
 
 void test_torrents_variable_rate(int num, int limit, int global_limit)
 {
-	std::cerr << "\ntest torrents variable rate" << num
+	std::cout << "\ntest torrents variable rate" << num
 		<< " l: " << limit
 		<< " g: " << global_limit << std::endl;
 	bandwidth_manager manager(0);
@@ -356,7 +355,7 @@ void test_torrents_variable_rate(int num, int limit, int global_limit)
 	std::copy(v1.begin(), v1.end(), std::back_inserter(v));
 	std::copy(v2.begin(), v2.end(), std::back_inserter(v));
 
-	run_test(v, manager, boost::bind(&do_change_rate, boost::ref(t1), boost::ref(t2), limit));
+	run_test(v, manager, std::bind(&do_change_rate, std::ref(t1), std::ref(t2), limit));
 
 	if (global_limit > 0 && global_limit < 2 * limit)
 		limit = global_limit / 2;
@@ -368,9 +367,9 @@ void test_torrents_variable_rate(int num, int limit, int global_limit)
 		sum += (*i)->m_quota;
 	}
 	sum /= sample_time;
-	std::cerr << sum << " target: " << limit << std::endl;
+	std::cout << sum << " target: " << limit << std::endl;
 	TEST_CHECK(sum > 0);
-	TEST_CHECK(close_to(sum, limit, 1000));
+	TEST_CHECK(close_to(sum, float(limit), 1000));
 
 	sum = 0.f;
 	for (connections_t::iterator i = v2.begin()
@@ -379,14 +378,14 @@ void test_torrents_variable_rate(int num, int limit, int global_limit)
 		sum += (*i)->m_quota;
 	}
 	sum /= sample_time;
-	std::cerr << sum << " target: " << limit << std::endl;
+	std::cout << sum << " target: " << limit << std::endl;
 	TEST_CHECK(sum > 0);
-	TEST_CHECK(close_to(sum, limit, 1000));
+	TEST_CHECK(close_to(sum, float(limit), 1000));
 }
 
 void test_peer_priority(int limit, bool torrent_limit)
 {
-	std::cerr << "\ntest peer priority " << limit << " " << torrent_limit << std::endl;
+	std::cout << "\ntest peer priority " << limit << " " << torrent_limit << std::endl;
 	bandwidth_manager manager(0);
 	bandwidth_channel t1;
 	global_bwc.throttle(0);
@@ -400,8 +399,8 @@ void test_peer_priority(int limit, bool torrent_limit)
 	spawn_connections(v1, manager, t1, 10, "p");
 	connections_t v;
 	std::copy(v1.begin(), v1.end(), std::back_inserter(v));
-	boost::shared_ptr<peer_connection> p(
-		new peer_connection(manager, t1, 1, false, "no-priority"));
+	std::shared_ptr<peer_connection> p =
+		std::make_shared<peer_connection>(manager, t1, 1, false, "no-priority");
 	v.push_back(p);
 	run_test(v, manager);
 
@@ -412,18 +411,18 @@ void test_peer_priority(int limit, bool torrent_limit)
 		sum += (*i)->m_quota;
 	}
 	sum /= sample_time;
-	std::cerr << sum << " target: " << limit << std::endl;
+	std::cout << sum << " target: " << limit << std::endl;
 	TEST_CHECK(sum > 0);
-	TEST_CHECK(close_to(sum, limit, 50));
+	TEST_CHECK(close_to(sum, float(limit), 50));
 
-	std::cerr << "non-prioritized rate: " << p->m_quota / sample_time
+	std::cout << "non-prioritized rate: " << p->m_quota / sample_time
 		<< " target: " << (limit / 200 / 10) << std::endl;
-	TEST_CHECK(close_to(p->m_quota / sample_time, limit / 200 / 10, 5));
+	TEST_CHECK(close_to(p->m_quota / sample_time, float(limit) / 200 / 10, 5));
 }
 
 void test_no_starvation(int limit)
 {
-	std::cerr << "\ntest no starvation " << limit << std::endl;
+	std::cout << "\ntest no starvation " << limit << std::endl;
 	bandwidth_manager manager(0);
 	bandwidth_channel t1;
 	bandwidth_channel t2;
@@ -436,8 +435,8 @@ void test_no_starvation(int limit)
 	spawn_connections(v1, manager, t1, num_peers, "p");
 	connections_t v;
 	std::copy(v1.begin(), v1.end(), std::back_inserter(v));
-	boost::shared_ptr<peer_connection> p(
-		new peer_connection(manager, t2, 1, false, "no-priority"));
+	std::shared_ptr<peer_connection> p =
+		std::make_shared<peer_connection>(manager, t2, 1, false, "no-priority");
 	v.push_back(p);
 	run_test(v, manager);
 
@@ -448,47 +447,50 @@ void test_no_starvation(int limit)
 		sum += (*i)->m_quota;
 	}
 	sum /= sample_time;
-	std::cerr << sum << " target: " << limit << std::endl;
+	std::cout << sum << " target: " << limit << std::endl;
 	TEST_CHECK(sum > 0);
-	TEST_CHECK(close_to(sum, limit, 50));
+	TEST_CHECK(close_to(sum, float(limit), 50));
 
-	std::cerr << "non-prioritized rate: " << p->m_quota / sample_time
+	std::cout << "non-prioritized rate: " << p->m_quota / sample_time
 		<< " target: " << (limit / 200 / num_peers) << std::endl;
-	TEST_CHECK(close_to(p->m_quota / sample_time, limit / 200 / num_peers, 5));
+	TEST_CHECK(close_to(p->m_quota / sample_time, float(limit) / 200 / num_peers, 5));
 }
 
+} // anonymous namespace
+
 TORRENT_TEST(equal_connection)
 {
-	test_equal_connections(2, 20);
-	test_equal_connections(2, 2000);
-	test_equal_connections(2, 20000);
-	test_equal_connections(3, 20000);
-	test_equal_connections(5, 20000);
-	test_equal_connections(7, 20000);
-	test_equal_connections(33, 60000);
-	test_equal_connections(33, 500000);
-	test_equal_connections(1, 100000000);
+	test_equal_connections( 2,      20);
+	test_equal_connections( 2,    2000);
+	test_equal_connections( 2,   20000);
+	test_equal_connections( 3,   20000);
+	test_equal_connections( 5,   20000);
+	test_equal_connections( 7,   20000);
+	test_equal_connections(33,   60000);
+	test_equal_connections(33,  500000);
+	test_equal_connections( 1, 1000000);
+	test_equal_connections( 1, 6000000);
 }
 
 TORRENT_TEST(conn_var_rate)
 {
-	test_connections_variable_rate(2, 20, 0);
-	test_connections_variable_rate(5, 20000, 0);
-	test_connections_variable_rate(3, 2000, 6000);
-	test_connections_variable_rate(5, 2000, 30000);
+	test_connections_variable_rate( 2,     20, 0);
+	test_connections_variable_rate( 5,  20000, 0);
+	test_connections_variable_rate( 3,   2000, 6000);
+	test_connections_variable_rate( 5,   2000, 30000);
 	test_connections_variable_rate(33, 500000, 0);
 }
 
 TORRENT_TEST(torrents)
 {
-	test_torrents(2, 400, 400, 0);
-	test_torrents(2, 100, 500, 0);
-	test_torrents(2, 3000, 3000, 6000);
-	test_torrents(1, 40000, 40000, 0);
-	test_torrents(24, 50000, 50000, 0);
-	test_torrents(5, 6000, 6000, 3000);
-	test_torrents(5, 6000, 5000, 4000);
-	test_torrents(5, 20000, 20000, 30000);
+	test_torrents( 2,   400,   400,     0);
+	test_torrents( 2,   100,   500,     0);
+	test_torrents( 2,  3000,  3000,  6000);
+	test_torrents( 1, 40000, 40000,     0);
+	test_torrents(24, 50000, 50000,     0);
+	test_torrents( 5,  6000,  6000,  3000);
+	test_torrents( 5,  6000,  5000,  4000);
+	test_torrents( 5, 20000, 20000, 30000);
 }
 
 TORRENT_TEST(torrent_var_rate)
@@ -513,5 +515,3 @@ TORRENT_TEST(no_starvation)
 {
 	test_no_starvation(40000);
 }
-
-
