@@ -223,6 +223,14 @@ TORRENT_TEST(print_deep_dict)
 	TEST_EQUAL(e.to_string(), "{\n 'a': 'foobar',\n 'ints': [\n   1,\n   2,\n   3 ],\n 'strings': [\n   'foo',\n   'bar' ] }");
 }
 
+TORRENT_TEST(dict_constructor)
+{
+	entry::dictionary_type e{{std::string("foo"), std::string("bar")},
+		{std::string("bar"), 1234}};
+
+	TEST_EQUAL(entry(e).to_string(), "{\n 'bar': 1234,\n 'foo': 'bar' }");
+}
+
 TORRENT_TEST(integer_to_str)
 {
 	using lt::detail::integer_to_str;
