@@ -36,26 +36,26 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "test.hpp"
 #include "settings.hpp"
 #include "setup_transfer.hpp"
+#include "test_utils.hpp"
 #include "libtorrent/create_torrent.hpp"
 #include "libtorrent/alert_types.hpp"
 #include "libtorrent/torrent_info.hpp"
 #include "libtorrent/torrent_status.hpp"
+#include "libtorrent/hex.hpp" // to_hex
+#include "libtorrent/aux_/path.hpp"
+
+namespace {
 
 namespace
 {
-	const int file_sizes[] =
-	{ 0, 5, 16 - 5, 16000, 17, 10, 8000, 8000, 1,1,1,1,1,100,1,1,1,1,100,1,1,1,1,1,1
-		,1,1,1,1,1,1,13,65000,34,75,2,30,400,500,23000,900,43000,400,4300,6, 4 };
-	const int num_files = sizeof(file_sizes) / sizeof(file_sizes[0]);
-
 	bool is_checking(int const state)
 	{
 		return state == lt::torrent_status::checking_files
-#ifndef TORRENT_NO_DEPRECATE
+#if TORRENT_ABI_VERSION == 1
 			|| state == lt::torrent_status::queued_for_checking
 #endif
 			|| state == lt::torrent_status::checking_resume_data;
-	};
+	}
 }
 
 enum
@@ -74,18 +74,22 @@ enum
 	// force-recheck. Make sure the stat cache is cleared and let us pick up the
 	// new files
 	force_recheck = 8,
+
+	// files that are *bigger* than expected still considered OK. We don't
+	// truncate files willy-nilly. Checking is a read-only operation.
+	extended_files = 16,
 };
 
 void test_checking(int flags)
 {
-	using namespace libtorrent;
-	namespace lt = libtorrent;
+	using namespace lt;
 
-	fprintf(stdout, "\n==== TEST CHECKING %s%s%s%s=====\n\n"
+	std::printf("\n==== TEST CHECKING %s%s%s%s%s=====\n\n"
 		, (flags & read_only_files) ? "read-only-files ":""
 		, (flags & corrupt_files) ? "corrupt ":""
 		, (flags & incomplete_files) ? "incomplete ":""
-		, (flags & force_recheck) ? "force_recheck ":"");
+		, (flags & force_recheck) ? "force_recheck ":""
+		, (flags & extended_files) ? "extended_files ":"");
 
 	error_code ec;
 	create_directory("test_torrent_dir", ec);
@@ -96,41 +100,51 @@ void test_checking(int flags)
 	std::srand(10);
 	int piece_size = 0x4000;
 
-	create_random_files("test_torrent_dir", file_sizes, num_files, &fs);
+	static std::array<const int, 46> const file_sizes
+	{{ 0, 5, 16 - 5, 16000, 17, 10, 8000, 8000, 1,1,1,1,1,100,1,1,1,1,100,1,1,1,1,1,1
+		,1,1,1,1,1,1,13,65000,34,75,2,30,400,500,23000,900,43000,400,4300,6, 4 }};
+
+	create_random_files("test_torrent_dir", file_sizes, &fs);
 
 	lt::create_torrent t(fs, piece_size, 0x4000
 		, lt::create_torrent::optimize_alignment);
 
 	// calculate the hash for all pieces
 	set_piece_hashes(t, ".", ec);
-	if (ec) fprintf(stdout, "ERROR: set_piece_hashes: (%d) %s\n"
+	if (ec) std::printf("ERROR: set_piece_hashes: (%d) %s\n"
 		, ec.value(), ec.message().c_str());
 
 	std::vector<char> buf;
 	bencode(std::back_inserter(buf), t.generate());
-	boost::shared_ptr<torrent_info> ti(new torrent_info(&buf[0], buf.size(), ec));
+	auto ti = std::make_shared<torrent_info>(buf, ec, from_span);
 
-	fprintf(stdout, "generated torrent: %s test_torrent_dir\n"
-		, to_hex(ti->info_hash().to_string()).c_str());
+	std::printf("generated torrent: %s test_torrent_dir\n"
+		, aux::to_hex(ti->info_hash()).c_str());
 
 	// truncate every file in half
-	if (flags & incomplete_files)
+	if (flags & (incomplete_files | extended_files))
 	{
-		for (int i = 0; i < num_files; ++i)
+		for (std::size_t i = 0; i < file_sizes.size(); ++i)
 		{
 			char name[1024];
-			snprintf(name, sizeof(name), "test%d", i);
+			std::snprintf(name, sizeof(name), "test%d", int(i));
 			char dirname[200];
-			snprintf(dirname, sizeof(dirname), "test_dir%d", i / 5);
+			std::snprintf(dirname, sizeof(dirname), "test_dir%d", int(i) / 5);
 			std::string path = combine_path("test_torrent_dir", dirname);
 			path = combine_path(path, name);
 
-			error_code ec;
-			file f(path, file::read_write, ec);
-			if (ec) fprintf(stdout, "ERROR: opening file \"%s\": (%d) %s\n"
+			file f(path, open_mode::read_write, ec);
+			if (ec) std::printf("ERROR: opening file \"%s\": (%d) %s\n"
 				, path.c_str(), ec.value(), ec.message().c_str());
-			f.set_size(file_sizes[i] / 2, ec);
-			if (ec) fprintf(stdout, "ERROR: truncating file \"%s\": (%d) %s\n"
+			if (flags & extended_files)
+			{
+				f.set_size(file_sizes[i] + 10, ec);
+			}
+			else
+			{
+				f.set_size(file_sizes[i] / 2, ec);
+			}
+			if (ec) std::printf("ERROR: truncating file \"%s\": (%d) %s\n"
 				, path.c_str(), ec.value(), ec.message().c_str());
 		}
 	}
@@ -138,29 +152,29 @@ void test_checking(int flags)
 	// overwrite the files with new random data
 	if (flags & corrupt_files)
 	{
-		fprintf(stdout, "corrupt file test. overwriting files\n");
+		std::printf("corrupt file test. overwriting files\n");
 		// increase the size of some files. When they're read only that forces
 		// the checker to open them in write-mode to truncate them
-		static const int file_sizes2[] =
-		{ 0, 5, 16 - 5, 16001, 30, 10, 8000, 8000, 1,1,1,1,1,100,1,1,1,1,100,1,1,1,1,1,1
-			,1,1,1,1,1,1,13,65000,34,75,2,30,400,500,23000,900,43000,400,4300,6, 4};
-		create_random_files("test_torrent_dir", file_sizes2, num_files);
+			static std::array<const int, 46> const file_sizes2
+		{{ 0, 5, 16 - 5, 16001, 30, 10, 8000, 8000, 1,1,1,1,1,100,1,1,1,1,100,1,1,1,1,1,1
+			,1,1,1,1,1,1,13,65000,34,75,2,30,400,500,23000,900,43000,400,4300,6, 4}};
+		create_random_files("test_torrent_dir", file_sizes2);
 	}
 
 	// make the files read only
 	if (flags & read_only_files)
 	{
-		fprintf(stdout, "making files read-only\n");
-		for (int i = 0; i < num_files; ++i)
+		std::printf("making files read-only\n");
+		for (std::size_t i = 0; i < file_sizes.size(); ++i)
 		{
 			char name[1024];
-			snprintf(name, sizeof(name), "test%d", i);
+			std::snprintf(name, sizeof(name), "test%d", int(i));
 			char dirname[200];
-			snprintf(dirname, sizeof(dirname), "test_dir%d", i / 5);
+			std::snprintf(dirname, sizeof(dirname), "test_dir%d", int(i) / 5);
 
 			std::string path = combine_path("test_torrent_dir", dirname);
 			path = combine_path(path, name);
-			fprintf(stdout, "   %s\n", path.c_str());
+			std::printf("   %s\n", path.c_str());
 
 #ifdef TORRENT_WINDOWS
 			SetFileAttributesA(path.c_str(), FILE_ATTRIBUTE_READONLY);
@@ -173,10 +187,10 @@ void test_checking(int flags)
 	if (flags & force_recheck)
 	{
 		remove_all("test_torrent_dir_tmp", ec);
-		if (ec) fprintf(stdout, "ERROR: removing \"test_torrent_dir_tmp\": (%d) %s\n"
+		if (ec) std::printf("ERROR: removing \"test_torrent_dir_tmp\": (%d) %s\n"
 			, ec.value(), ec.message().c_str());
 		rename("test_torrent_dir", "test_torrent_dir_tmp", ec);
-		if (ec) fprintf(stdout, "ERROR: renaming dir \"test_torrent_dir\": (%d) %s\n"
+		if (ec) std::printf("ERROR: renaming dir \"test_torrent_dir\": (%d) %s\n"
 			, ec.value(), ec.message().c_str());
 	}
 
@@ -214,17 +228,17 @@ void test_checking(int flags)
 
 		st = tor1.status();
 
-		printf("%d %f %s\n", st.state, st.progress_ppm / 10000.f, st.errc.message().c_str());
+		std::printf("%d %f %s\n", st.state, st.progress_ppm / 10000.0, st.errc.message().c_str());
 
 		if (!is_checking(st.state) || st.errc) break;
-		test_sleep(500);
+		std::this_thread::sleep_for(lt::milliseconds(500));
 	}
 
 	if (flags & incomplete_files)
 	{
 		TEST_CHECK(!st.is_seeding);
 
-		test_sleep(500);
+		std::this_thread::sleep_for(lt::milliseconds(500));
 		st = tor1.status();
 		TEST_CHECK(!st.is_seeding);
 	}
@@ -235,25 +249,25 @@ void test_checking(int flags)
 
 		TEST_CHECK(!st.errc);
 		if (st.errc)
-			fprintf(stdout, "error: %s\n", st.errc.message().c_str());
+			std::printf("error: %s\n", st.errc.message().c_str());
 	}
 
 	if ((flags & (incomplete_files | corrupt_files)) == 0)
 	{
 		TEST_CHECK(st.is_seeding);
 		if (st.errc)
-			fprintf(stdout, "error: %s\n", st.errc.message().c_str());
+			std::printf("error: %s\n", st.errc.message().c_str());
 	}
 
 	// make the files writable again
 	if (flags & read_only_files)
 	{
-		for (int i = 0; i < num_files; ++i)
+		for (std::size_t i = 0; i < file_sizes.size(); ++i)
 		{
 			char name[1024];
-			snprintf(name, sizeof(name), "test%d", i);
+			std::snprintf(name, sizeof(name), "test%d", int(i));
 			char dirname[200];
-			snprintf(dirname, sizeof(dirname), "test_dir%d", i / 5);
+			std::snprintf(dirname, sizeof(dirname), "test_dir%d", int(i) / 5);
 
 			std::string path = combine_path("test_torrent_dir", dirname);
 			path = combine_path(path, name);
@@ -266,10 +280,12 @@ void test_checking(int flags)
 	}
 
 	remove_all("test_torrent_dir", ec);
-	if (ec) fprintf(stdout, "ERROR: removing test_torrent_dir: (%d) %s\n"
+	if (ec) std::printf("ERROR: removing test_torrent_dir: (%d) %s\n"
 		, ec.value(), ec.message().c_str());
 }
 
+} // anonymous namespace
+
 TORRENT_TEST(checking)
 {
 	test_checking(0);
@@ -290,6 +306,11 @@ TORRENT_TEST(incomplete)
 	test_checking(incomplete_files);
 }
 
+TORRENT_TEST(extended)
+{
+	test_checking(extended_files);
+}
+
 TORRENT_TEST(corrupt)
 {
 	test_checking(corrupt_files);
@@ -310,34 +331,39 @@ TORRENT_TEST(discrete_checking)
 
 	int const megabyte = 0x100000;
 	int const piece_size = 2 * megabyte;
-	int const file_sizes[] = { 9 * megabyte, 4 * megabyte };
-	int const num_files = sizeof(file_sizes) / sizeof(file_sizes[0]);
+	static std::array<int const, 2> const file_sizes{{ 9 * megabyte, 3 * megabyte }};
 
 	file_storage fs;
-	create_random_files("test_torrent_dir", file_sizes, num_files, &fs);
-	lt::create_torrent t(fs, piece_size, 0, lt::create_torrent::optimize_alignment);
+	create_random_files("test_torrent_dir", file_sizes, &fs);
+	TEST_EQUAL(fs.num_files(), 2);
+
+	lt::create_torrent t(fs, piece_size, 1, lt::create_torrent::optimize_alignment);
 	set_piece_hashes(t, ".", ec);
 	if (ec) printf("ERROR: set_piece_hashes: (%d) %s\n", ec.value(), ec.message().c_str());
 
 	std::vector<char> buf;
 	bencode(std::back_inserter(buf), t.generate());
-	boost::shared_ptr<torrent_info> ti(new torrent_info(&buf[0], buf.size(), ec));
-	printf("generated torrent: %s test_torrent_dir\n", to_hex(ti->info_hash().to_string()).c_str());
+	auto ti = std::make_shared<torrent_info>(buf, ec, from_span);
+	printf("generated torrent: %s test_torrent_dir\n", aux::to_hex(ti->info_hash().to_string()).c_str());
+
+	// we have two files, but there's a padfile now too
 	TEST_EQUAL(ti->num_files(), 3);
+
 	{
 		session ses1(settings());
 		add_torrent_params p;
-		p.file_priorities.resize(ti->num_files());
-		p.file_priorities[0] = 1;
+		p.file_priorities.resize(std::size_t(ti->num_files()));
+		p.file_priorities[0] = 1_pri;
 		p.save_path = ".";
 		p.ti = ti;
 		torrent_handle tor1 = ses1.add_torrent(p, ec);
 		// change the priority of a file while checking and make sure it doesn't interrupt the checking.
-		std::vector<int> prio(ti->num_files(), 0);
-		prio[2] = 1;
+		std::vector<download_priority_t> prio(std::size_t(ti->num_files()), 0_pri);
+		prio[2] = 1_pri;
 		tor1.prioritize_files(prio);
-		TEST_CHECK(wait_for_alert(ses1, torrent_checked_alert::alert_type, "torrent checked", 1, seconds(50)));
-		TEST_CHECK(tor1.status(0).is_seeding);
+		TEST_CHECK(wait_for_alert(ses1, torrent_checked_alert::alert_type
+			, "torrent checked", pop_alerts::pop_all, seconds(50)));
+		TEST_CHECK(tor1.status({}).is_seeding);
 	}
 	remove_all("test_torrent_dir", ec);
 	if (ec) fprintf(stdout, "ERROR: removing test_torrent_dir: (%d) %s\n", ec.value(), ec.message().c_str());
