@@ -37,25 +37,26 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/socket.hpp"
 #include "libtorrent/io.hpp"
-#include "libtorrent/alloca.hpp"
+#include "libtorrent/aux_/alloca.hpp" // for use of private TORRENT_ALLOCA
 #include "libtorrent/time.hpp"
 #include "libtorrent/peer_info.hpp"
 #include "libtorrent/bdecode.hpp"
 #include "libtorrent/bencode.hpp"
 #include "libtorrent/entry.hpp"
 #include "libtorrent/torrent_info.hpp"
-
-#include "libtorrent/aux_/disable_warnings_push.hpp"
+#include "libtorrent/aux_/path.hpp"
 
 #include <cstring>
-#include <boost/bind.hpp>
+#include <functional>
 #include <iostream>
+#include <fstream>
 #include <cstdarg>
+#include <cstdio> // for vsnprintf
 
-#include "libtorrent/aux_/disable_warnings_pop.hpp"
+using namespace lt;
+using namespace std::placeholders;
 
-using namespace libtorrent;
-namespace lt = libtorrent;
+namespace {
 
 void log(char const* fmt, ...)
 {
@@ -63,39 +64,46 @@ void log(char const* fmt, ...)
 	va_start(v, fmt);
 
 	char buf[1024];
-	vsnprintf(buf, sizeof(buf), fmt, v);
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wformat-nonliteral"
+#endif
+	std::vsnprintf(buf, sizeof(buf), fmt, v);
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
 	va_end(v);
 
-	fprintf(stderr, "\x1b[1m\x1b[36m%s: %s\x1b[0m\n"
+	std::printf("\x1b[1m\x1b[36m%s: %s\x1b[0m\n"
 		, time_now_string(), buf);
 }
 
 void print_session_log(lt::session& ses)
 {
-	print_alerts(ses, "ses", true, true);
+	print_alerts(ses, "ses", true);
 }
 
-int read_message(tcp::socket& s, char* buffer, int max_size)
+int read_message(tcp::socket& s, span<char> buffer)
 {
-	using namespace libtorrent::detail;
+	using namespace lt::detail;
 	error_code ec;
-	boost::asio::read(s, boost::asio::buffer(buffer, 4)
+	boost::asio::read(s, boost::asio::buffer(buffer.data(), 4)
 		, boost::asio::transfer_all(), ec);
 	if (ec)
 	{
 		TEST_ERROR(ec.message());
 		return -1;
 	}
-	char* ptr = buffer;
-	int length = read_int32(ptr);
-	if (length > max_size)
+	char const* ptr = buffer.data();
+	int const length = read_int32(ptr);
+	if (length > buffer.size())
 	{
 		log("message size: %d", length);
-		TEST_ERROR("message size exceeds max limt");
+		TEST_ERROR("message size exceeds max limit");
 		return -1;
 	}
 
-	boost::asio::read(s, boost::asio::buffer(buffer, length)
+	boost::asio::read(s, boost::asio::buffer(buffer.data(), std::size_t(length))
 		, boost::asio::transfer_all(), ec);
 	if (ec)
 	{
@@ -105,58 +113,59 @@ int read_message(tcp::socket& s, char* buffer, int max_size)
 	return length;
 }
 
-void print_message(char const* buffer, int len)
+void print_message(span<char const> buffer)
 {
 	char const* message_name[] = {"choke", "unchoke", "interested", "not_interested"
 		, "have", "bitfield", "request", "piece", "cancel", "dht_port", "", "", ""
 		, "suggest_piece", "have_all", "have_none", "reject_request", "allowed_fast"};
 
-	char message[50];
+	std::stringstream message;
 	char extra[300];
 	extra[0] = 0;
-	if (len == 0)
+	if (buffer.empty())
 	{
-		strcpy(message, "keepalive");
+		message << "keepalive";
 	}
 	else
 	{
-		int msg = buffer[0];
+		int const msg = buffer[0];
 		if (msg >= 0 && msg < int(sizeof(message_name)/sizeof(message_name[0])))
-			strcpy(message, message_name[msg]);
+			message << message_name[msg];
 		else if (msg == 20)
-			snprintf(message, sizeof(message), "extension msg [%d]", buffer[1]);
+			message << "extension msg [" << int(buffer[1]) << "]";
 		else
-			snprintf(message, sizeof(message), "unknown[%d]", msg);
+			message << "unknown[" << msg << "]";
 
-		if (msg == 0x6 && len == 13)
+		if (msg == 0x6 && buffer.size() == 13)
 		{
 			peer_request r;
-			const char* ptr = buffer + 1;
-			r.piece = detail::read_int32(ptr);
+			const char* ptr = buffer.data() + 1;
+			r.piece = piece_index_t(detail::read_int32(ptr));
 			r.start = detail::read_int32(ptr);
 			r.length = detail::read_int32(ptr);
-			snprintf(extra, sizeof(extra), "p: %d s: %d l: %d", r.piece, r.start, r.length);
+			std::snprintf(extra, sizeof(extra), "p: %d s: %d l: %d"
+				, static_cast<int>(r.piece), r.start, r.length);
 		}
-		else if (msg == 0x11 && len == 5)
+		else if (msg == 0x11 && buffer.size() == 5)
 		{
-			const char* ptr = buffer + 1;
+			const char* ptr = buffer.data() + 1;
 			int index = detail::read_int32(ptr);
-			snprintf(extra, sizeof(extra), "p: %d", index);
+			std::snprintf(extra, sizeof(extra), "p: %d", index);
 		}
-		else if (msg == 20 && len > 4 && buffer[1] == 0 )
+		else if (msg == 20 && buffer.size() > 4 && buffer[1] == 0 )
 		{
-			snprintf(extra, sizeof(extra), "%s"
-				, bdecode(buffer + 2, buffer + len).to_string().c_str());
+			std::snprintf(extra, sizeof(extra), "%s"
+				, print_entry(bdecode(buffer.subspan(2))).c_str());
 		}
 	}
 
-	log("<== %s %s", message, extra);
+	log("<== %s %s", message.str().c_str(), extra);
 }
 
 void send_allow_fast(tcp::socket& s, int piece)
 {
 	log("==> allow fast: %d", piece);
-	using namespace libtorrent::detail;
+	using namespace lt::detail;
 	char msg[] = "\0\0\0\x05\x11\0\0\0\0";
 	char* ptr = msg + 5;
 	write_int32(piece, ptr);
@@ -169,7 +178,7 @@ void send_allow_fast(tcp::socket& s, int piece)
 void send_suggest_piece(tcp::socket& s, int piece)
 {
 	log("==> suggest piece: %d", piece);
-	using namespace libtorrent::detail;
+	using namespace lt::detail;
 	char msg[] = "\0\0\0\x05\x0d\0\0\0\0";
 	char* ptr = msg + 5;
 	write_int32(piece, ptr);
@@ -219,15 +228,29 @@ void send_have_none(tcp::socket& s)
 	if (ec) TEST_ERROR(ec.message());
 }
 
+void send_dht_port(tcp::socket& s, int port)
+{
+	using namespace lt::detail;
+
+	log("==> dht_port");
+	char msg[] = "\0\0\0\x03\x09\0\0"; // dht_port
+	char* ptr = msg + 5;
+	write_uint16(port, ptr);
+	error_code ec;
+	boost::asio::write(s, boost::asio::buffer(msg, 7)
+		, boost::asio::transfer_all(), ec);
+	if (ec) TEST_ERROR(ec.message());
+}
+
 void send_bitfield(tcp::socket& s, char const* bits)
 {
-	using namespace libtorrent::detail;
+	using namespace lt::detail;
 
-	int num_pieces = strlen(bits);
+	int num_pieces = int(strlen(bits));
 	int packet_size = (num_pieces+7)/8 + 5;
-	char* msg = (char*)TORRENT_ALLOCA(char, packet_size);
-	memset(msg, 0, packet_size);
-	char* ptr = msg;
+	TORRENT_ALLOCA(msg, char, packet_size);
+	std::fill(msg.begin(), msg.end(), 0);
+	char* ptr = msg.data();
 	write_int32(packet_size-4, ptr);
 	write_int8(5, ptr);
 	log("==> bitfield [%s]", bits);
@@ -236,7 +259,7 @@ void send_bitfield(tcp::socket& s, char const* bits)
 		ptr[i/8] |= (bits[i] == '1' ? 1 : 0) << i % 8;
 	}
 	error_code ec;
-	boost::asio::write(s, boost::asio::buffer(msg, packet_size)
+	boost::asio::write(s, boost::asio::buffer(msg.data(), std::size_t(msg.size()))
 		, boost::asio::transfer_all(), ec);
 	if (ec) TEST_ERROR(ec.message());
 }
@@ -276,11 +299,7 @@ void do_handshake(tcp::socket& s, sha1_hash const& ih, char* buffer)
 
 	// check for extension protocol support
 	bool const lt_extension_protocol = (extensions[5] & 0x10) != 0;
-#ifndef TORRENT_DISABLE_EXTENSIONS
 	TEST_CHECK(lt_extension_protocol == true);
-#else
-	TEST_CHECK(lt_extension_protocol == false);
-#endif
 
 	// check for DHT support
 	bool const dht_support = (extensions[7] & 0x1) != 0;
@@ -305,10 +324,10 @@ void send_extension_handshake(tcp::socket& s, entry const& e)
 
 	bencode(std::back_inserter(buf), e);
 
-	using namespace libtorrent::detail;
+	using namespace lt::detail;
 
 	char* ptr = &buf[0];
-	write_uint32(buf.size() - 4, ptr);
+	write_uint32(int(buf.size()) - 4, ptr);
 	write_uint8(20, ptr);
 	write_uint8(0, ptr);
 
@@ -320,12 +339,12 @@ void send_extension_handshake(tcp::socket& s, entry const& e)
 
 void send_request(tcp::socket& s, peer_request req)
 {
-	using namespace libtorrent::detail;
+	using namespace lt::detail;
 
-	log("==> request %d (%d,%d)", req.piece, req.start, req.length);
+	log("==> request %d (%d,%d)", static_cast<int>(req.piece), req.start, req.length);
 	char msg[] = "\0\0\0\x0d\x06            "; // have_none
 	char* ptr = msg + 5;
-	write_uint32(req.piece, ptr);
+	write_uint32(static_cast<int>(req.piece), ptr);
 	write_uint32(req.start, ptr);
 	write_uint32(req.length, ptr);
 	error_code ec;
@@ -334,17 +353,18 @@ void send_request(tcp::socket& s, peer_request req)
 	if (ec) TEST_ERROR(ec.message());
 }
 
-entry read_extension_handshake(tcp::socket& s, char* recv_buffer, int size)
+entry read_extension_handshake(tcp::socket& s, span<char> recv_buffer)
 {
 	for (;;)
 	{
-		int len = read_message(s, recv_buffer, size);
+		int const len = read_message(s, recv_buffer);
 		if (len == -1)
 		{
 			TEST_ERROR("failed to read message");
 			return entry();
 		}
-		print_message(recv_buffer, len);
+		recv_buffer = recv_buffer.first(len);
+		print_message(recv_buffer);
 
 		if (len < 4) continue;
 		int msg = recv_buffer[0];
@@ -352,10 +372,11 @@ entry read_extension_handshake(tcp::socket& s, char* recv_buffer, int size)
 		int extmsg = recv_buffer[1];
 		if (extmsg != 0) continue;
 
-		return bdecode(recv_buffer + 2, recv_buffer + len);
+		return bdecode(recv_buffer.subspan(2));
 	}
 }
 
+#ifndef TORRENT_DISABLE_EXTENSIONS
 void send_ut_metadata_msg(tcp::socket& s, int ut_metadata_msg, int type, int piece)
 {
 	std::vector<char> buf;
@@ -371,10 +392,10 @@ void send_ut_metadata_msg(tcp::socket& s, int ut_metadata_msg, int type, int pie
 	e["piece"] = piece;
 	bencode(std::back_inserter(buf), e);
 
-	using namespace libtorrent::detail;
+	using namespace lt::detail;
 
 	char* ptr = &buf[0];
-	write_uint32(buf.size() - 4, ptr);
+	write_uint32(int(buf.size()) - 4, ptr);
 	write_uint8(20, ptr);
 	write_uint8(ut_metadata_msg, ptr);
 
@@ -386,61 +407,81 @@ void send_ut_metadata_msg(tcp::socket& s, int ut_metadata_msg, int type, int pie
 	if (ec) TEST_ERROR(ec.message());
 }
 
-entry read_ut_metadata_msg(tcp::socket& s, char* recv_buffer, int size)
+entry read_ut_metadata_msg(tcp::socket& s, span<char> recv_buffer)
 {
 	for (;;)
 	{
-		int len = read_message(s, recv_buffer, size);
+		int const len = read_message(s, recv_buffer);
 		if (len == -1)
 		{
 			TEST_ERROR("failed to read message");
 			return entry();
 		}
-		print_message(recv_buffer, len);
+		auto const buffer = recv_buffer.first(len);
+		print_message(buffer);
 
 		if (len < 4) continue;
-		int msg = recv_buffer[0];
+		int const msg = buffer[0];
 		if (msg != 20) continue;
-		int extmsg = recv_buffer[1];
+		int const extmsg = buffer[1];
 		if (extmsg != 1) continue;
 
-		return bdecode(recv_buffer + 2, recv_buffer + len);
+		return bdecode(buffer.subspan(2));
 	}
 }
+#endif // TORRENT_DISABLE_EXTENSIONS
 
-boost::shared_ptr<torrent_info> setup_peer(tcp::socket& s, sha1_hash& ih
-	, boost::shared_ptr<lt::session>& ses, bool incoming = true
-	, int flags = 0, torrent_handle* th = NULL)
+std::shared_ptr<torrent_info> setup_peer(tcp::socket& s, sha1_hash& ih
+	, std::shared_ptr<lt::session>& ses, bool const incoming = true
+	, bool const magnet_link = false, bool const dht = false
+	, torrent_flags_t const flags = torrent_flags_t{}
+	, torrent_handle* th = nullptr)
 {
-	boost::shared_ptr<torrent_info> t = ::create_torrent();
+	std::ofstream out_file;
+	std::ofstream* file = nullptr;
+	if (flags & torrent_flags::seed_mode)
+	{
+		error_code ec;
+		create_directory("tmp1_fast", ec);
+		out_file.open(combine_path("tmp1_fast", "temporary").c_str(), std::ios_base::trunc | std::ios_base::binary);
+		file = &out_file;
+	}
+	else
+	{
+		error_code ec;
+		remove(combine_path("tmp1_fast","temporary").c_str(), ec);
+		if (ec) log("remove(): %s", ec.message().c_str());
+	}
+
+	std::shared_ptr<torrent_info> t = ::create_torrent(file);
+	out_file.close();
 	ih = t->info_hash();
 	settings_pack sett = settings();
 	sett.set_str(settings_pack::listen_interfaces, "0.0.0.0:48900");
 	sett.set_bool(settings_pack::enable_upnp, false);
 	sett.set_bool(settings_pack::enable_natpmp, false);
 	sett.set_bool(settings_pack::enable_lsd, false);
-	sett.set_bool(settings_pack::enable_dht, false);
+	sett.set_bool(settings_pack::enable_dht, dht);
 	sett.set_int(settings_pack::in_enc_policy, settings_pack::pe_disabled);
 	sett.set_int(settings_pack::out_enc_policy, settings_pack::pe_disabled);
 	sett.set_bool(settings_pack::enable_outgoing_utp, false);
 	sett.set_bool(settings_pack::enable_incoming_utp, false);
-#ifndef TORRENT_NO_DEPRECATE
+#if TORRENT_ABI_VERSION == 1
 	sett.set_bool(settings_pack::rate_limit_utp, true);
 #endif
 	ses.reset(new lt::session(sett, lt::session::add_default_plugins));
 
-	error_code ec;
 	add_torrent_params p;
-	p.flags &= ~add_torrent_params::flag_paused;
-	p.flags &= ~add_torrent_params::flag_auto_managed;
+	p.flags &= ~torrent_flags::paused;
+	p.flags &= ~torrent_flags::auto_managed;
 	p.flags |= flags;
-	p.ti = t;
-	p.save_path = "./tmp1_fast";
+	if (magnet_link)
+		p.info_hash = ih;
+	else
+		p.ti = t;
+	p.save_path = "tmp1_fast";
 
-	remove("./tmp1_fast/temporary", ec);
-	if (ec) log("remove(): %s", ec.message().c_str());
-	ec.clear();
-	torrent_handle ret = ses->add_torrent(p, ec);
+	torrent_handle ret = ses->add_torrent(p);
 	if (th) *th = ret;
 
 	// wait for the torrent to be ready
@@ -448,6 +489,7 @@ boost::shared_ptr<torrent_info> setup_peer(tcp::socket& s, sha1_hash& ih
 
 	if (incoming)
 	{
+		error_code ec;
 		s.connect(tcp::endpoint(address::from_string("127.0.0.1", ec), ses->listen_port()), ec);
 		if (ec) TEST_ERROR(ec.message());
 	}
@@ -455,8 +497,7 @@ boost::shared_ptr<torrent_info> setup_peer(tcp::socket& s, sha1_hash& ih
 	{
 		tcp::acceptor l(lt::get_io_service(s));
 		l.open(tcp::v4());
-		l.bind(tcp::endpoint(address_v4::from_string("127.0.0.1")
-			, 3000 + rand() % 60000));
+		l.bind(tcp::endpoint(address_v4::from_string("127.0.0.1"), 0));
 		l.listen();
 		tcp::endpoint addr = l.local_endpoint();
 
@@ -470,14 +511,16 @@ boost::shared_ptr<torrent_info> setup_peer(tcp::socket& s, sha1_hash& ih
 	return t;
 }
 
+} // anonymous namespace
+
 // makes sure that pieces that are allowed and then
 // rejected aren't requested again
 TORRENT_TEST(reject_fast)
 {
-	std::cerr << "\n === test reject ===\n" << std::endl;
+	std::cout << "\n === test reject ===\n" << std::endl;
 
 	sha1_hash ih;
-	boost::shared_ptr<lt::session> ses;
+	std::shared_ptr<lt::session> ses;
 	io_service ios;
 	tcp::socket s(ios);
 	setup_peer(s, ih, ses);
@@ -495,21 +538,22 @@ TORRENT_TEST(reject_fast)
 	allowed_fast.push_back(3);
 
 	std::for_each(allowed_fast.begin(), allowed_fast.end()
-		, boost::bind(&send_allow_fast, boost::ref(s), _1));
+		, std::bind(&send_allow_fast, std::ref(s), _1));
 	print_session_log(*ses);
 
 	while (!allowed_fast.empty())
 	{
 		print_session_log(*ses);
-		int len = read_message(s, recv_buffer, sizeof(recv_buffer));
+		int const len = read_message(s, recv_buffer);
 		if (len == -1) break;
-		print_message(recv_buffer, len);
-		int msg = recv_buffer[0];
+		auto buffer = span<char const>(recv_buffer).first(len);
+		print_message(buffer);
+		int msg = buffer[0];
 		if (msg != 0x6) continue;
 
-		using namespace libtorrent::detail;
-		char* ptr = recv_buffer + 1;
-		int piece = read_int32(ptr);
+		using namespace lt::detail;
+		char const* ptr = buffer.data() + 1;
+		int const piece = read_int32(ptr);
 
 		std::vector<int>::iterator i = std::find(allowed_fast.begin()
 			, allowed_fast.end(), piece);
@@ -537,16 +581,16 @@ TORRENT_TEST(reject_fast)
 	}
 	print_session_log(*ses);
 	s.close();
-	test_sleep(500);
+	std::this_thread::sleep_for(lt::milliseconds(500));
 	print_session_log(*ses);
 }
 
 TORRENT_TEST(invalid_suggest)
 {
-	std::cerr << "\n === test suggest ===\n" << std::endl;
+	std::cout << "\n === test suggest ===\n" << std::endl;
 
 	sha1_hash ih;
-	boost::shared_ptr<lt::session> ses;
+	std::shared_ptr<lt::session> ses;
 	io_service ios;
 	tcp::socket s(ios);
 	setup_peer(s, ih, ses);
@@ -561,20 +605,22 @@ TORRENT_TEST(invalid_suggest)
 	// request for that piece index.
 	send_suggest_piece(s, -234);
 	send_unchoke(s);
-	test_sleep(500);
+	std::this_thread::sleep_for(lt::milliseconds(500));
 	print_session_log(*ses);
 
-	int len = read_message(s, recv_buffer, sizeof(recv_buffer));
+	int len = read_message(s, recv_buffer);
+	auto buffer = span<char const>(recv_buffer).first(len);
 	int idx = -1;
 	while (len > 0)
 	{
-		if (recv_buffer[0] == 6)
+		if (buffer[0] == 6)
 		{
-			char* ptr = recv_buffer + 1;
+			char const* ptr = buffer.data() + 1;
 			idx = detail::read_int32(ptr);
 			break;
 		}
-		len = read_message(s, recv_buffer, sizeof(recv_buffer));
+		len = read_message(s, recv_buffer);
+		buffer = span<char const>(recv_buffer).first(len);
 	}
 	TEST_CHECK(idx != -234);
 	TEST_CHECK(idx != -1);
@@ -583,10 +629,10 @@ TORRENT_TEST(invalid_suggest)
 
 TORRENT_TEST(reject_suggest)
 {
-	std::cerr << "\n === test suggest ===\n" << std::endl;
+	std::cout << "\n === test suggest ===\n" << std::endl;
 
 	sha1_hash ih;
-	boost::shared_ptr<lt::session> ses;
+	std::shared_ptr<lt::session> ses;
 	io_service ios;
 	tcp::socket s(ios);
 	setup_peer(s, ih, ses);
@@ -604,7 +650,7 @@ TORRENT_TEST(reject_suggest)
 	suggested.push_back(3);
 
 	std::for_each(suggested.begin(), suggested.end()
-		, boost::bind(&send_suggest_piece, boost::ref(s), _1));
+		, std::bind(&send_suggest_piece, std::ref(s), _1));
 	print_session_log(*ses);
 
 	send_unchoke(s);
@@ -617,15 +663,16 @@ TORRENT_TEST(reject_suggest)
 	while (!suggested.empty() && fail_counter > 0)
 	{
 		print_session_log(*ses);
-		int len = read_message(s, recv_buffer, sizeof(recv_buffer));
+		int const len = read_message(s, recv_buffer);
 		if (len == -1) break;
-		print_message(recv_buffer, len);
-		int msg = recv_buffer[0];
+		auto buffer = span<char const>(recv_buffer).first(len);
+		print_message(buffer);
+		int const msg = buffer[0];
 		fail_counter--;
 		if (msg != 0x6) continue;
 
-		using namespace libtorrent::detail;
-		char* ptr = recv_buffer + 1;
+		using namespace lt::detail;
+		char const* ptr = buffer.data() + 1;
 		int const piece = read_int32(ptr);
 
 		std::vector<int>::iterator i = std::find(suggested.begin()
@@ -656,16 +703,16 @@ TORRENT_TEST(reject_suggest)
 	TEST_CHECK(fail_counter > 0);
 
 	s.close();
-	test_sleep(500);
+	std::this_thread::sleep_for(lt::milliseconds(500));
 	print_session_log(*ses);
 }
 
 TORRENT_TEST(suggest_order)
 {
-	std::cerr << "\n === test suggest ===\n" << std::endl;
+	std::cout << "\n === test suggest ===\n" << std::endl;
 
 	sha1_hash ih;
-	boost::shared_ptr<lt::session> ses;
+	std::shared_ptr<lt::session> ses;
 	io_service ios;
 	tcp::socket s(ios);
 	setup_peer(s, ih, ses);
@@ -683,7 +730,7 @@ TORRENT_TEST(suggest_order)
 	suggested.push_back(3);
 
 	std::for_each(suggested.begin(), suggested.end()
-		, boost::bind(&send_suggest_piece, boost::ref(s), _1));
+		, std::bind(&send_suggest_piece, std::ref(s), _1));
 	print_session_log(*ses);
 
 	send_unchoke(s);
@@ -693,17 +740,18 @@ TORRENT_TEST(suggest_order)
 	while (!suggested.empty() && fail_counter > 0)
 	{
 		print_session_log(*ses);
-		int len = read_message(s, recv_buffer, sizeof(recv_buffer));
+		int const len = read_message(s, recv_buffer);
 		if (len == -1) break;
-		print_message(recv_buffer, len);
-		int msg = recv_buffer[0];
+		auto const buffer = span<char const>(recv_buffer).first(len);
+		print_message({recv_buffer, len});
+		int const msg = recv_buffer[0];
 		fail_counter--;
 
 		// we're just interested in requests
 		if (msg != 0x6) continue;
 
-		using namespace libtorrent::detail;
-		char* ptr = recv_buffer + 1;
+		using namespace lt::detail;
+		char const* ptr = buffer.data() + 1;
 		int const piece = read_int32(ptr);
 
 		// make sure we receive the requests inverse order of sending the suggest
@@ -716,19 +764,19 @@ TORRENT_TEST(suggest_order)
 	TEST_CHECK(fail_counter > 0);
 
 	s.close();
-	test_sleep(500);
+	std::this_thread::sleep_for(lt::milliseconds(500));
 	print_session_log(*ses);
 }
 
 TORRENT_TEST(multiple_bitfields)
 {
-	std::cerr << "\n === test multiple bitfields ===\n" << std::endl;
+	std::cout << "\n === test multiple bitfields ===\n" << std::endl;
 
 	sha1_hash ih;
-	boost::shared_ptr<lt::session> ses;
+	std::shared_ptr<lt::session> ses;
 	io_service ios;
 	tcp::socket s(ios);
-	boost::shared_ptr<torrent_info> ti = setup_peer(s, ih, ses);
+	std::shared_ptr<torrent_info> ti = setup_peer(s, ih, ses);
 	print_session_log(*ses);
 
 	char recv_buffer[1000];
@@ -736,7 +784,7 @@ TORRENT_TEST(multiple_bitfields)
 	print_session_log(*ses);
 
 	std::string bitfield;
-	bitfield.resize(ti->num_pieces(), '0');
+	bitfield.resize(std::size_t(ti->num_pieces()), '0');
 	send_bitfield(s, bitfield.c_str());
 	print_session_log(*ses);
 	bitfield[0] = '1';
@@ -750,19 +798,19 @@ TORRENT_TEST(multiple_bitfields)
 	print_session_log(*ses);
 
 	s.close();
-	test_sleep(500);
+	std::this_thread::sleep_for(lt::milliseconds(500));
 	print_session_log(*ses);
 }
 
 TORRENT_TEST(multiple_have_all)
 {
-	std::cerr << "\n === test multiple have_all ===\n" << std::endl;
+	std::cout << "\n === test multiple have_all ===\n" << std::endl;
 
 	sha1_hash ih;
-	boost::shared_ptr<lt::session> ses;
+	std::shared_ptr<lt::session> ses;
 	io_service ios;
 	tcp::socket s(ios);
-	boost::shared_ptr<torrent_info> ti = setup_peer(s, ih, ses);
+	std::shared_ptr<torrent_info> ti = setup_peer(s, ih, ses);
 
 	char recv_buffer[1000];
 	do_handshake(s, ih, recv_buffer);
@@ -780,24 +828,24 @@ TORRENT_TEST(multiple_have_all)
 
 	s.close();
 	print_session_log(*ses);
-	test_sleep(500);
+	std::this_thread::sleep_for(lt::milliseconds(500));
 	print_session_log(*ses);
 }
 
-#ifndef TORRENT_DISABLE_EXTENSIONS
 // makes sure that pieces that are lost are not requested
 TORRENT_TEST(dont_have)
 {
-	using namespace libtorrent::detail;
+	using namespace lt::detail;
 
-	std::cerr << "\n === test dont_have ===\n" << std::endl;
+	std::cout << "\n === test dont_have ===\n" << std::endl;
 
 	sha1_hash ih;
 	torrent_handle th;
-	boost::shared_ptr<lt::session> ses;
+	std::shared_ptr<lt::session> ses;
 	io_service ios;
 	tcp::socket s(ios);
-	boost::shared_ptr<torrent_info> ti = setup_peer(s, ih, ses, true, 0, &th);
+	std::shared_ptr<torrent_info> ti = setup_peer(s, ih, ses, true
+		, false, false, torrent_flags_t{}, &th);
 
 	char recv_buffer[1000];
 	do_handshake(s, ih, recv_buffer);
@@ -805,7 +853,7 @@ TORRENT_TEST(dont_have)
 	send_have_all(s);
 	print_session_log(*ses);
 
-	test_sleep(300);
+	std::this_thread::sleep_for(lt::milliseconds(300));
 	print_session_log(*ses);
 
 	std::vector<peer_info> pi;
@@ -823,24 +871,25 @@ TORRENT_TEST(dont_have)
 	{
 		print_session_log(*ses);
 
-		int len = read_message(s, recv_buffer, sizeof(recv_buffer));
+		int const len = read_message(s, recv_buffer);
 		if (len == -1) break;
-		print_message(recv_buffer, len);
+		auto const buffer = span<char const>(recv_buffer).first(len);
+		print_message(buffer);
 		if (len == 0) continue;
-		int msg = recv_buffer[0];
+		int const msg = buffer[0];
 		if (msg != 20) continue;
-		int ext_msg = recv_buffer[1];
+		int const ext_msg = buffer[1];
 		if (ext_msg != 0) continue;
 
-		bdecode_node e;
 		int pos = 0;
-		int ret = bdecode(recv_buffer + 2, recv_buffer + len, e, ec, &pos);
-		if (ret != 0)
+		ec.clear();
+		bdecode_node e = bdecode(buffer.subspan(2), ec, &pos);
+		if (ec)
 		{
 			log("failed to parse extension handshake: %s at pos %d"
 				, ec.message().c_str(), pos);
 		}
-		TEST_EQUAL(ret, 0);
+		TEST_CHECK(!ec);
 
 		log("extension handshake: %s", print_entry(e).c_str());
 		bdecode_node m = e.dict_find_dict("m");
@@ -850,7 +899,7 @@ TORRENT_TEST(dont_have)
 		TEST_CHECK(dont_have);
 		if (!dont_have) return;
 
-		lt_dont_have = dont_have.int_value();
+		lt_dont_have = int(dont_have.int_value());
 	}
 	print_session_log(*ses);
 
@@ -866,7 +915,7 @@ TORRENT_TEST(dont_have)
 
 	print_session_log(*ses);
 
-	test_sleep(1000);
+	std::this_thread::sleep_for(lt::milliseconds(1000));
 
 	print_session_log(*ses);
 
@@ -875,31 +924,68 @@ TORRENT_TEST(dont_have)
 	TEST_EQUAL(pi.size(), 1);
 	if (pi.size() != 1) return;
 
-	TEST_EQUAL(pi[0].flags & peer_info::seed, 0);
+	TEST_CHECK(!(pi[0].flags & peer_info::seed));
 	TEST_EQUAL(pi[0].pieces.count(), pi[0].pieces.size() - 1);
-	TEST_EQUAL(pi[0].pieces[3], false);
-	TEST_EQUAL(pi[0].pieces[2], true);
-	TEST_EQUAL(pi[0].pieces[1], true);
-	TEST_EQUAL(pi[0].pieces[0], true);
+	TEST_EQUAL(pi[0].pieces[piece_index_t(3)], false);
+	TEST_EQUAL(pi[0].pieces[piece_index_t(2)], true);
+	TEST_EQUAL(pi[0].pieces[piece_index_t(1)], true);
+	TEST_EQUAL(pi[0].pieces[piece_index_t(0)], true);
 
 	print_session_log(*ses);
 }
 
+TORRENT_TEST(extension_handshake)
+{
+	using namespace lt::detail;
+
+	sha1_hash ih;
+	std::shared_ptr<lt::session> ses;
+	io_service ios;
+	tcp::socket s(ios);
+	std::shared_ptr<torrent_info> ti = setup_peer(s, ih, ses);
+
+	char recv_buffer[1000];
+	do_handshake(s, ih, recv_buffer);
+	print_session_log(*ses);
+	send_have_all(s);
+	print_session_log(*ses);
+
+	entry extensions;
+	send_extension_handshake(s, extensions);
+
+	extensions = read_extension_handshake(s, recv_buffer);
+
+	std::cout << extensions << '\n';
+
+	// these extensions are built-in
+	TEST_CHECK(extensions["m"]["lt_donthave"].integer() != 0);
+#ifndef TORRENT_DISABLE_SHARE_MODE
+	TEST_CHECK(extensions["m"]["share_mode"].integer() != 0);
+#endif
+	TEST_CHECK(extensions["m"]["upload_only"].integer() != 0);
+	TEST_CHECK(extensions["m"]["ut_holepunch"].integer() != 0);
+
+	// these require extensions to be enabled
+#ifndef TORRENT_DISABLE_EXTENSIONS
+	TEST_CHECK(extensions["m"]["ut_metadata"].integer() != 0);
+	TEST_CHECK(extensions["m"]["ut_pex"].integer() != 0);
+#endif
+}
+
+#ifndef TORRENT_DISABLE_EXTENSIONS
 // TEST metadata extension messages and edge cases
 
 // this tests sending a request for a metadata piece that's too high. This is
 // pos
 TORRENT_TEST(invalid_metadata_request)
 {
-	using namespace libtorrent::detail;
-
-	std::cerr << "\n === test invalid metadata ===\n" << std::endl;
+	using namespace lt::detail;
 
 	sha1_hash ih;
-	boost::shared_ptr<lt::session> ses;
+	std::shared_ptr<lt::session> ses;
 	io_service ios;
 	tcp::socket s(ios);
-	boost::shared_ptr<torrent_info> ti = setup_peer(s, ih, ses);
+	std::shared_ptr<torrent_info> ti = setup_peer(s, ih, ses);
 
 	char recv_buffer[1000];
 	do_handshake(s, ih, recv_buffer);
@@ -911,9 +997,9 @@ TORRENT_TEST(invalid_metadata_request)
 	extensions["m"]["ut_metadata"] = 1;
 	send_extension_handshake(s, extensions);
 
-	extensions = read_extension_handshake(s, recv_buffer, sizeof(recv_buffer));
+	extensions = read_extension_handshake(s, recv_buffer);
 
-	int ut_metadata = extensions["m"]["ut_metadata"].integer();
+	int ut_metadata = int(extensions["m"]["ut_metadata"].integer());
 
 	log("ut_metadata: %d", ut_metadata);
 
@@ -927,15 +1013,13 @@ TORRENT_TEST(invalid_metadata_request)
 	// we assume we were not disconnected because of the invalid one
 	send_ut_metadata_msg(s, ut_metadata, 0, 0);
 
-	entry ut_metadata_msg = read_ut_metadata_msg(s, recv_buffer
-		, sizeof(recv_buffer));
+	entry ut_metadata_msg = read_ut_metadata_msg(s, recv_buffer);
 
 	// the first response should be "dont-have"
 	TEST_EQUAL(ut_metadata_msg["msg_type"].integer(), 2);
 	TEST_EQUAL(ut_metadata_msg["piece"].integer(), 1);
 
-	ut_metadata_msg = read_ut_metadata_msg(s, recv_buffer
-		, sizeof(recv_buffer));
+	ut_metadata_msg = read_ut_metadata_msg(s, recv_buffer);
 
 	// the second response should be the payload
 	TEST_EQUAL(ut_metadata_msg["msg_type"].integer(), 1);
@@ -943,13 +1027,15 @@ TORRENT_TEST(invalid_metadata_request)
 
 	print_session_log(*ses);
 }
+#endif // TORRENT_DISABLE_EXTENSIONS
+
 
 TORRENT_TEST(invalid_request)
 {
-	std::cerr << "\n === test request ===\n" << std::endl;
+	std::cout << "\n === test request ===\n" << std::endl;
 
 	sha1_hash ih;
-	boost::shared_ptr<lt::session> ses;
+	std::shared_ptr<lt::session> ses;
 	io_service ios;
 	tcp::socket s(ios);
 	setup_peer(s, ih, ses);
@@ -960,19 +1046,21 @@ TORRENT_TEST(invalid_request)
 	send_have_none(s);
 
 	peer_request req;
-	req.piece = 124134235;
+	req.piece = piece_index_t(124134235);
 	req.start = 0;
 	req.length = 0x4000;
 	send_request(s, req);
 }
 
+namespace {
+
 void have_all_test(bool const incoming)
 {
 	sha1_hash ih;
-	boost::shared_ptr<lt::session> ses;
+	std::shared_ptr<lt::session> ses;
 	io_service ios;
 	tcp::socket s(ios);
-	setup_peer(s, ih, ses, incoming, add_torrent_params::flag_seed_mode);
+	setup_peer(s, ih, ses, incoming, false, false, torrent_flags::seed_mode);
 
 	char recv_buffer[1000];
 	do_handshake(s, ih, recv_buffer);
@@ -982,14 +1070,15 @@ void have_all_test(bool const incoming)
 	// since we advertised support for FAST extensions
 	for (;;)
 	{
-		int const len = read_message(s, recv_buffer, sizeof(recv_buffer));
+		int const len = read_message(s, recv_buffer);
 		if (len == -1)
 		{
 			TEST_ERROR("failed to receive have-all despite advertising support for FAST");
 			break;
 		}
-		print_message(recv_buffer, len);
-		int const msg = recv_buffer[0];
+		auto const buffer = span<char const>(recv_buffer).first(len);
+		print_message(buffer);
+		int const msg = buffer[0];
 		if (msg == 0xe) // have-all
 		{
 			// success!
@@ -1003,20 +1092,38 @@ void have_all_test(bool const incoming)
 	}
 }
 
+} // anonymous namespace
+
 TORRENT_TEST(outgoing_have_all)
 {
-	std::cerr << "\n === test outgoing have-all ===\n" << std::endl;
-	have_all_test(true);
+	std::cout << "\n === test outgoing have-all ===\n" << std::endl;
+	have_all_test(false);
 }
 
 TORRENT_TEST(incoming_have_all)
 {
-	std::cerr << "\n === test outgoing have-all ===\n" << std::endl;
-	have_all_test(false);
+	std::cout << "\n === test incoming have-all ===\n" << std::endl;
+	have_all_test(true);
 }
 
-#endif // TORRENT_DISABLE_EXTENSIONS
+TORRENT_TEST(dht_port_no_support)
+{
+	std::cout << "\n === test DHT port (without advertising support) ===\n" << std::endl;
 
+	sha1_hash ih;
+	std::shared_ptr<lt::session> ses;
+	io_service ios;
+	tcp::socket s(ios);
+	setup_peer(s, ih, ses, true, true, true);
+
+	char recv_buffer[1000];
+	do_handshake(s, ih, recv_buffer);
+	send_dht_port(s, 6881);
+	print_session_log(*ses);
+
+	s.close();
+	std::this_thread::sleep_for(lt::milliseconds(500));
+	print_session_log(*ses);
+}
 // TODO: test sending invalid requests (out of bound piece index, offsets and
 // sizes)
-
