@@ -32,73 +32,90 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "test.hpp"
 #include "libtorrent/packet_buffer.hpp"
+#include "libtorrent/packet_pool.hpp"
 
-using libtorrent::packet_buffer;
+using lt::packet_buffer;
+using lt::packet_ptr;
+using lt::packet_pool;
+using lt::packet;
+
+namespace {
+
+packet_ptr make_pkt(packet_pool& pool, int const val)
+{
+	packet_ptr ret = pool.acquire(20);
+	*reinterpret_cast<std::uint8_t*>(ret->buf) = std::uint8_t(val);
+	return ret;
+}
+
+int get_val(packet* pkt)
+{
+	TORRENT_ASSERT(pkt != nullptr);
+	return *reinterpret_cast<std::uint8_t*>(pkt->buf);
+}
+
+} // anonymous namespace
 
 // test packet_buffer
 TORRENT_TEST(insert)
 {
-	packet_buffer<int> pb;
-
-	int a123 = 123;
-	int a125 = 125;
-	int a500 = 500;
-	int a501 = 501;
+	packet_pool pool;
+	packet_buffer pb;
 
 	TEST_EQUAL(pb.capacity(), 0);
 	TEST_EQUAL(pb.size(), 0);
 	TEST_EQUAL(pb.span(), 0);
 
-	pb.insert(123, &a123);
-	TEST_EQUAL(pb.at(123 + 16), 0);
+	pb.insert(123, make_pkt(pool, 123));
+	TEST_CHECK(pb.at(123 + 16) == nullptr);
 
-	TEST_CHECK(pb.at(123) == &a123);
+	TEST_EQUAL(get_val(pb.at(123)), 123);
 	TEST_CHECK(pb.capacity() > 0);
 	TEST_EQUAL(pb.size(), 1);
 	TEST_EQUAL(pb.span(), 1);
 	TEST_EQUAL(pb.cursor(), 123);
 
-	pb.insert(125, &a125);
+	pb.insert(125, make_pkt(pool, 125));
 
-	TEST_CHECK(pb.at(125) == &a125);
+	TEST_EQUAL(get_val(pb.at(125)), 125);
 	TEST_EQUAL(pb.size(), 2);
 	TEST_EQUAL(pb.span(), 3);
 	TEST_EQUAL(pb.cursor(), 123);
 
-	pb.insert(500, &a500);
+	pb.insert(500, make_pkt(pool, 4));
 	TEST_EQUAL(pb.size(), 3);
 	TEST_EQUAL(pb.span(), 501 - 123);
 	TEST_EQUAL(pb.capacity(), 512);
 
-	pb.insert(500, &a501);
+	pb.insert(500, make_pkt(pool, 5));
 	TEST_EQUAL(pb.size(), 3);
-	pb.insert(500, &a500);
+	TEST_EQUAL(get_val(pb.insert(500, make_pkt(pool, 4)).get()), 5);
 	TEST_EQUAL(pb.size(), 3);
 
-	TEST_CHECK(pb.remove(123) == &a123);
+	TEST_EQUAL(get_val(pb.remove(123).get()), 123);
 	TEST_EQUAL(pb.size(), 2);
 	TEST_EQUAL(pb.span(), 501 - 125);
 	TEST_EQUAL(pb.cursor(), 125);
-	TEST_CHECK(pb.remove(125) == &a125);
+	TEST_EQUAL(get_val(pb.remove(125).get()), 125);
 	TEST_EQUAL(pb.size(), 1);
 	TEST_EQUAL(pb.span(), 1);
 	TEST_EQUAL(pb.cursor(), 500);
 
-	TEST_CHECK(pb.remove(500) == &a500);
+	TEST_EQUAL(get_val(pb.remove(500).get()), 4);
 	TEST_EQUAL(pb.size(), 0);
 	TEST_EQUAL(pb.span(), 0);
 
 	for (int i = 0; i < 0xff; ++i)
 	{
 		int index = (i + 0xfff0) & 0xffff;
-		pb.insert(index, reinterpret_cast<int*>(index + 1));
-		fprintf(stderr, "insert: %u (mask: %x)\n", index, int(pb.capacity() - 1));
+		pb.insert(packet_buffer::index_type(index), make_pkt(pool, index + 1));
+		std::printf("insert: %u (mask: %x)\n", index, int(pb.capacity() - 1));
 		TEST_EQUAL(pb.capacity(), 512);
 		if (i >= 14)
 		{
 			index = (index - 14) & 0xffff;
-			fprintf(stderr, "remove: %u\n", index);
-			TEST_CHECK(pb.remove(index) == reinterpret_cast<int*>(index + 1));
+			std::printf("remove: %u\n", index);
+			TEST_EQUAL(get_val(pb.remove(packet_buffer::index_type(index)).get()), std::uint8_t(index + 1));
 			TEST_EQUAL(pb.size(), 14);
 		}
 	}
@@ -107,60 +124,62 @@ TORRENT_TEST(insert)
 TORRENT_TEST(wrap)
 {
 	// test wrapping the indices
-	packet_buffer<void> pb;
+	packet_pool pool;
+	packet_buffer pb;
 
 	TEST_EQUAL(pb.size(), 0);
 
-	pb.insert(0xfffe, (void*)1);
-	TEST_CHECK(pb.at(0xfffe) == (void*)1);
+	pb.insert(0xfffe, make_pkt(pool, 1));
+	TEST_EQUAL(get_val(pb.at(0xfffe)), 1);
 
-	pb.insert(2, (void*)2);
-	TEST_CHECK(pb.at(2) == (void*)2);
+	pb.insert(2, make_pkt(pool, 2));
+	TEST_EQUAL(get_val(pb.at(2)), 2);
 
 	pb.remove(0xfffe);
-	TEST_CHECK(pb.at(0xfffe) == (void*)0);
-	TEST_CHECK(pb.at(2) == (void*)2);
+	TEST_CHECK(pb.at(0xfffe) == nullptr);
+	TEST_EQUAL(get_val(pb.at(2)), 2);
 }
 
 TORRENT_TEST(wrap2)
 {
 	// test wrapping the indices
-	packet_buffer<void> pb;
+	packet_pool pool;
+	packet_buffer pb;
 
 	TEST_EQUAL(pb.size(), 0);
 
-	pb.insert(0xfff3, (void*)1);
-	TEST_CHECK(pb.at(0xfff3) == (void*)1);
+	pb.insert(0xfff3, make_pkt(pool, 1));
+	TEST_EQUAL(get_val(pb.at(0xfff3)), 1);
 
-	int new_index = (0xfff3 + pb.capacity()) & 0xffff;
-	pb.insert(new_index, (void*)2);
-	TEST_CHECK(pb.at(new_index) == (void*)2);
+	auto const new_index = packet_buffer::index_type((0xfff3 + pb.capacity()) & 0xffff);
+	pb.insert(new_index, make_pkt(pool, 2));
+	TEST_EQUAL(get_val(pb.at(new_index)), 2);
 
-	void* old = pb.remove(0xfff3);
-	TEST_CHECK(old == (void*)1);
-	TEST_CHECK(pb.at(0xfff3) == (void*)0);
-	TEST_CHECK(pb.at(new_index) == (void*)2);
+	packet_ptr old = pb.remove(0xfff3);
+	TEST_CHECK(get_val(old.get()) == 1);
+	TEST_CHECK(pb.at(0xfff3) == nullptr);
+	TEST_EQUAL(get_val(pb.at(new_index)), 2);
 }
 
 TORRENT_TEST(reverse_wrap)
 {
 	// test wrapping the indices backwards
-	packet_buffer<void> pb;
+	packet_pool pool;
+	packet_buffer pb;
 
 	TEST_EQUAL(pb.size(), 0);
 
-	pb.insert(0xfff3, (void*)1);
-	TEST_CHECK(pb.at(0xfff3) == (void*)1);
+	pb.insert(0xfff3, make_pkt(pool, 1));
+	TEST_EQUAL(get_val(pb.at(0xfff3)), 1);
 
-	int new_index = (0xfff3 + pb.capacity()) & 0xffff;
-	pb.insert(new_index, (void*)2);
-	TEST_CHECK(pb.at(new_index) == (void*)2);
+	auto const new_index = packet_buffer::index_type((0xfff3 + pb.capacity()) & 0xffff);
+	pb.insert(new_index, make_pkt(pool, 2));
+	TEST_EQUAL(get_val(pb.at(new_index)), 2);
 
-	void* old = pb.remove(0xfff3);
-	TEST_CHECK(old == (void*)1);
-	TEST_CHECK(pb.at(0xfff3) == (void*)0);
-	TEST_CHECK(pb.at(new_index) == (void*)2);
+	packet_ptr old = pb.remove(0xfff3);
+	TEST_CHECK(get_val(old.get()) == 1);
+	TEST_CHECK(pb.at(0xfff3) == nullptr);
+	TEST_EQUAL(get_val(pb.at(new_index)), 2);
 
-	pb.insert(0xffff, (void*)0xffff);
+	pb.insert(0xffff, make_pkt(pool, 3));
 }
-
