@@ -32,10 +32,12 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/session.hpp"
 #include "libtorrent/alert_types.hpp"
-#include "libtorrent/thread.hpp"
-#include "libtorrent/file.hpp"
+#include "libtorrent/aux_/path.hpp"
 #include "libtorrent/session_status.hpp"
 #include "libtorrent/torrent_info.hpp"
+#include "libtorrent/hex.hpp" // for to_hex
+#include "libtorrent/time.hpp"
+#include "libtorrent/aux_/openssl.hpp"
 
 #include "test.hpp"
 #include "test_utils.hpp"
@@ -44,11 +46,6 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/aux_/disable_warnings_push.hpp"
 
-#include <boost/bind.hpp>
-#include <boost/tuple/tuple.hpp>
-
-#include <fstream>
-#include <iostream>
 #include <boost/asio/connect.hpp>
 
 #ifdef TORRENT_USE_OPENSSL
@@ -56,8 +53,16 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/aux_/disable_warnings_pop.hpp"
 
-using namespace libtorrent;
-using boost::tuples::ignore;
+#include <functional>
+#include <tuple>
+#include <fstream>
+#include <iostream>
+
+using namespace std::placeholders;
+using namespace lt;
+using std::ignore;
+
+namespace {
 
 struct test_config_t
 {
@@ -96,8 +101,16 @@ bool on_alert(alert const* a)
 	if (peer_disconnected_alert const* e = alert_cast<peer_disconnected_alert>(a))
 	{
 		++peer_disconnects;
-		if (e->error.category() == boost::asio::error::get_ssl_category())
+		string_view const cat = e->error.category().name();
+		if (cat == boost::asio::error::get_ssl_category().name()
+#if BOOST_VERSION >= 106400
+			|| cat == boost::asio::ssl::error::get_stream_category().name()
+#endif
+			)
 			++ssl_peer_disconnects;
+
+		std::printf("--- peer_errors: %d ssl_disconnects: %d\n"
+			, peer_errors, ssl_peer_disconnects);
 	}
 
 	if (peer_error_alert const* e = alert_cast<peer_error_alert>(a))
@@ -105,8 +118,16 @@ bool on_alert(alert const* a)
 		++peer_disconnects;
 		++peer_errors;
 
-		if (e->error.category() == boost::asio::error::get_ssl_category())
+		string_view const cat = e->error.category().name();
+		if (cat == boost::asio::error::get_ssl_category().name()
+#if BOOST_VERSION >= 106400
+			|| cat == boost::asio::ssl::error::get_stream_category().name()
+#endif
+			)
 			++ssl_peer_disconnects;
+
+		std::printf("--- peer_errors: %d ssl_disconnects: %d\n"
+			, peer_errors, ssl_peer_disconnects);
 	}
 	return false;
 }
@@ -121,17 +142,21 @@ void test_ssl(int const test_idx, bool const use_utp)
 
 	test_config_t const& test = test_config[test_idx];
 
-	fprintf(stderr, "\n%s TEST: %s Protocol: %s\n\n", time_now_string(), test.name, use_utp ? "uTP": "TCP");
+	std::printf("\n%s TEST: %s Protocol: %s\n\n", time_now_string()
+		, test.name, use_utp ? "uTP": "TCP");
 
 	// in case the previous run was terminated
 	error_code ec;
 	remove_all("tmp1_ssl", ec);
 	remove_all("tmp2_ssl", ec);
 
-	int ssl_port = 1024 + rand() % 50000;
+	int port = 1024 + rand() % 50000;
 	settings_pack sett = settings();
 	sett.set_int(settings_pack::max_retry_port_bind, 100);
-	sett.set_str(settings_pack::listen_interfaces, "0.0.0.0:48075");
+
+	char listen_iface[100];
+	std::snprintf(listen_iface, sizeof(listen_iface), "0.0.0.0:%ds", port);
+	sett.set_str(settings_pack::listen_interfaces, listen_iface);
 	sett.set_bool(settings_pack::enable_incoming_utp, use_utp);
 	sett.set_bool(settings_pack::enable_outgoing_utp, use_utp);
 	sett.set_bool(settings_pack::enable_incoming_tcp, !use_utp);
@@ -142,16 +167,21 @@ void test_ssl(int const test_idx, bool const use_utp)
 	sett.set_bool(settings_pack::enable_natpmp, false);
 	// if a peer fails once, don't try it again
 	sett.set_int(settings_pack::max_failcount, 1);
-	sett.set_int(settings_pack::ssl_listen, ssl_port);
 
-	libtorrent::session ses1(sett, 0);
+	lt::session ses1(sett, {});
 
+	// this +20 is here to use a different port as ses1
+	port += 20;
+
+	// the +20 below is the port we use for non-SSL connections
 	if (test.downloader_has_ssl_listen_port)
-		sett.set_int(settings_pack::ssl_listen, ssl_port + 20);
+		std::snprintf(listen_iface, sizeof(listen_iface), "0.0.0.0:%d,0.0.0.0:%ds", port + 20, port);
 	else
-		sett.set_int(settings_pack::ssl_listen, 0);
+		std::snprintf(listen_iface, sizeof(listen_iface), "0.0.0.0:%d", port + 20);
 
-	libtorrent::session ses2(sett, 0);
+	sett.set_str(settings_pack::listen_interfaces, listen_iface);
+
+	lt::session ses2(sett, {});
 
 	wait_for_listen(ses1, "ses1");
 	wait_for_listen(ses2, "ses2");
@@ -161,20 +191,20 @@ void test_ssl(int const test_idx, bool const use_utp)
 
 	create_directory("tmp1_ssl", ec);
 	std::ofstream file("tmp1_ssl/temporary");
-	boost::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary"
+	std::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary"
 		, 16 * 1024, 13, false, combine_path("..", combine_path("ssl", "root_ca_cert.pem")));
 	file.close();
 
 	add_torrent_params addp;
 	addp.save_path = "tmp1_ssl";
-	addp.flags &= ~add_torrent_params::flag_paused;
-	addp.flags &= ~add_torrent_params::flag_auto_managed;
+	addp.flags &= ~torrent_flags::paused;
+	addp.flags &= ~torrent_flags::auto_managed;
 
 	peer_disconnects = 0;
 	ssl_peer_disconnects = 0;
 	peer_errors = 0;
 
-	boost::tie(tor1, tor2, ignore) = setup_transfer(&ses1, &ses2, 0
+	std::tie(tor1, tor2, ignore) = setup_transfer(&ses1, &ses2, nullptr
 		, true, false, false, "_ssl", 16 * 1024, &t, false, &addp, true);
 
 	if (test.seed_has_cert)
@@ -208,36 +238,24 @@ void test_ssl(int const test_idx, bool const use_utp)
 	wait_for_downloading(ses2, "ses2");
 
 	// connect the peers after setting the certificates
-	int port = 0;
-	if (test.use_ssl_ports)
-		if (test.downloader_has_ssl_listen_port)
-			port = ses2.ssl_listen_port();
-		else
-			port = 13512;
-	else
-		port = ses2.listen_port();
-
-	fprintf(stderr, "\n\n%s: ses1: connecting peer port: %d\n\n\n"
+	if (test.use_ssl_ports == false) port += 20;
+	std::printf("\n\n%s: ses1: connecting peer port: %d\n\n\n"
 		, time_now_string(), port);
 	tor1.connect_peer(tcp::endpoint(address::from_string("127.0.0.1", ec)
-		, port));
+		, std::uint16_t(port)));
 
-#ifdef TORRENT_USE_VALGRIND
-	const int timeout = 100;
-#else
 	const int timeout = 40;
-#endif
 	for (int i = 0; i < timeout; ++i)
 	{
-		print_alerts(ses1, "ses1", true, true, true, &on_alert);
-		print_alerts(ses2, "ses2", true, true, true, &on_alert);
+		print_alerts(ses1, "ses1", true, true, &on_alert);
+		print_alerts(ses2, "ses2", true, true, &on_alert);
 
 		torrent_status st1 = tor1.status();
 		torrent_status st2 = tor2.status();
 
 		if (i % 10 == 0)
 		{
-			std::cerr << time_now_string() << " "
+			std::cout << time_now_string() << " "
 				<< "\033[32m" << int(st1.download_payload_rate / 1000.f) << "kB/s "
 				<< "\033[33m" << int(st1.upload_payload_rate / 1000.f) << "kB/s "
 				<< "\033[0m" << int(st1.progress * 100) << "% "
@@ -251,20 +269,20 @@ void test_ssl(int const test_idx, bool const use_utp)
 				<< std::endl;
 		}
 
+		if (st2.is_finished) break;
+
 		if (peer_disconnects >= 2)
 		{
-			fprintf(stderr, "too many disconnects (%d), breaking\n", peer_disconnects);
+			std::printf("too many disconnects (%d), breaking\n", peer_disconnects);
 			break;
 		}
 
-		if (st2.is_finished) break;
-
 		if (st2.state != torrent_status::downloading)
 		{
 			static char const* state_str[] =
 				{"checking (q)", "checking", "dl metadata"
 				, "downloading", "finished", "seeding", "allocating", "checking (r)"};
-			std::cerr << "st2 state: " << state_str[st2.state] << std::endl;
+			std::cout << "st2 state: " << state_str[st2.state] << std::endl;
 		}
 
 		TEST_CHECK(st1.state == torrent_status::seeding
@@ -272,21 +290,22 @@ void test_ssl(int const test_idx, bool const use_utp)
 		TEST_CHECK(st2.state == torrent_status::downloading
 			|| st2.state == torrent_status::checking_resume_data);
 
-		test_sleep(100);
+		std::this_thread::sleep_for(lt::milliseconds(100));
 	}
 
-	fprintf(stderr, "peer_errors: %d expected_errors: %d\n"
+	std::printf("peer_errors: %d expected_errors: %d\n"
 		, peer_errors, test.peer_errors);
 
-	fprintf(stderr, "ssl_disconnects: %d  expected: %d\n", ssl_peer_disconnects, test.ssl_disconnects);
+	std::printf("ssl_disconnects: %d  expected: %d\n", ssl_peer_disconnects, test.ssl_disconnects);
 	if (!use_utp)
 	{
 		TEST_EQUAL(ssl_peer_disconnects > 0, test.ssl_disconnects > 0);
 		TEST_EQUAL(peer_errors > 0, test.peer_errors > 0);
 	}
 
-	fprintf(stderr, "%s: EXPECT: %s\n", time_now_string(), test.expected_to_complete ? "SUCCEESS" : "FAILURE");
-	fprintf(stderr, "%s: RESULT: %s\n", time_now_string(), tor2.status().is_seeding ? "SUCCEESS" : "FAILURE");
+	char const* now = time_now_string();
+	std::printf("%s: EXPECT: %s\n", now, test.expected_to_complete ? "SUCCEESS" : "FAILURE");
+	std::printf("%s: RESULT: %s\n", now, tor2.status().is_seeding ? "SUCCEESS" : "FAILURE");
 	TEST_EQUAL(tor2.status().is_seeding, test.expected_to_complete);
 
 	// this allows shutting down the sessions in parallel
@@ -294,7 +313,7 @@ void test_ssl(int const test_idx, bool const use_utp)
 	p2 = ses2.abort();
 }
 
-std::string password_callback(int length, boost::asio::ssl::context::password_purpose p
+std::string password_callback(std::size_t /*length*/, boost::asio::ssl::context::password_purpose p
 	, std::string pw)
 {
 	if (p != boost::asio::ssl::context::for_reading) return "";
@@ -304,7 +323,7 @@ std::string password_callback(int length, boost::asio::ssl::context::password_pu
 struct attack_t
 {
 	// flags controlling the connection attempt
-	boost::uint32_t flags;
+	std::uint32_t flags;
 	// whether or not we expect to be able to connect
 	bool expect;
 };
@@ -337,24 +356,24 @@ attack_t const attacks[] =
 
 const int num_attacks = sizeof(attacks)/sizeof(attacks[0]);
 
-bool try_connect(libtorrent::session& ses1, int port
-	, boost::shared_ptr<torrent_info> const& t, boost::uint32_t flags)
+bool try_connect(lt::session& ses1, int port
+	, std::shared_ptr<torrent_info> const& t, std::uint32_t flags)
 {
 	using boost::asio::ssl::context;
 
-	fprintf(stderr, "\nMALICIOUS PEER TEST: ");
-	if (flags & invalid_certificate) fprintf(stderr, "invalid-certificate ");
-	else if (flags & valid_certificate) fprintf(stderr, "valid-certificate ");
-	else fprintf(stderr, "no-certificate ");
+	std::printf("\nMALICIOUS PEER TEST: ");
+	if (flags & invalid_certificate) std::printf("invalid-certificate ");
+	else if (flags & valid_certificate) std::printf("valid-certificate ");
+	else std::printf("no-certificate ");
 
-	if (flags & invalid_sni_hash) fprintf(stderr, "invalid-SNI-hash ");
-	else if (flags & valid_sni_hash) fprintf(stderr, "valid-SNI-hash ");
-	else fprintf(stderr, "no-SNI-hash ");
+	if (flags & invalid_sni_hash) std::printf("invalid-SNI-hash ");
+	else if (flags & valid_sni_hash) std::printf("valid-SNI-hash ");
+	else std::printf("no-SNI-hash ");
 
-	if (flags & valid_bittorrent_hash) fprintf(stderr, "valid-bittorrent-hash ");
-	else fprintf(stderr, "invalid-bittorrent-hash ");
+	if (flags & valid_bittorrent_hash) std::printf("valid-bittorrent-hash ");
+	else std::printf("invalid-bittorrent-hash ");
 
-	fprintf(stderr, " port: %d\n", port);
+	std::printf(" port: %d\n", port);
 
 	error_code ec;
 	boost::asio::io_service ios;
@@ -373,7 +392,7 @@ bool try_connect(libtorrent::session& ses1, int port
 	ctx.set_verify_mode(context::verify_none, ec);
 	if (ec)
 	{
-		fprintf(stderr, "Failed to set SSL verify mode: %s\n"
+		std::printf("Failed to set SSL verify mode: %s\n"
 			, ec.message().c_str());
 		TEST_CHECK(!ec);
 		return false;
@@ -393,38 +412,38 @@ bool try_connect(libtorrent::session& ses1, int port
 
 	if (flags & (valid_certificate | invalid_certificate))
 	{
-		fprintf(stderr, "set_password_callback\n");
-		ctx.set_password_callback(boost::bind(&password_callback, _1, _2, "test"), ec);
+		std::printf("set_password_callback\n");
+		ctx.set_password_callback(std::bind(&password_callback, _1, _2, "test"), ec);
 		if (ec)
 		{
-			fprintf(stderr, "Failed to set certificate password callback: %s\n"
+			std::printf("Failed to set certificate password callback: %s\n"
 				, ec.message().c_str());
 			TEST_CHECK(!ec);
 			return false;
 		}
-		fprintf(stderr, "use_certificate_file \"%s\"\n", certificate.c_str());
+		std::printf("use_certificate_file \"%s\"\n", certificate.c_str());
 		ctx.use_certificate_file(certificate, context::pem, ec);
 		if (ec)
 		{
-			fprintf(stderr, "Failed to set certificate file: %s\n"
+			std::printf("Failed to set certificate file: %s\n"
 				, ec.message().c_str());
 			TEST_CHECK(!ec);
 			return false;
 		}
-		fprintf(stderr, "use_private_key_file \"%s\"\n", private_key.c_str());
+		std::printf("use_private_key_file \"%s\"\n", private_key.c_str());
 		ctx.use_private_key_file(private_key, context::pem, ec);
 		if (ec)
 		{
-			fprintf(stderr, "Failed to set private key: %s\n"
+			std::printf("Failed to set private key: %s\n"
 				, ec.message().c_str());
 			TEST_CHECK(!ec);
 			return false;
 		}
-		fprintf(stderr, "use_tmp_dh_file \"%s\"\n", dh_params.c_str());
+		std::printf("use_tmp_dh_file \"%s\"\n", dh_params.c_str());
 		ctx.use_tmp_dh_file(dh_params, ec);
 		if (ec)
 		{
-			fprintf(stderr, "Failed to set DH params: %s\n"
+			std::printf("Failed to set DH params: %s\n"
 				, ec.message().c_str());
 			TEST_CHECK(!ec);
 			return false;
@@ -433,14 +452,14 @@ bool try_connect(libtorrent::session& ses1, int port
 
 	boost::asio::ssl::stream<tcp::socket> ssl_sock(ios, ctx);
 
-	fprintf(stderr, "connecting 127.0.0.1:%d\n", port);
+	std::printf("connecting 127.0.0.1:%d\n", port);
 	ssl_sock.lowest_layer().connect(tcp::endpoint(
-		address_v4::from_string("127.0.0.1"), port), ec);
-	print_alerts(ses1, "ses1", true, true, true, &on_alert);
+		address_v4::from_string("127.0.0.1"), std::uint16_t(port)), ec);
+	print_alerts(ses1, "ses1", true, true, &on_alert);
 
 	if (ec)
 	{
-		fprintf(stderr, "Failed to connect: %s\n"
+		std::printf("Failed to connect: %s\n"
 			, ec.message().c_str());
 		TEST_CHECK(!ec);
 		return false;
@@ -448,9 +467,9 @@ bool try_connect(libtorrent::session& ses1, int port
 
 	if (flags & valid_sni_hash)
 	{
-		std::string name = to_hex(t->info_hash().to_string());
-		fprintf(stderr, "SNI: %s\n", name.c_str());
-		SSL_set_tlsext_host_name(ssl_sock.native_handle(), name.c_str());
+		std::string name = aux::to_hex(t->info_hash());
+		std::printf("SNI: %s\n", name.c_str());
+		aux::openssl_set_tlsext_hostname(ssl_sock.native_handle(), name.c_str());
 	}
 	else if (flags & invalid_sni_hash)
 	{
@@ -460,17 +479,17 @@ bool try_connect(libtorrent::session& ses1, int port
 		for (int i = 0; i < 40; ++i)
 			name += hex_alphabet[rand() % 16];
 
-		fprintf(stderr, "SNI: %s\n", name.c_str());
-		SSL_set_tlsext_host_name(ssl_sock.native_handle(), name.c_str());
+		std::printf("SNI: %s\n", name.c_str());
+		aux::openssl_set_tlsext_hostname(ssl_sock.native_handle(), name.c_str());
 	}
 
-	fprintf(stderr, "SSL handshake\n");
+	std::printf("SSL handshake\n");
 	ssl_sock.handshake(boost::asio::ssl::stream_base::client, ec);
 
-	print_alerts(ses1, "ses1", true, true, true, &on_alert);
+	print_alerts(ses1, "ses1", true, true, &on_alert);
 	if (ec)
 	{
-		fprintf(stderr, "Failed SSL handshake: %s\n"
+		std::printf("Failed SSL handshake: %s\n"
 			, ec.message().c_str());
 		return false;
 	}
@@ -495,40 +514,40 @@ bool try_connect(libtorrent::session& ses1, int port
 	// fill in the peer-id
 	std::generate(handshake + 48, handshake + 68, &rand);
 
-	fprintf(stderr, "bittorrent handshake\n");
+	std::printf("bittorrent handshake\n");
 	boost::asio::write(ssl_sock, boost::asio::buffer(handshake, (sizeof(handshake) - 1)), ec);
-	print_alerts(ses1, "ses1", true, true, true, &on_alert);
+	print_alerts(ses1, "ses1", true, true, &on_alert);
 	if (ec)
 	{
-		fprintf(stderr, "failed to write bittorrent handshake: %s\n"
+		std::printf("failed to write bittorrent handshake: %s\n"
 			, ec.message().c_str());
 		return false;
 	}
 
 	char buf[68];
-	fprintf(stderr, "read bittorrent handshake\n");
+	std::printf("read bittorrent handshake\n");
 	boost::asio::read(ssl_sock, boost::asio::buffer(buf, sizeof(buf)), ec);
-	print_alerts(ses1, "ses1", true, true, true, &on_alert);
+	print_alerts(ses1, "ses1", true, true, &on_alert);
 	if (ec)
 	{
-		fprintf(stderr, "failed to read bittorrent handshake: %s\n"
+		std::printf("failed to read bittorrent handshake: %s\n"
 			, ec.message().c_str());
 		return false;
 	}
 
 	if (memcmp(buf, "\x13" "BitTorrent protocol", 20) != 0)
 	{
-		fprintf(stderr, "invalid bittorrent handshake\n");
+		std::printf("invalid bittorrent handshake\n");
 		return false;
 	}
 
 	if (memcmp(buf + 28, &t->info_hash()[0], 20) != 0)
 	{
-		fprintf(stderr, "invalid info-hash in bittorrent handshake\n");
+		std::printf("invalid info-hash in bittorrent handshake\n");
 		return false;
 	}
 
-	fprintf(stderr, "successfully connected over SSL and shook hand over bittorrent\n");
+	std::printf("successfully connected over SSL and shook hand over bittorrent\n");
 
 	return true;
 }
@@ -539,23 +558,25 @@ void test_malicious_peer()
 	remove_all("tmp3_ssl", ec);
 
 	// set up session
-	int ssl_port = 1024 + rand() % 50000;
+	int port = 1024 + rand() % 50000;
 	settings_pack sett = settings();
 	sett.set_int(settings_pack::max_retry_port_bind, 100);
-	sett.set_str(settings_pack::listen_interfaces, "0.0.0.0:48075");
-	sett.set_int(settings_pack::ssl_listen, ssl_port);
+
+	char listen_iface[100];
+	std::snprintf(listen_iface, sizeof(listen_iface), "0.0.0.0:%ds", port);
+	sett.set_str(settings_pack::listen_interfaces, listen_iface);
 	sett.set_bool(settings_pack::enable_dht, false);
 	sett.set_bool(settings_pack::enable_lsd, false);
 	sett.set_bool(settings_pack::enable_upnp, false);
 	sett.set_bool(settings_pack::enable_natpmp, false);
 
-	libtorrent::session ses1(sett, 0);
+	lt::session ses1(sett, {});
 	wait_for_listen(ses1, "ses1");
 
 	// create torrent
 	create_directory("tmp3_ssl", ec);
 	std::ofstream file("tmp3_ssl/temporary");
-	boost::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary"
+	std::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary"
 		, 16 * 1024, 13, false, combine_path("..", combine_path("ssl", "root_ca_cert.pem")));
 	file.close();
 
@@ -563,8 +584,8 @@ void test_malicious_peer()
 
 	add_torrent_params addp;
 	addp.save_path = "tmp3_ssl";
-	addp.flags &= ~add_torrent_params::flag_paused;
-	addp.flags &= ~add_torrent_params::flag_auto_managed;
+	addp.flags &= ~torrent_flags::paused;
+	addp.flags &= ~torrent_flags::auto_managed;
 	addp.ti = t;
 
 	torrent_handle tor1 = ses1.add_torrent(addp, ec);
@@ -585,20 +606,18 @@ void test_malicious_peer()
 
 	for (int i = 0; i < num_attacks; ++i)
 	{
-		bool success = try_connect(ses1, ssl_port, t, attacks[i].flags);
+		bool const success = try_connect(ses1, port, t, attacks[i].flags);
 		TEST_EQUAL(success, attacks[i].expect);
 	}
 }
-#endif // TORRENT_USE_OPENSSL
+
+} // anonymous namespace
 
 TORRENT_TEST(malicious_peer)
 {
-#ifdef TORRENT_USE_OPENSSL
 	test_malicious_peer();
-#endif
 }
 
-#ifdef TORRENT_USE_OPENSSL
 TORRENT_TEST(utp_config0) { test_ssl(0, true); }
 TORRENT_TEST(utp_config1) { test_ssl(1, true); }
 TORRENT_TEST(utp_config2) { test_ssl(2, true); }
@@ -618,5 +637,7 @@ TORRENT_TEST(tcp_config5) { test_ssl(5, false); }
 TORRENT_TEST(tcp_config6) { test_ssl(6, false); }
 TORRENT_TEST(tcp_config7) { test_ssl(7, false); }
 TORRENT_TEST(tcp_config8) { test_ssl(8, false); }
-#endif
+#else
+TORRENT_TEST(disabled) {}
+#endif // TORRENT_USE_OPENSSL
 
