@@ -78,7 +78,7 @@ bool prioritize_files(torrent_handle const& h, aux::vector<download_priority_t,
 void test_running_torrent(std::shared_ptr<torrent_info> info, std::int64_t file_size)
 {
 	settings_pack pack = settings();
-	pack.set_int(settings_pack::alert_mask, alert::piece_progress_notification | alert::storage_notification);
+	pack.set_int(settings_pack::alert_mask, alert_category::piece_progress | alert_category::storage);
 	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:48130");
 	pack.set_int(settings_pack::max_retry_port_bind, 10);
 	lt::session ses(pack);
@@ -231,7 +231,7 @@ TORRENT_TEST(total_wanted)
 	auto info = std::make_shared<torrent_info>(tmp, from_span);
 
 	settings_pack pack = settings();
-	pack.set_int(settings_pack::alert_mask, alert::storage_notification);
+	pack.set_int(settings_pack::alert_mask, alert_category::storage);
 	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:48130");
 	pack.set_int(settings_pack::max_retry_port_bind, 10);
 	lt::session ses(pack);
@@ -277,6 +277,7 @@ TORRENT_TEST(added_peers)
 	add_torrent_params p = parse_magnet_uri(
 		"magnet:?xt=urn:btih:abababababababababababababababababababab&x.pe=127.0.0.1:48081&x.pe=127.0.0.2:48082");
 	p.ti = info;
+	p.info_hash.clear();
 	p.save_path = ".";
 
 	torrent_handle h = ses.add_torrent(std::move(p));
@@ -290,6 +291,89 @@ TORRENT_TEST(added_peers)
 	if (ra) TEST_EQUAL(ra->params.peers.size(), 2);
 }
 
+TORRENT_TEST(mismatching_info_hash)
+{
+	file_storage fs;
+	fs.add_file("test_torrent_dir4/tmp1", 1024);
+	lt::create_torrent t(fs, 1024);
+	std::vector<char> tmp;
+	bencode(std::back_inserter(tmp), t.generate());
+	auto info = std::make_shared<torrent_info>(tmp, from_span);
+
+	add_torrent_params p;
+	p.ti = std::move(info);
+
+	// this info-hash is definitely different from the one in `info`, this
+	// should trigger a failure
+	p.info_hash = lt::sha1_hash("01010101010101010101");
+	p.save_path = ".";
+
+	lt::session ses(settings());
+	error_code ec;
+	torrent_handle h = ses.add_torrent(std::move(p), ec);
+	TEST_CHECK(ec == lt::errors::mismatching_info_hash);
+	TEST_CHECK(!h.is_valid());
+}
+
+TORRENT_TEST(exceed_file_prio)
+{
+	file_storage fs;
+	fs.add_file("test_torrent_dir4/tmp1", 1024);
+	lt::create_torrent t(fs, 1024);
+	std::vector<char> tmp;
+	bencode(std::back_inserter(tmp), t.generate());
+	auto info = std::make_shared<torrent_info>(tmp, from_span);
+
+	add_torrent_params p;
+	p.ti = std::move(info);
+
+	p.file_priorities.resize(9999, lt::low_priority);
+	p.save_path = ".";
+
+	lt::session ses(settings());
+	error_code ec;
+	torrent_handle h = ses.add_torrent(std::move(p));
+	auto const prios = h.get_file_priorities();
+	TEST_CHECK(prios.size() == 1);
+}
+
+TORRENT_TEST(exceed_piece_prio)
+{
+	file_storage fs;
+	fs.add_file("test_torrent_dir4/tmp1", 1024);
+	lt::create_torrent t(fs, 1024);
+	std::vector<char> tmp;
+	bencode(std::back_inserter(tmp), t.generate());
+	auto info = std::make_shared<torrent_info>(tmp, from_span);
+	std::size_t const num_pieces = std::size_t(info->num_pieces());
+
+	add_torrent_params p;
+	p.ti = std::move(info);
+
+	p.piece_priorities.resize(9999, lt::low_priority);
+	p.save_path = ".";
+
+	lt::session ses(settings());
+	error_code ec;
+	torrent_handle h = ses.add_torrent(std::move(p));
+	auto const prios = h.get_piece_priorities();
+	TEST_CHECK(prios.size() == num_pieces);
+}
+
+TORRENT_TEST(exceed_piece_prio_magnet)
+{
+	add_torrent_params p;
+	p.info_hash = sha1_hash("abababababababababab");
+	p.piece_priorities.resize(9999, lt::low_priority);
+	p.save_path = ".";
+
+	lt::session ses(settings());
+	error_code ec;
+	torrent_handle h = ses.add_torrent(std::move(p));
+	auto const prios = h.get_piece_priorities();
+	TEST_CHECK(prios.empty());
+}
+
 TORRENT_TEST(torrent)
 {
 /*	{
@@ -761,7 +845,7 @@ TORRENT_TEST(symlinks_restore)
 	p.flags &= ~lt::torrent_flags::paused;
 	p.save_path = work_dir;
 	settings_pack pack = settings();
-	pack.set_int(libtorrent::settings_pack::alert_mask, libtorrent::alert::status_notification | libtorrent::alert::error_notification);
+	pack.set_int(libtorrent::settings_pack::alert_mask, libtorrent::alert_category::status | libtorrent::alert_category::error);
 	lt::session ses(std::move(pack));
 	ses.add_torrent(p);
 
