@@ -265,10 +265,8 @@ TORRENT_TEST(set_web_seeds)
 #define SEPARATOR "/"
 #endif
 
-TORRENT_TEST(sanitize_long_path)
+TORRENT_TEST(sanitize_path_truncate)
 {
-	// test sanitize_append_path_element
-
 	std::string path;
 	sanitize_append_path_element(path,
 		"abcdefghi_abcdefghi_abcdefghi_abcdefghi_abcdefghi_"
@@ -295,6 +293,24 @@ TORRENT_TEST(sanitize_long_path)
 		"abcdefghi_abcdefghi_abcdefghi_abcdefghi_.test");
 }
 
+TORRENT_TEST(sanitize_path_truncate_utf)
+{
+	std::string path;
+	// msvc doesn't like unicode string literals, so we encode it as UTF-8 explicitly
+	sanitize_append_path_element(path,
+		"abcdefghi_abcdefghi_abcdefghi_abcdefghi_abcdefghi_"
+		"abcdefghi_abcdefghi_abcdefghi_abcdefghi_abcdefghi_"
+		"abcdefghi_abcdefghi_abcdefghi_abcdefghi_abcdefghi_"
+		"abcdefghi_abcdefghi_abcdefghi_abcdefghi_abcdefghi_"
+		"abcdefghi_abcdefghi_abcdefghi_abcdefghi" "\xE2" "\x80" "\x94" "abcde.jpg");
+	TEST_EQUAL(path,
+		"abcdefghi_abcdefghi_abcdefghi_abcdefghi_abcdefghi_"
+		"abcdefghi_abcdefghi_abcdefghi_abcdefghi_abcdefghi_"
+		"abcdefghi_abcdefghi_abcdefghi_abcdefghi_abcdefghi_"
+		"abcdefghi_abcdefghi_abcdefghi_abcdefghi_abcdefghi_"
+		"abcdefghi_abcdefghi_abcdefghi_abcdefghi" "\xE2" "\x80" "\x94" ".jpg");
+}
+
 TORRENT_TEST(sanitize_path_trailing_dots)
 {
 	std::string path;
@@ -628,28 +644,28 @@ TORRENT_TEST(verify_encoding)
 	test = "filename\xf8\x9f\x9f\x9f\x9f""foobar";
 	TEST_CHECK(!verify_encoding(test));
 	std::printf("%s\n", test.c_str());
-	TEST_CHECK(test == "filename_____foobar");
+	TEST_CHECK(test == "filename_foobar");
 
 	// redundant (overlong) 2-byte sequence
 	// ascii code 0x2e encoded with a leading 0
 	test = "filename\xc0\xae";
 	TEST_CHECK(!verify_encoding(test));
 	std::printf("%s\n", test.c_str());
-	TEST_CHECK(test == "filename__");
+	TEST_CHECK(test == "filename_");
 
 	// redundant (overlong) 3-byte sequence
 	// ascii code 0x2e encoded with two leading 0s
 	test = "filename\xe0\x80\xae";
 	TEST_CHECK(!verify_encoding(test));
 	std::printf("%s\n", test.c_str());
-	TEST_CHECK(test == "filename___");
+	TEST_CHECK(test == "filename_");
 
 	// redundant (overlong) 4-byte sequence
 	// ascii code 0x2e encoded with three leading 0s
 	test = "filename\xf0\x80\x80\xae";
 	TEST_CHECK(!verify_encoding(test));
 	std::printf("%s\n", test.c_str());
-	TEST_CHECK(test == "filename____");
+	TEST_CHECK(test == "filename_");
 
 	// missing byte header
 	test = "filename\xed\0\x80";
