@@ -44,12 +44,13 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/http_tracker_connection.hpp" // for parse_tracker_response
 #include "libtorrent/torrent_info.hpp"
 #include "libtorrent/announce_entry.hpp"
+#include "libtorrent/torrent.hpp"
+#include "libtorrent/aux_/path.hpp"
+#include "libtorrent/socket_io.hpp"
 
 #include <fstream>
-#include <boost/bind.hpp>
 
-using namespace libtorrent;
-namespace lt = libtorrent;
+using namespace lt;
 
 // TODO: test scrape requests
 // TODO: test parse peers6
@@ -65,9 +66,11 @@ namespace lt = libtorrent;
 
 TORRENT_TEST(parse_hostname_peers)
 {
-	char const response[] = "d5:peersld7:peer id20:aaaaaaaaaaaaaaaaaaaa2:ip13:test_hostname4:porti1000eed7:peer id20:bbbbabaababababababa2:ip12:another_host4:porti1001eeee";
+	char const response[] = "d5:peersld7:peer id20:aaaaaaaaaaaaaaaaaaaa"
+		"2:ip13:test_hostname4:porti1000eed"
+		"7:peer id20:bbbbabaababababababa2:ip12:another_host4:porti1001eeee";
 	error_code ec;
-	tracker_response resp = parse_tracker_response(response, sizeof(response) - 1
+	tracker_response resp = parse_tracker_response(response
 		, ec, false, sha1_hash());
 
 	TEST_EQUAL(ec, error_code());
@@ -91,7 +94,7 @@ TORRENT_TEST(parse_peers4)
 	char const response[] = "d5:peers12:\x01\x02\x03\x04\x30\x10"
 		"\x09\x08\x07\x06\x20\x10" "e";
 	error_code ec;
-	tracker_response resp = parse_tracker_response(response, sizeof(response) - 1
+	tracker_response resp = parse_tracker_response(response
 		, ec, false, sha1_hash());
 
 	TEST_EQUAL(ec, error_code());
@@ -100,18 +103,19 @@ TORRENT_TEST(parse_peers4)
 	{
 		ipv4_peer_entry const& e0 = resp.peers4[0];
 		ipv4_peer_entry const& e1 = resp.peers4[1];
-		TEST_CHECK(e0.ip == address_v4::from_string("1.2.3.4").to_bytes());
+		TEST_CHECK(e0.ip == addr4("1.2.3.4").to_bytes());
 		TEST_EQUAL(e0.port, 0x3010);
 
-		TEST_CHECK(e1.ip == address_v4::from_string("9.8.7.6").to_bytes());
+		TEST_CHECK(e1.ip == addr4("9.8.7.6").to_bytes());
 		TEST_EQUAL(e1.port, 0x2010);
 	}
 }
 
+#if TORRENT_USE_I2P
 TORRENT_TEST(parse_i2p_peers)
 {
 	// d8:completei8e10:incompletei4e8:intervali3600e5:peers352: ...
-	boost::uint8_t const response[] = { 0x64, 0x38, 0x3a, 0x63, 0x6f, 0x6d,
+	std::uint8_t const response[] = { 0x64, 0x38, 0x3a, 0x63, 0x6f, 0x6d,
 		0x70, 0x6c, 0x65, 0x74, 0x65, 0x69, 0x38, 0x65, 0x31, 0x30,
 		0x3a, 0x69, 0x6e, 0x63, 0x6f, 0x6d, 0x70, 0x6c, 0x65, 0x74,
 		0x65, 0x69, 0x34, 0x65, 0x38, 0x3a, 0x69, 0x6e, 0x74, 0x65,
@@ -155,7 +159,7 @@ TORRENT_TEST(parse_i2p_peers)
 		0xa0, 0x75, 0xab, 0x65 };
 	error_code ec;
 	tracker_response resp = parse_tracker_response(
-		reinterpret_cast<char const*>(response), sizeof(response)
+		{ reinterpret_cast<char const*>(response), sizeof(response) }
 		, ec, tracker_request::i2p, sha1_hash());
 
 	TEST_EQUAL(ec, error_code());
@@ -163,30 +167,33 @@ TORRENT_TEST(parse_i2p_peers)
 
 	if (resp.peers.size() == 11)
 	{
-		TEST_EQUAL(resp.peers[0].hostname, "wgcobfq73pzmtmcttiy2knon5bm2a7gn6j6idaiccf53ikwrecdq.b32.i2p");
-		TEST_EQUAL(resp.peers[10].hostname, "ufunemgwuun5t2sn3oay4zv7jvwdezwcrirgwr6b2fjgczvaowvq.b32.i2p");
+		TEST_EQUAL(resp.peers[0].hostname
+			, "wgcobfq73pzmtmcttiy2knon5bm2a7gn6j6idaiccf53ikwrecdq.b32.i2p");
+		TEST_EQUAL(resp.peers[10].hostname
+			, "ufunemgwuun5t2sn3oay4zv7jvwdezwcrirgwr6b2fjgczvaowvq.b32.i2p");
 	}
 }
+#endif // TORRENT_USE_I2P
 
 TORRENT_TEST(parse_interval)
 {
 	char const response[] = "d8:intervali1042e12:min intervali10e5:peers0:e";
 	error_code ec;
-	tracker_response resp = parse_tracker_response(response, sizeof(response) - 1
+	tracker_response resp = parse_tracker_response(response
 		, ec, false, sha1_hash());
 
 	TEST_EQUAL(ec, error_code());
 	TEST_EQUAL(resp.peers.size(), 0);
 	TEST_EQUAL(resp.peers4.size(), 0);
-	TEST_EQUAL(resp.interval, 1042);
-	TEST_EQUAL(resp.min_interval, 10);
+	TEST_EQUAL(resp.interval.count(), 1042);
+	TEST_EQUAL(resp.min_interval.count(), 10);
 }
 
 TORRENT_TEST(parse_warning)
 {
 	char const response[] = "d5:peers0:15:warning message12:test messagee";
 	error_code ec;
-	tracker_response resp = parse_tracker_response(response, sizeof(response) - 1
+	tracker_response resp = parse_tracker_response(response
 		, ec, false, sha1_hash());
 
 	TEST_EQUAL(ec, error_code());
@@ -198,7 +205,7 @@ TORRENT_TEST(parse_failure_reason)
 {
 	char const response[] = "d5:peers0:14:failure reason12:test messagee";
 	error_code ec;
-	tracker_response resp = parse_tracker_response(response, sizeof(response) - 1
+	tracker_response resp = parse_tracker_response(response
 		, ec, false, sha1_hash());
 
 	TEST_EQUAL(ec, errors::tracker_failure);
@@ -208,9 +215,10 @@ TORRENT_TEST(parse_failure_reason)
 
 TORRENT_TEST(parse_scrape_response)
 {
-	char const response[] = "d5:filesd20:aaaaaaaaaaaaaaaaaaaad8:completei1e10:incompletei2e10:downloadedi3e11:downloadersi6eeee";
+	char const response[] = "d5:filesd20:aaaaaaaaaaaaaaaaaaaad"
+		"8:completei1e10:incompletei2e10:downloadedi3e11:downloadersi6eeee";
 	error_code ec;
-	tracker_response resp = parse_tracker_response(response, sizeof(response) - 1
+	tracker_response resp = parse_tracker_response(response
 		, ec, true, sha1_hash("aaaaaaaaaaaaaaaaaaaa"));
 
 	TEST_EQUAL(ec, error_code());
@@ -222,9 +230,10 @@ TORRENT_TEST(parse_scrape_response)
 
 TORRENT_TEST(parse_scrape_response_with_zero)
 {
-	char const response[] = "d5:filesd20:aaa\0aaaaaaaaaaaaaaaad8:completei4e10:incompletei5e10:downloadedi6eeee";
+	char const response[] = "d5:filesd20:aaa\0aaaaaaaaaaaaaaaad"
+		"8:completei4e10:incompletei5e10:downloadedi6eeee";
 	error_code ec;
-	tracker_response resp = parse_tracker_response(response, sizeof(response) - 1
+	tracker_response resp = parse_tracker_response(response
 		, ec, true, sha1_hash("aaa\0aaaaaaaaaaaaaaaa"));
 
 	TEST_EQUAL(ec, error_code());
@@ -238,41 +247,42 @@ TORRENT_TEST(parse_external_ip)
 {
 	char const response[] = "d5:peers0:11:external ip4:\x01\x02\x03\x04" "e";
 	error_code ec;
-	tracker_response resp = parse_tracker_response(response, sizeof(response) - 1
+	tracker_response resp = parse_tracker_response(response
 		, ec, false, sha1_hash());
 
 	TEST_EQUAL(ec, error_code());
 	TEST_EQUAL(resp.peers.size(), 0);
-	TEST_EQUAL(resp.external_ip, address_v4::from_string("1.2.3.4"));
+	TEST_EQUAL(resp.external_ip, addr4("1.2.3.4"));
 }
 
-#if TORRENT_USE_IPV6
 TORRENT_TEST(parse_external_ip6)
 {
-	char const response[] = "d5:peers0:11:external ip16:\xf1\x02\x03\x04\0\0\0\0\0\0\0\0\0\0\xff\xff" "e";
+	char const response[] = "d5:peers0:11:external ip"
+		"16:\xf1\x02\x03\x04\0\0\0\0\0\0\0\0\0\0\xff\xff" "e";
 	error_code ec;
-	tracker_response resp = parse_tracker_response(response, sizeof(response) - 1
+	tracker_response resp = parse_tracker_response(response
 		, ec, false, sha1_hash());
 
 	TEST_EQUAL(ec, error_code());
 	TEST_EQUAL(resp.peers.size(), 0);
-	TEST_EQUAL(resp.external_ip, address_v6::from_string("f102:0304::ffff"));
+	TEST_EQUAL(resp.external_ip, addr6("f102:0304::ffff"));
 }
-#endif
 
+namespace {
 peer_entry extract_peer(char const* peer_field, error_code expected_ec, bool expected_ret)
 {
 	error_code ec;
 	peer_entry result;
 	bdecode_node n;
 	bdecode(peer_field, peer_field + strlen(peer_field)
-		, n, ec, NULL, 1000, 1000);
+		, n, ec, nullptr, 1000, 1000);
 	TEST_CHECK(!ec);
 	bool ret = extract_peer_info(n, result, ec);
 	TEST_EQUAL(expected_ret, ret);
 	TEST_EQUAL(expected_ec, ec);
 	return result;
 }
+} // anonymous namespace
 
 TORRENT_TEST(extract_peer)
 {
@@ -288,7 +298,7 @@ TORRENT_TEST(extract_peer_hostname)
 	peer_entry result = extract_peer("d2:ip11:example.com4:porti1ee"
 		, error_code(), true);
 	TEST_EQUAL(result.hostname, "example.com");
-	TEST_EQUAL(result.pid, (peer_id::min)());
+	TEST_EQUAL(result.pid, peer_id::min());
 	TEST_EQUAL(result.port, 1);
 }
 
@@ -313,11 +323,13 @@ TORRENT_TEST(extract_peer_missing_port)
 		, errors::invalid_tracker_response, false);
 }
 
-bool connect_alert(libtorrent::alert const* a, tcp::endpoint& ep)
+namespace {
+
+bool connect_alert(lt::alert const* a, tcp::endpoint& ep)
 {
 	if (peer_connect_alert const* pc = alert_cast<peer_connect_alert>(a))
 	{
-		ep = pc->ip;
+		ep = pc->endpoint;
 		return true;
 	}
 	return false;
@@ -325,6 +337,8 @@ bool connect_alert(libtorrent::alert const* a, tcp::endpoint& ep)
 
 void test_udp_tracker(std::string const& iface, address tracker, tcp::endpoint const& expected_peer)
 {
+	using namespace std::placeholders;
+
 	int const udp_port = start_udp_tracker(tracker);
 
 	int prev_udp_announces = num_udp_announces();
@@ -332,25 +346,24 @@ void test_udp_tracker(std::string const& iface, address tracker, tcp::endpoint c
 	settings_pack pack = settings();
 	pack.set_bool(settings_pack::announce_to_all_trackers, true);
 	pack.set_bool(settings_pack::announce_to_all_tiers, true);
-	pack.set_str(settings_pack::listen_interfaces, iface + ":48875");
 
-	boost::scoped_ptr<lt::session> s(new lt::session(pack));
+	std::unique_ptr<lt::session> s(new lt::session(pack));
 
 	error_code ec;
 	remove_all("tmp1_tracker", ec);
 	create_directory("tmp1_tracker", ec);
 	std::ofstream file(combine_path("tmp1_tracker", "temporary").c_str());
-	boost::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
+	std::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
 	file.close();
 
 	char tracker_url[200];
-	snprintf(tracker_url, sizeof(tracker_url), "udp://%s:%d/announce", iface.c_str(), udp_port);
+	std::snprintf(tracker_url, sizeof(tracker_url), "udp://%s:%d/announce", iface.c_str(), udp_port);
 	t->add_tracker(tracker_url, 0);
 
 	add_torrent_params addp;
-	addp.flags &= ~add_torrent_params::flag_paused;
-	addp.flags &= ~add_torrent_params::flag_auto_managed;
-	addp.flags |= add_torrent_params::flag_seed_mode;
+	addp.flags &= ~torrent_flags::paused;
+	addp.flags &= ~torrent_flags::auto_managed;
+	addp.flags |= torrent_flags::seed_mode;
 	addp.ti = t;
 	addp.save_path = "tmp1_tracker";
 	torrent_handle h = s->add_torrent(addp);
@@ -358,12 +371,12 @@ void test_udp_tracker(std::string const& iface, address tracker, tcp::endpoint c
 	tcp::endpoint peer_ep;
 	for (int i = 0; i < 50; ++i)
 	{
-		print_alerts(*s, "s", false, false, false, boost::bind(&connect_alert, _1, boost::ref(peer_ep)));
+		print_alerts(*s, "s", false, false, std::bind(&connect_alert, _1, std::ref(peer_ep)));
 
 		if (num_udp_announces() == prev_udp_announces + 1)
 			break;
 
-		test_sleep(100);
+		std::this_thread::sleep_for(lt::milliseconds(100));
 	}
 
 	// we should have announced to the tracker by now
@@ -371,47 +384,59 @@ void test_udp_tracker(std::string const& iface, address tracker, tcp::endpoint c
 
 	// if we remove the torrent before it has received the response from the
 	// tracker, it won't announce again to stop. So, wait a bit before removing.
-	test_sleep(1000);
+	std::this_thread::sleep_for(lt::milliseconds(1000));
 
 	s->remove_torrent(h);
 
 	for (int i = 0; i < 50; ++i)
 	{
-		print_alerts(*s, "s", true, true, false, boost::bind(&connect_alert, _1, boost::ref(peer_ep)));
+		print_alerts(*s, "s", true, false, std::bind(&connect_alert, _1, std::ref(peer_ep)));
 		if (num_udp_announces() == prev_udp_announces + 2)
 			break;
 
-		test_sleep(100);
+		std::this_thread::sleep_for(lt::milliseconds(100));
 	}
 
+	std::printf("peer_ep: %s expected: %s\n"
+		, lt::print_endpoint(peer_ep).c_str()
+		, lt::print_endpoint(expected_peer).c_str());
 	TEST_CHECK(peer_ep == expected_peer);
+	std::printf("destructing session\n");
 
-	fprintf(stderr, "destructing session\n");
 	s.reset();
-	fprintf(stderr, "done\n");
+	std::printf("done\n");
 
 	// we should have announced the stopped event now
 	TEST_EQUAL(num_udp_announces(), prev_udp_announces + 2);
+
+	stop_udp_tracker();
 }
 
+} // anonymous namespace
+
 TORRENT_TEST(udp_tracker_v4)
 {
-	test_udp_tracker("127.0.0.1", address_v4::any(), ep("1.3.3.7", 1337));
+	// if the machine running the test doesn't have an actual IPv4 connection
+	// the test would fail with any other address than loopback (because it
+	// would be unreachable). This is true for some CI's, running containers
+	// without an internet connection
+	test_udp_tracker("127.0.0.1", address_v4::any(), ep("127.0.0.2", 1337));
 }
 
-#if TORRENT_USE_IPV6
 TORRENT_TEST(udp_tracker_v6)
 {
 	if (supports_ipv6())
 	{
-		test_udp_tracker("[::1]", address_v6::any(), ep("::1.3.3.7", 1337));
+		// if the machine running the test doesn't have an actual IPv6 connection
+		// the test would fail with any other address than loopback (because it
+		// would be unreachable)
+		test_udp_tracker("[::1]", address_v6::any(), ep("::1", 1337));
 	}
 }
-#endif
 
 TORRENT_TEST(http_peers)
 {
-	int http_port = start_web_server();
+	int const http_port = start_web_server();
 
 	settings_pack pack = settings();
 	pack.set_bool(settings_pack::announce_to_all_trackers, true);
@@ -420,30 +445,29 @@ TORRENT_TEST(http_peers)
 	pack.set_int(settings_pack::tracker_receive_timeout, 1);
 	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:39775");
 
-	boost::scoped_ptr<lt::session> s(new lt::session(pack));
+	std::unique_ptr<lt::session> s(new lt::session(pack));
 
 	error_code ec;
 	remove_all("tmp2_tracker", ec);
 	create_directory("tmp2_tracker", ec);
 	std::ofstream file(combine_path("tmp2_tracker", "temporary").c_str());
-	boost::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
+	std::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
 	file.close();
 
 	char tracker_url[200];
-	// and this should not be announced to (since the one before it succeeded)
-	snprintf(tracker_url, sizeof(tracker_url), "http://127.0.0.1:%d/announce"
+	std::snprintf(tracker_url, sizeof(tracker_url), "http://127.0.0.1:%d/announce"
 		, http_port);
 	t->add_tracker(tracker_url, 0);
 
 	add_torrent_params addp;
-	addp.flags &= ~add_torrent_params::flag_paused;
-	addp.flags &= ~add_torrent_params::flag_auto_managed;
-	addp.flags |= add_torrent_params::flag_seed_mode;
+	addp.flags &= ~torrent_flags::paused;
+	addp.flags &= ~torrent_flags::auto_managed;
+	addp.flags |= torrent_flags::seed_mode;
 	addp.ti = t;
 	addp.save_path = "tmp2_tracker";
 	torrent_handle h = s->add_torrent(addp);
 
-	libtorrent::torrent_status status = h.status();
+	lt::torrent_status status = h.status();
 	TEST_CHECK(status.current_tracker.empty());
 
 	// wait to hit the tracker
@@ -451,34 +475,35 @@ TORRENT_TEST(http_peers)
 
 	status = h.status();
 	TEST_CHECK(!status.current_tracker.empty());
-	TEST_CHECK(status.current_tracker == tracker_url);
+	TEST_EQUAL(status.current_tracker, tracker_url);
 
 	// we expect to have certain peers in our peer list now
 	// these peers are hard coded in web_server.py
-	std::vector<peer_list_entry> peers;
-	h.get_full_peer_list(peers);
+	h.save_resume_data();
+	alert const* a = wait_for_alert(*s, save_resume_data_alert::alert_type);
 
-	std::set<tcp::endpoint> expected_peers;
-	expected_peers.insert(tcp::endpoint(address_v4::from_string("65.65.65.65"), 16962));
-	expected_peers.insert(tcp::endpoint(address_v4::from_string("67.67.67.67"), 17476));
-#if TORRENT_USE_IPV6
-	expected_peers.insert(tcp::endpoint(address_v6::from_string("4545:4545:4545:4545:4545:4545:4545:4545"), 17990));
-#endif
-
-	TEST_EQUAL(peers.size(), expected_peers.size());
-	for (std::vector<peer_list_entry>::iterator i = peers.begin()
-		, end(peers.end()); i != end; ++i)
+	TEST_CHECK(a);
+	save_resume_data_alert const* ra = alert_cast<save_resume_data_alert>(a);
+	TEST_CHECK(ra);
+	if (ra)
 	{
-		TEST_EQUAL(expected_peers.count(i->ip), 1);
+		std::set<tcp::endpoint> expected_peers;
+		expected_peers.insert(ep("65.65.65.65", 16962));
+		expected_peers.insert(ep("67.67.67.67", 17476));
+		expected_peers.insert(ep("4545:4545:4545:4545:4545:4545:4545:4545", 17990));
+		for (auto const& ip : ra->params.peers)
+		{
+			TEST_EQUAL(expected_peers.count(ip), 1);
+		}
 	}
 
-	fprintf(stderr, "destructing session\n");
+	std::printf("destructing session\n");
 	s.reset();
-	fprintf(stderr, "done\n");
+	std::printf("done\n");
 
-	fprintf(stderr, "stop_web_server\n");
+	std::printf("stop_web_server\n");
 	stop_web_server();
-	fprintf(stderr, "done\n");
+	std::printf("done\n");
 }
 
 TORRENT_TEST(current_tracker)
@@ -493,29 +518,29 @@ TORRENT_TEST(current_tracker)
 	pack.set_int(settings_pack::tracker_receive_timeout, 1);
 	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:39775");
 
-	boost::scoped_ptr<lt::session> s(new lt::session(pack));
+	std::unique_ptr<lt::session> s(new lt::session(pack));
 
 	error_code ec;
 	remove_all("tmp3_tracker", ec);
 	create_directory("tmp3_tracker", ec);
 	std::ofstream file(combine_path("tmp3_tracker", "temporary").c_str());
-	boost::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
+	std::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
 	file.close();
 
 	char tracker_url[200];
-	snprintf(tracker_url, sizeof(tracker_url), "http://127.0.0.1:%d/announce"
+	std::snprintf(tracker_url, sizeof(tracker_url), "http://127.0.0.1:%d/announce"
 		, http_port);
 	t->add_tracker(tracker_url, 0);
 
 	add_torrent_params addp;
-	addp.flags &= ~add_torrent_params::flag_paused;
-	addp.flags &= ~add_torrent_params::flag_auto_managed;
-	addp.flags |= add_torrent_params::flag_seed_mode;
+	addp.flags &= ~torrent_flags::paused;
+	addp.flags &= ~torrent_flags::auto_managed;
+	addp.flags |= torrent_flags::seed_mode;
 	addp.ti = t;
 	addp.save_path = "tmp3_tracker";
 	torrent_handle h = s->add_torrent(addp);
 
-	libtorrent::torrent_status status = h.status();
+	lt::torrent_status status = h.status();
 	TEST_CHECK(status.current_tracker.empty());
 
 	// wait to hit the tracker announce
@@ -530,11 +555,13 @@ TORRENT_TEST(current_tracker)
 	status = h.status();
 	TEST_CHECK(status.current_tracker.empty());
 
-	fprintf(stderr, "destructing session\n");
+	std::printf("destructing session\n");
 	s.reset();
-	fprintf(stderr, "done\n");
+	std::printf("done\n");
 }
 
+namespace {
+
 void test_proxy(bool proxy_trackers)
 {
 	int http_port = start_web_server();
@@ -545,32 +572,31 @@ void test_proxy(bool proxy_trackers)
 	pack.set_int(settings_pack::tracker_completion_timeout, 2);
 	pack.set_int(settings_pack::tracker_receive_timeout, 1);
 	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:39775");
-	pack.set_bool(settings_pack::force_proxy, true);
 
 	pack.set_str(settings_pack::proxy_hostname, "non-existing.com");
 	pack.set_int(settings_pack::proxy_type, settings_pack::socks5);
 	pack.set_int(settings_pack::proxy_port, 4444);
 	pack.set_bool(settings_pack::proxy_tracker_connections, proxy_trackers);
 
-	boost::scoped_ptr<lt::session> s(new lt::session(pack));
+	std::unique_ptr<lt::session> s(new lt::session(pack));
 
 	error_code ec;
 	remove_all("tmp2_tracker", ec);
 	create_directory("tmp2_tracker", ec);
 	std::ofstream file(combine_path("tmp2_tracker", "temporary").c_str());
-	boost::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
+	std::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
 	file.close();
 
 	char tracker_url[200];
 	// and this should not be announced to (since the one before it succeeded)
-	snprintf(tracker_url, sizeof(tracker_url), "http://127.0.0.1:%d/announce"
+	std::snprintf(tracker_url, sizeof(tracker_url), "http://127.0.0.1:%d/announce"
 		, http_port);
 	t->add_tracker(tracker_url, 0);
 
 	add_torrent_params addp;
-	addp.flags &= ~add_torrent_params::flag_paused;
-	addp.flags &= ~add_torrent_params::flag_auto_managed;
-	addp.flags |= add_torrent_params::flag_seed_mode;
+	addp.flags &= ~torrent_flags::paused;
+	addp.flags &= ~torrent_flags::auto_managed;
+	addp.flags |= torrent_flags::seed_mode;
 	addp.ti = t;
 	addp.save_path = "tmp2_tracker";
 	torrent_handle h = s->add_torrent(addp);
@@ -579,76 +605,77 @@ void test_proxy(bool proxy_trackers)
 	const alert* a = wait_for_alert(*s, tracker_reply_alert::alert_type, "s");
 	if (proxy_trackers)
 	{
-		TEST_CHECK(a == NULL);
+		TEST_CHECK(a == nullptr);
 	}
 	else
 	{
-		TEST_CHECK(a != NULL);
+		TEST_CHECK(a != nullptr);
 	}
 
-	fprintf(stderr, "destructing session\n");
+	std::printf("destructing session\n");
 	s.reset();
-	fprintf(stderr, "done\n");
+	std::printf("done\n");
 
-	fprintf(stderr, "stop_web_server\n");
+	std::printf("stop_web_server\n");
 	stop_web_server();
-	fprintf(stderr, "done\n");
+	std::printf("done\n");
 }
 
+} // anonymous namespace
+
 TORRENT_TEST(tracker_proxy)
 {
-	fprintf(stderr, "\n\nnot proxying tracker connections (expect to reach the tracker)\n\n");
+	std::printf("\n\nnot proxying tracker connections (expect to reach the tracker)\n\n");
 	test_proxy(false);
 
-	fprintf(stderr, "\n\nproxying tracker connections through non-existent proxy (do not expect to reach the tracker)\n\n");
+	std::printf("\n\nproxying tracker connections through non-existent proxy "
+		"(do not expect to reach the tracker)\n\n");
 	test_proxy(true);
 }
 
 #ifndef TORRENT_DISABLE_LOGGING
-
-int count_stopped_events(session& ses, int expected = 0)
-{
-	int count = 0;
-	int num = 70; // this number is adjusted per version, an estimate
-	time_point const end_time = clock_type::now() + seconds(15);
-	while (true)
-	{
-		time_point const now = clock_type::now();
-		if (now > end_time) return count;
-
-		ses.wait_for_alert(end_time - now);
-		std::vector<alert*> alerts;
-		ses.pop_alerts(&alerts);
-		for (int i = 0; i < int(alerts.size()); ++i)
-		{
-			alert* a = alerts[i];
-			std::printf("%d: [%s] %s\n", num, a->what(), a->message().c_str());
-			if (a->type() == log_alert::alert_type)
-			{
-				std::string const msg = a->message();
-				if (msg.find("&event=stopped") != std::string::npos)
-				{
-					count++;
-					--expected;
-				}
-			}
-			num--;
-		}
-		if (num <= 0 && expected <= 0) return count;
-	}
-	return count;
-};
-
+namespace {
 void test_stop_tracker_timeout(int const timeout)
 {
 	// trick the min interval so that the stopped anounce is permitted immediately
 	// after the initial announce
-	int port = start_web_server(false, false, true);
+	int port = start_web_server(false, false, true, -1);
+
+	auto count_stopped_events = [](session& ses, int expected)
+	{
+		int count = 0;
+		int num = 70; // this number is adjusted per version, an estimate
+		time_point const end_time = clock_type::now() + seconds(15);
+		while (true)
+		{
+			time_point const now = clock_type::now();
+			if (now > end_time) return count;
+
+			ses.wait_for_alert(end_time - now);
+			std::vector<alert*> alerts;
+			ses.pop_alerts(&alerts);
+			for (auto a : alerts)
+			{
+				std::printf("%d: [%s] %s\n", num, a->what(), a->message().c_str());
+				if (a->type() == log_alert::alert_type)
+				{
+					std::string const msg = a->message();
+					if (msg.find("&event=stopped") != std::string::npos)
+					{
+						count++;
+						--expected;
+					}
+				}
+				num--;
+			}
+			if (num <= 0 && expected <= 0) return count;
+		}
+	};
 
 	settings_pack p = settings();
 	p.set_bool(settings_pack::announce_to_all_trackers, true);
 	p.set_bool(settings_pack::announce_to_all_tiers, true);
-	p.set_str(settings_pack::listen_interfaces, "0.0.0.0:6881");
+	p.set_str(settings_pack::listen_interfaces, "127.0.0.1:6881");
 	p.set_int(settings_pack::stop_tracker_timeout, timeout);
 
 	lt::session s(p);
@@ -657,20 +684,20 @@ void test_stop_tracker_timeout(int const timeout)
 	remove_all("tmp4_tracker", ec);
 	create_directory("tmp4_tracker", ec);
 	std::ofstream file(combine_path("tmp4_tracker", "temporary").c_str());
-	boost::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
+	std::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
 	file.close();
 
 	add_torrent_params tp;
-	tp.flags &= ~add_torrent_params::flag_paused;
-	tp.flags &= ~add_torrent_params::flag_auto_managed;
-	tp.flags |= add_torrent_params::flag_seed_mode;
+	tp.flags &= ~torrent_flags::paused;
+	tp.flags &= ~torrent_flags::auto_managed;
+	tp.flags |= torrent_flags::seed_mode;
 	tp.ti = t;
 	tp.save_path = "tmp4_tracker";
 	torrent_handle h = s.add_torrent(tp);
 
 	char tracker_url[200];
-	snprintf(tracker_url, sizeof(tracker_url), "http://127.0.0.1:%d/announce", port);
-	announce_entry ae(tracker_url);
+	std::snprintf(tracker_url, sizeof(tracker_url), "http://127.0.0.1:%d/announce", port);
+	announce_entry ae{tracker_url};
 	h.add_tracker(ae);
 
 	// make sure it announced a event=started properly
@@ -678,13 +705,20 @@ void test_stop_tracker_timeout(int const timeout)
 
 	s.remove_torrent(h);
 
-	int const count = count_stopped_events(s, 1);
-	TEST_EQUAL(count, 1);
+	int const count = count_stopped_events(s, (timeout == 0) ? 0 : 1);
+	TEST_EQUAL(count, (timeout == 0) ? 0 : 1);
 }
+} // anonymous namespace
 
 TORRENT_TEST(stop_tracker_timeout)
 {
 	std::printf("\n\nexpect to get ONE request with &event=stopped\n\n");
 	test_stop_tracker_timeout(1);
 }
+
+TORRENT_TEST(stop_tracker_timeout_zero_timeout)
+{
+	std::printf("\n\nexpect to NOT get a request with &event=stopped\n\n");
+	test_stop_tracker_timeout(0);
+}
 #endif
