@@ -34,42 +34,43 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "setup_transfer.hpp"
 #include "web_seed_suite.hpp"
 #include "settings.hpp"
+#include "libtorrent/random.hpp"
 #include "libtorrent/create_torrent.hpp"
 #include "libtorrent/torrent_info.hpp"
 
-using namespace libtorrent;
+using namespace lt;
 
 TORRENT_TEST(web_seed_redirect)
 {
-	using namespace libtorrent;
+	using namespace lt;
 
 	error_code ec;
 
 	file_storage fs;
 	int piece_size = 0x4000;
 
-	char random_data[16000];
-	std::generate(random_data, random_data + sizeof(random_data), random_byte);
-	file f("test_file", file::write_only, ec);
+	std::array<char, 16000> random_data;
+	aux::random_bytes(random_data);
+	file f("test_file", open_mode::write_only, ec);
 	if (ec)
 	{
-		fprintf(stderr, "failed to create file \"test_file\": (%d) %s\n"
+		std::printf("failed to create file \"test_file\": (%d) %s\n"
 			, ec.value(), ec.message().c_str());
 		TEST_ERROR("failed to create file");
 		return;
 	}
-	file::iovec_t b = { random_data, size_t(16000)};
-	f.writev(0, &b, 1, ec);
+	iovec_t b = random_data;
+	f.writev(0, b, ec);
 	fs.add_file("test_file", 16000);
 
 	int port = start_web_server();
 
 	// generate a torrent with pad files to make sure they
 	// are not requested web seeds
-	libtorrent::create_torrent t(fs, piece_size, 0x4000);
+	lt::create_torrent t(fs, piece_size, 0x4000);
 
 	char tmp[512];
-	snprintf(tmp, sizeof(tmp), "http://127.0.0.1:%d/redirect", port);
+	std::snprintf(tmp, sizeof(tmp), "http://127.0.0.1:%d/redirect", port);
 	t.add_url_seed(tmp);
 
 	// calculate the hash for all pieces
@@ -77,7 +78,7 @@ TORRENT_TEST(web_seed_redirect)
 
 	if (ec)
 	{
-		fprintf(stderr, "error creating hashes for test torrent: %s\n"
+		std::printf("error creating hashes for test torrent: %s\n"
 			, ec.message().c_str());
 		TEST_ERROR("failed to create hashes");
 		return;
@@ -85,20 +86,17 @@ TORRENT_TEST(web_seed_redirect)
 
 	std::vector<char> buf;
 	bencode(std::back_inserter(buf), t.generate());
-	boost::shared_ptr<torrent_info> torrent_file(new torrent_info(&buf[0]
-		, buf.size(), ec));
+	auto torrent_file = std::make_shared<torrent_info>(buf, ec, from_span);
 
 	{
 		settings_pack p = settings();
 		p.set_int(settings_pack::max_queued_disk_bytes, 256 * 1024);
-		libtorrent::session ses(p);
+		lt::session ses(p);
 
 		// disable keep-alive because otherwise the test will choke on seeing
 		// the disconnect (from the redirect)
-		test_transfer(ses, torrent_file, 0, 0, "http", true, false, false, false);
+		test_transfer(ses, torrent_file, 0, "http", true, false, false, false);
 	}
 
 	stop_web_server();
 }
-
-
