@@ -30,7 +30,6 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
-#include "libtorrent/thread.hpp"
 #include "libtorrent/bencode.hpp"
 #include "libtorrent/entry.hpp"
 #include "libtorrent/address.hpp"
@@ -40,41 +39,41 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/socket_io.hpp"
 #include "libtorrent/io.hpp"
 #include "libtorrent/aux_/time.hpp"
+#include "libtorrent/broadcast_socket.hpp" // for is_v6
 #include "udp_tracker.hpp"
 #include "test_utils.hpp"
 
-#include <boost/detail/atomic_count.hpp>
-#include <boost/shared_ptr.hpp>
-#include <boost/bind.hpp>
+#include <functional>
+#include <thread>
+#include <atomic>
+#include <memory>
 
-#if defined TORRENT_DEBUG && TORRENT_USE_IOSTREAM
-#include <iostream>
-#endif
-
-using namespace libtorrent;
+using namespace lt;
+using namespace std::placeholders;
 
 struct udp_tracker
 {
 
-	libtorrent::io_service m_ios;
-	boost::detail::atomic_count m_udp_announces;
-	udp::socket m_socket;
-	int m_port;
-	bool m_abort;
+	lt::io_service m_ios;
+	std::atomic<int> m_udp_announces{0};
+	udp::socket m_socket{m_ios};
+	int m_port = 0;
+	bool m_abort = false;
 
-	boost::shared_ptr<libtorrent::thread> m_thread;
+	std::shared_ptr<std::thread> m_thread;
 
-	void on_udp_receive(error_code const& ec, size_t bytes_transferred, udp::endpoint* from, char* buffer, int size)
+	void on_udp_receive(error_code const& ec, size_t const bytes_transferred
+		, udp::endpoint* from, char* buffer, std::size_t const size)
 	{
 		if (ec)
 		{
-			fprintf(stderr, "%s: UDP tracker, read failed: %s\n", time_now_string(), ec.message().c_str());
+			std::printf("%s: UDP tracker, read failed: %s\n", time_now_string(), ec.message().c_str());
 			return;
 		}
 
 		if (bytes_transferred < 16)
 		{
-			fprintf(stderr, "%s: UDP message too short (from: %s)\n", time_now_string(), print_endpoint(*from).c_str());
+			std::printf("%s: UDP message too short (from: %s)\n", time_now_string(), print_endpoint(*from).c_str());
 			return;
 		}
 
@@ -83,12 +82,12 @@ struct udp_tracker
 			return;
 		}
 
-		fprintf(stderr, "%s: UDP message %d bytes\n", time_now_string(), int(bytes_transferred));
+		std::printf("%s: UDP message %d bytes\n", time_now_string(), int(bytes_transferred));
 
 		char* ptr = buffer;
 		detail::read_uint64(ptr);
-		boost::uint32_t const action = detail::read_uint32(ptr);
-		boost::uint32_t const transaction_id = detail::read_uint32(ptr);
+		std::uint32_t const action = detail::read_uint32(ptr);
+		std::uint32_t const transaction_id = detail::read_uint32(ptr);
 
 		error_code e;
 
@@ -96,23 +95,36 @@ struct udp_tracker
 		{
 			case 0: // connect
 
-				fprintf(stderr, "%s: UDP connect from %s\n", time_now_string()
+				if (bytes_transferred < 16)
+				{
+					std::printf("invalid connect message: %d Bytes, expected 16 Bytes\n"
+						, int(bytes_transferred));
+					return;
+				}
+				std::printf("%s: UDP connect from %s\n", time_now_string()
 					, print_endpoint(*from).c_str());
 				ptr = buffer;
 				detail::write_uint32(0, ptr); // action = connect
 				detail::write_uint32(transaction_id, ptr); // transaction_id
 				detail::write_uint64(10, ptr); // connection_id
 				m_socket.send_to(boost::asio::buffer(buffer, 16), *from, 0, e);
-				if (e) fprintf(stderr, "%s: UDP send_to failed. ERROR: %s\n"
+				if (e) std::printf("%s: UDP send_to failed. ERROR: %s\n"
 					, time_now_string(), e.message().c_str());
-				else fprintf(stderr, "%s: UDP sent response to: %s\n"
+				else std::printf("%s: UDP sent response to: %s\n"
 					, time_now_string(), print_endpoint(*from).c_str());
 				break;
 
 			case 1: // announce
 
+				if (bytes_transferred < 84)
+				{
+					std::printf("invalid announce message: %d Bytes, expected 84 Bytes\n"
+						, int(bytes_transferred));
+					return;
+				}
+
 				++m_udp_announces;
-				fprintf(stderr, "%s: UDP announce [%d]\n", time_now_string()
+				std::printf("%s: UDP announce [%d]\n", time_now_string()
 					, int(m_udp_announces));
 				ptr = buffer;
 				detail::write_uint32(1, ptr); // action = announce
@@ -121,83 +133,80 @@ struct udp_tracker
 				detail::write_uint32(1, ptr); // incomplete
 				detail::write_uint32(1, ptr); // complete
 				// 1 peers
-#if TORRENT_USE_IPV6
-				if (from->address().is_v6())
+				if (is_v6(*from))
 				{
 					detail::write_uint32(0, ptr);
 					detail::write_uint32(0, ptr);
 					detail::write_uint32(0, ptr);
+					detail::write_uint8(0, ptr);
+					detail::write_uint8(0, ptr);
+					detail::write_uint8(0, ptr);
 					detail::write_uint8(1, ptr);
-					detail::write_uint8(3, ptr);
-					detail::write_uint8(3, ptr);
-					detail::write_uint8(7, ptr);
 					detail::write_uint16(1337, ptr);
 				}
 				else
-#endif
 				{
-					detail::write_uint8(1, ptr);
-					detail::write_uint8(3, ptr);
-					detail::write_uint8(3, ptr);
-					detail::write_uint8(7, ptr);
+					detail::write_uint8(127, ptr);
+					detail::write_uint8(0, ptr);
+					detail::write_uint8(0, ptr);
+					detail::write_uint8(2, ptr);
 					detail::write_uint16(1337, ptr);
 				}
-				m_socket.send_to(boost::asio::buffer(buffer, ptr - buffer), *from, 0, e);
+				m_socket.send_to(boost::asio::buffer(buffer
+					, static_cast<std::size_t>(ptr - buffer)), *from, 0, e);
 				if (e) std::printf("%s: UDP send_to failed. ERROR: %s\n"
 					, time_now_string(), e.message().c_str());
-				else fprintf(stderr, "%s: UDP sent response to: %s\n"
+				else std::printf("%s: UDP sent response to: %s\n"
 					, time_now_string(), print_endpoint(*from).c_str());
 				break;
 			case 2:
 				// ignore scrapes
-				fprintf(stderr, "%s: UDP scrape (ignored)\n", time_now_string());
+				std::printf("%s: UDP scrape (ignored)\n", time_now_string());
 				break;
 			default:
-				fprintf(stderr, "%s: UDP unknown message: %d\n", time_now_string()
+				std::printf("%s: UDP unknown message: %d\n", time_now_string()
 					, action);
 				break;
 		}
 
 		m_socket.async_receive_from(
 			boost::asio::buffer(buffer, size), *from, 0
-			, boost::bind(&udp_tracker::on_udp_receive, this, _1, _2, from, buffer, size));
+			, std::bind(&udp_tracker::on_udp_receive, this, _1, _2, from, buffer, size));
 	}
 
-	udp_tracker(address iface)
-		: m_udp_announces(0)
-		, m_socket(m_ios)
-		, m_port(0)
-		, m_abort(false)
+	explicit udp_tracker(address iface)
 	{
 		error_code ec;
 		m_socket.open(iface.is_v4() ? udp::v4() : udp::v6(), ec);
 		if (ec)
 		{
-			fprintf(stderr, "UDP Error opening listen UDP tracker socket: %s\n", ec.message().c_str());
+			std::printf("UDP Error opening listen UDP tracker socket: %s\n", ec.message().c_str());
 			return;
 		}
 
 		m_socket.bind(udp::endpoint(iface, 0), ec);
 		if (ec)
 		{
-			fprintf(stderr, "UDP Error binding UDP tracker socket to port 0: %s\n", ec.message().c_str());
+			std::printf("UDP Error binding UDP tracker socket to port 0: %s\n", ec.message().c_str());
 			return;
 		}
 		m_port = m_socket.local_endpoint(ec).port();
 		if (ec)
 		{
-			fprintf(stderr, "UDP Error getting local endpoint of UDP tracker socket: %s\n", ec.message().c_str());
+			std::printf("UDP Error getting local endpoint of UDP tracker socket: %s\n", ec.message().c_str());
 			return;
 		}
 
-		fprintf(stderr, "%s: UDP tracker initialized on port %d\n", time_now_string(), m_port);
+		std::printf("%s: UDP tracker [%p] initialized on port %d\n"
+			, time_now_string(), static_cast<void*>(this), m_port);
 
-		m_thread.reset(new libtorrent::thread(boost::bind(&udp_tracker::thread_fun, this)));
+		m_thread = std::make_shared<std::thread>(&udp_tracker::thread_fun, this);
 	}
 
 	void stop()
 	{
-		std::printf("%s: UDP tracker, stop\n", time_now_string());
+		std::printf("%s: UDP tracker [%p], stop\n", time_now_string()
+			, static_cast<void*>(this));
 		m_abort = true;
 		m_socket.cancel();
 		m_socket.close();
@@ -205,7 +214,9 @@ struct udp_tracker
 
 	~udp_tracker()
 	{
-		m_ios.post(boost::bind(&udp_tracker::stop, this));
+		std::printf("%s: UDP tracker [%p], ~udp_tracker\n"
+			, time_now_string(), static_cast<void*>(this));
+		m_ios.post(std::bind(&udp_tracker::stop, this));
 		if (m_thread) m_thread->join();
 	}
 
@@ -213,13 +224,6 @@ struct udp_tracker
 
 	int num_hits() const { return m_udp_announces; }
 
-	static void incoming_packet(error_code const& ec, size_t bytes_transferred, size_t *ret, error_code* error, bool* done)
-	{
-		*ret = bytes_transferred;
-		*error = ec;
-		*done = true;
-	}
-
 	void thread_fun()
 	{
 		char buffer[2000];
@@ -227,25 +231,28 @@ struct udp_tracker
 		error_code ec;
 		udp::endpoint from;
 		m_socket.async_receive_from(
-			boost::asio::buffer(buffer, sizeof(buffer)), from, 0
-			, boost::bind(&udp_tracker::on_udp_receive, this, _1, _2, &from, &buffer[0], sizeof(buffer)));
+			boost::asio::buffer(buffer, int(sizeof(buffer))), from, 0
+			, std::bind(&udp_tracker::on_udp_receive, this, _1, _2, &from, &buffer[0], int(sizeof(buffer))));
 
 		m_ios.run(ec);
 
 		if (ec)
 		{
-			fprintf(stderr, "UDP Error running UDP tracker service: %s\n", ec.message().c_str());
+			std::printf("UDP Error running UDP tracker service: %s\n", ec.message().c_str());
 			return;
 		}
 
-		fprintf(stderr, "UDP exiting UDP tracker thread\n");
+		std::printf("UDP exiting UDP tracker [%p] thread\n", static_cast<void*>(this));
 	}
 };
 
-boost::shared_ptr<udp_tracker> g_udp_tracker;
+namespace {
+std::shared_ptr<udp_tracker> g_udp_tracker;
+}
 
 int start_udp_tracker(address iface)
 {
+	TORRENT_ASSERT(!g_udp_tracker);
 	g_udp_tracker.reset(new udp_tracker(iface));
 	return g_udp_tracker->port();
 }
